<?php

namespace ScopedGtbabel\Matrix\Operators;

use ScopedGtbabel\Matrix\Matrix;
use ScopedGtbabel\Matrix\Builder;
use ScopedGtbabel\Matrix\Exception;
use Throwable;
class Multiplication extends \ScopedGtbabel\Matrix\Operators\Operator
{
    /**
     * Execute the multiplication
     *
     * @param mixed $value The matrix or numeric value to multiply the current base value by
     * @throws Exception If the provided argument is not appropriate for the operation
     * @return $this The operation object, allowing multiple multiplications to be chained
     **/
    public function execute($value, string $type = 'multiplication') : \ScopedGtbabel\Matrix\Operators\Operator
    {
        if (\is_array($value)) {
            $value = new \ScopedGtbabel\Matrix\Matrix($value);
        }
        if (\is_object($value) && $value instanceof \ScopedGtbabel\Matrix\Matrix) {
            return $this->multiplyMatrix($value, $type);
        } elseif (\is_numeric($value)) {
            return $this->multiplyScalar($value, $type);
        }
        throw new \ScopedGtbabel\Matrix\Exception("Invalid argument for {$type}");
    }
    /**
     * Execute the multiplication for a scalar
     *
     * @param mixed $value The numeric value to multiply with the current base value
     * @return $this The operation object, allowing multiple mutiplications to be chained
     **/
    protected function multiplyScalar($value, string $type = 'multiplication') : \ScopedGtbabel\Matrix\Operators\Operator
    {
        try {
            for ($row = 0; $row < $this->rows; ++$row) {
                for ($column = 0; $column < $this->columns; ++$column) {
                    $this->matrix[$row][$column] *= $value;
                }
            }
        } catch (\Throwable $e) {
            throw new \ScopedGtbabel\Matrix\Exception("Invalid argument for {$type}");
        }
        return $this;
    }
    /**
     * Execute the multiplication for a matrix
     *
     * @param Matrix $value The numeric value to multiply with the current base value
     * @return $this The operation object, allowing multiple mutiplications to be chained
     * @throws Exception If the provided argument is not appropriate for the operation
     **/
    protected function multiplyMatrix(\ScopedGtbabel\Matrix\Matrix $value, string $type = 'multiplication') : \ScopedGtbabel\Matrix\Operators\Operator
    {
        $this->validateReflectingDimensions($value);
        $newRows = $this->rows;
        $newColumns = $value->columns;
        $matrix = \ScopedGtbabel\Matrix\Builder::createFilledMatrix(0, $newRows, $newColumns)->toArray();
        try {
            for ($row = 0; $row < $newRows; ++$row) {
                for ($column = 0; $column < $newColumns; ++$column) {
                    $columnData = $value->getColumns($column + 1)->toArray();
                    foreach ($this->matrix[$row] as $key => $valueData) {
                        $matrix[$row][$column] += $valueData * $columnData[$key][0];
                    }
                }
            }
        } catch (\Throwable $e) {
            throw new \ScopedGtbabel\Matrix\Exception("Invalid argument for {$type}");
        }
        $this->matrix = $matrix;
        return $this;
    }
}
