<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel;

use DateTime;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date as SharedDateHelper;
class Week
{
    /**
     * WEEKNUM.
     *
     * Returns the week of the year for a specified date.
     * The WEEKNUM function considers the week containing January 1 to be the first week of the year.
     * However, there is a European standard that defines the first week as the one with the majority
     * of days (four or more) falling in the new year. This means that for years in which there are
     * three days or less in the first week of January, the WEEKNUM function returns week numbers
     * that are incorrect according to the European standard.
     *
     * Excel Function:
     *        WEEKNUM(dateValue[,style])
     *
     * @param mixed $dateValue Excel date serial value (float), PHP date timestamp (integer),
     *                                    PHP DateTime object, or a standard date string
     * @param int $method Week begins on Sunday or Monday
     *                                        1 or omitted    Week begins on Sunday.
     *                                        2                Week begins on Monday.
     *                                        11               Week begins on Monday.
     *                                        12               Week begins on Tuesday.
     *                                        13               Week begins on Wednesday.
     *                                        14               Week begins on Thursday.
     *                                        15               Week begins on Friday.
     *                                        16               Week begins on Saturday.
     *                                        17               Week begins on Sunday.
     *                                        21               ISO (Jan. 4 is week 1, begins on Monday).
     *
     * @return int|string Week Number
     */
    public static function number($dateValue, $method = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Constants::STARTWEEK_SUNDAY)
    {
        $origDateValueNull = empty($dateValue);
        try {
            $method = self::validateMethod($method);
            if ($dateValue === null) {
                // boolean not allowed
                $dateValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::getExcelCalendar() === \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::CALENDAR_MAC_1904 || $method === \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Constants::DOW_SUNDAY ? 0 : 1;
            }
            $dateValue = self::validateDateValue($dateValue);
            if (!$dateValue && self::buggyWeekNum1900($method)) {
                // This seems to be an additional Excel bug.
                return 0;
            }
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Execute function
        $PHPDateObject = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($dateValue);
        if ($method == \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Constants::STARTWEEK_MONDAY_ISO) {
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::silly1900($PHPDateObject);
            return (int) $PHPDateObject->format('W');
        }
        if (self::buggyWeekNum1904($method, $origDateValueNull, $PHPDateObject)) {
            return 0;
        }
        \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::silly1900($PHPDateObject, '+ 5 years');
        // 1905 calendar matches
        $dayOfYear = (int) $PHPDateObject->format('z');
        $PHPDateObject->modify('-' . $dayOfYear . ' days');
        $firstDayOfFirstWeek = (int) $PHPDateObject->format('w');
        $daysInFirstWeek = (6 - $firstDayOfFirstWeek + $method) % 7;
        $daysInFirstWeek += 7 * !$daysInFirstWeek;
        $endFirstWeek = $daysInFirstWeek - 1;
        $weekOfYear = \floor(($dayOfYear - $endFirstWeek + 13) / 7);
        return (int) $weekOfYear;
    }
    /**
     * ISOWEEKNUM.
     *
     * Returns the ISO 8601 week number of the year for a specified date.
     *
     * Excel Function:
     *        ISOWEEKNUM(dateValue)
     *
     * @param mixed $dateValue Excel date serial value (float), PHP date timestamp (integer),
     *                                    PHP DateTime object, or a standard date string
     *
     * @return int|string Week Number
     */
    public static function isoWeekNumber($dateValue)
    {
        if (self::apparentBug($dateValue)) {
            return 52;
        }
        try {
            $dateValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::getDateValue($dateValue);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Execute function
        $PHPDateObject = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($dateValue);
        \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::silly1900($PHPDateObject);
        return (int) $PHPDateObject->format('W');
    }
    /**
     * WEEKDAY.
     *
     * Returns the day of the week for a specified date. The day is given as an integer
     * ranging from 0 to 7 (dependent on the requested style).
     *
     * Excel Function:
     *        WEEKDAY(dateValue[,style])
     *
     * @param null|float|int|string $dateValue Excel date serial value (float), PHP date timestamp (integer),
     *                                    PHP DateTime object, or a standard date string
     * @param mixed $style A number that determines the type of return value
     *                                        1 or omitted    Numbers 1 (Sunday) through 7 (Saturday).
     *                                        2                Numbers 1 (Monday) through 7 (Sunday).
     *                                        3                Numbers 0 (Monday) through 6 (Sunday).
     *
     * @return int|string Day of the week value
     */
    public static function day($dateValue, $style = 1)
    {
        try {
            $dateValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::getDateValue($dateValue);
            $style = self::validateStyle($style);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Execute function
        $PHPDateObject = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($dateValue);
        \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::silly1900($PHPDateObject);
        $DoW = (int) $PHPDateObject->format('w');
        switch ($style) {
            case 1:
                ++$DoW;
                break;
            case 2:
                $DoW = self::dow0Becomes7($DoW);
                break;
            case 3:
                $DoW = self::dow0Becomes7($DoW) - 1;
                break;
        }
        return $DoW;
    }
    /**
     * @param mixed $style expect int
     */
    private static function validateStyle($style) : int
    {
        $style = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($style);
        if (!\is_numeric($style)) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE());
        }
        $style = (int) $style;
        if ($style < 1 || $style > 3) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN());
        }
        return $style;
    }
    private static function dow0Becomes7(int $DoW) : int
    {
        return $DoW === 0 ? 7 : $DoW;
    }
    /**
     * @param mixed $dateValue Excel date serial value (float), PHP date timestamp (integer),
     *                                    PHP DateTime object, or a standard date string
     */
    private static function apparentBug($dateValue) : bool
    {
        if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::getExcelCalendar() !== \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::CALENDAR_MAC_1904) {
            if (\is_bool($dateValue)) {
                return \true;
            }
            if (\is_numeric($dateValue) && !(int) $dateValue) {
                return \true;
            }
        }
        return \false;
    }
    /**
     * Validate dateValue parameter.
     *
     * @param mixed $dateValue
     */
    private static function validateDateValue($dateValue) : float
    {
        if (\is_bool($dateValue)) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE());
        }
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::getDateValue($dateValue);
    }
    /**
     * Validate method parameter.
     *
     * @param mixed $method
     */
    private static function validateMethod($method) : int
    {
        if ($method === null) {
            $method = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Constants::STARTWEEK_SUNDAY;
        }
        $method = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($method);
        if (!\is_numeric($method)) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE());
        }
        $method = (int) $method;
        if (!\array_key_exists($method, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Constants::METHODARR)) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN());
        }
        $method = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Constants::METHODARR[$method];
        return $method;
    }
    private static function buggyWeekNum1900(int $method) : bool
    {
        return $method === \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Constants::DOW_SUNDAY && \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::getExcelCalendar() === \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::CALENDAR_WINDOWS_1900;
    }
    private static function buggyWeekNum1904(int $method, bool $origNull, \DateTime $dateObject) : bool
    {
        // This appears to be another Excel bug.
        return $method === \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Constants::DOW_SUNDAY && \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::getExcelCalendar() === \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::CALENDAR_MAC_1904 && !$origNull && $dateObject->format('Y-m-d') === '1904-01-01';
    }
}
