<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering;

use ScopedGtbabel\Complex\Complex as ComplexObject;
use ScopedGtbabel\Complex\Exception as ComplexException;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Complex
{
    /**
     * COMPLEX.
     *
     * Converts real and imaginary coefficients into a complex number of the form x +/- yi or x +/- yj.
     *
     * Excel Function:
     *        COMPLEX(realNumber,imaginary[,suffix])
     *
     * @param mixed $realNumber the real float coefficient of the complex number
     * @param mixed $imaginary the imaginary float coefficient of the complex number
     * @param mixed $suffix The character suffix for the imaginary component of the complex number.
     *                          If omitted, the suffix is assumed to be "i".
     *
     * @return string
     */
    public static function COMPLEX($realNumber = 0.0, $imaginary = 0.0, $suffix = 'i')
    {
        $realNumber = $realNumber === null ? 0.0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($realNumber);
        $imaginary = $imaginary === null ? 0.0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($imaginary);
        $suffix = $suffix === null ? 'i' : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($suffix);
        try {
            $realNumber = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\EngineeringValidations::validateFloat($realNumber);
            $imaginary = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\EngineeringValidations::validateFloat($imaginary);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($suffix == 'i' || $suffix == 'j' || $suffix == '') {
            $complex = new \ScopedGtbabel\Complex\Complex($realNumber, $imaginary, $suffix);
            return (string) $complex;
        }
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
    }
    /**
     * IMAGINARY.
     *
     * Returns the imaginary coefficient of a complex number in x + yi or x + yj text format.
     *
     * Excel Function:
     *        IMAGINARY(complexNumber)
     *
     * @param string $complexNumber the complex number for which you want the imaginary
     *                                         coefficient
     *
     * @return float|string
     */
    public static function IMAGINARY($complexNumber)
    {
        $complexNumber = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($complexNumber);
        try {
            $complex = new \ScopedGtbabel\Complex\Complex($complexNumber);
        } catch (\ScopedGtbabel\Complex\Exception $e) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return $complex->getImaginary();
    }
    /**
     * IMREAL.
     *
     * Returns the real coefficient of a complex number in x + yi or x + yj text format.
     *
     * Excel Function:
     *        IMREAL(complexNumber)
     *
     * @param string $complexNumber the complex number for which you want the real coefficient
     *
     * @return float|string
     */
    public static function IMREAL($complexNumber)
    {
        $complexNumber = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($complexNumber);
        try {
            $complex = new \ScopedGtbabel\Complex\Complex($complexNumber);
        } catch (\ScopedGtbabel\Complex\Exception $e) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return $complex->getReal();
    }
}
