<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants as FinancialConstants;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Periodic
{
    /**
     * FV.
     *
     * Returns the Future Value of a cash flow with constant payments and interest rate (annuities).
     *
     * Excel Function:
     *        FV(rate,nper,pmt[,pv[,type]])
     *
     * @param mixed $rate The interest rate per period
     * @param mixed $numberOfPeriods Total number of payment periods in an annuity as an integer
     * @param mixed $payment The payment made each period: it cannot change over the
     *                            life of the annuity. Typically, pmt contains principal
     *                            and interest but no other fees or taxes.
     * @param mixed $presentValue present Value, or the lump-sum amount that a series of
     *                            future payments is worth right now
     * @param mixed $type A number 0 or 1 and indicates when payments are due:
     *                      0 or omitted    At the end of the period.
     *                      1               At the beginning of the period.
     *
     * @return float|string
     */
    public static function futureValue($rate, $numberOfPeriods, $payment = 0.0, $presentValue = 0.0, $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD)
    {
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $numberOfPeriods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($numberOfPeriods);
        $payment = $payment === null ? 0.0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($payment);
        $presentValue = $presentValue === null ? 0.0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($presentValue);
        $type = $type === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($type);
        try {
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateRate($rate);
            $numberOfPeriods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($numberOfPeriods);
            $payment = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFloat($payment);
            $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePresentValue($presentValue);
            $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePeriodType($type);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        return self::calculateFutureValue($rate, $numberOfPeriods, $payment, $presentValue, $type);
    }
    /**
     * PV.
     *
     * Returns the Present Value of a cash flow with constant payments and interest rate (annuities).
     *
     * @param mixed $rate Interest rate per period
     * @param mixed $numberOfPeriods Number of periods as an integer
     * @param mixed $payment Periodic payment (annuity)
     * @param mixed $futureValue Future Value
     * @param mixed $type Payment type: 0 = at the end of each period, 1 = at the beginning of each period
     *
     * @return float|string Result, or a string containing an error
     */
    public static function presentValue($rate, $numberOfPeriods, $payment = 0.0, $futureValue = 0.0, $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD)
    {
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $numberOfPeriods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($numberOfPeriods);
        $payment = $payment === null ? 0.0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($payment);
        $futureValue = $futureValue === null ? 0.0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($futureValue);
        $type = $type === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($type);
        try {
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateRate($rate);
            $numberOfPeriods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($numberOfPeriods);
            $payment = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFloat($payment);
            $futureValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFutureValue($futureValue);
            $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePeriodType($type);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Validate parameters
        if ($numberOfPeriods < 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return self::calculatePresentValue($rate, $numberOfPeriods, $payment, $futureValue, $type);
    }
    /**
     * NPER.
     *
     * Returns the number of periods for a cash flow with constant periodic payments (annuities), and interest rate.
     *
     * @param mixed $rate Interest rate per period
     * @param mixed $payment Periodic payment (annuity)
     * @param mixed $presentValue Present Value
     * @param mixed $futureValue Future Value
     * @param mixed $type Payment type: 0 = at the end of each period, 1 = at the beginning of each period
     *
     * @return float|string Result, or a string containing an error
     */
    public static function periods($rate, $payment, $presentValue, $futureValue = 0.0, $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD)
    {
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $payment = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($payment);
        $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($presentValue);
        $futureValue = $futureValue === null ? 0.0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($futureValue);
        $type = $type === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($type);
        try {
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateRate($rate);
            $payment = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFloat($payment);
            $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePresentValue($presentValue);
            $futureValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFutureValue($futureValue);
            $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePeriodType($type);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Validate parameters
        if ($payment == 0.0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return self::calculatePeriods($rate, $payment, $presentValue, $futureValue, $type);
    }
    private static function calculateFutureValue(float $rate, int $numberOfPeriods, float $payment, float $presentValue, int $type) : float
    {
        if ($rate !== null && $rate != 0) {
            return -$presentValue * (1 + $rate) ** $numberOfPeriods - $payment * (1 + $rate * $type) * ((1 + $rate) ** $numberOfPeriods - 1) / $rate;
        }
        return -$presentValue - $payment * $numberOfPeriods;
    }
    private static function calculatePresentValue(float $rate, int $numberOfPeriods, float $payment, float $futureValue, int $type) : float
    {
        if ($rate != 0.0) {
            return (-$payment * (1 + $rate * $type) * (((1 + $rate) ** $numberOfPeriods - 1) / $rate) - $futureValue) / (1 + $rate) ** $numberOfPeriods;
        }
        return -$futureValue - $payment * $numberOfPeriods;
    }
    /**
     * @return float|string
     */
    private static function calculatePeriods(float $rate, float $payment, float $presentValue, float $futureValue, int $type)
    {
        if ($rate != 0.0) {
            if ($presentValue == 0.0) {
                return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
            }
            return \log(($payment * (1 + $rate * $type) / $rate - $futureValue) / ($presentValue + $payment * (1 + $rate * $type) / $rate)) / \log(1 + $rate);
        }
        return (-$presentValue - $futureValue) / $payment;
    }
}
