<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants as FinancialConstants;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Cumulative
{
    /**
     * CUMIPMT.
     *
     * Returns the cumulative interest paid on a loan between the start and end periods.
     *
     * Excel Function:
     *        CUMIPMT(rate,nper,pv,start,end[,type])
     *
     * @param mixed $rate The Interest rate
     * @param mixed $periods The total number of payment periods
     * @param mixed $presentValue Present Value
     * @param mixed $start The first period in the calculation.
     *                       Payment periods are numbered beginning with 1.
     * @param mixed $end the last period in the calculation
     * @param mixed $type A number 0 or 1 and indicates when payments are due:
     *                    0 or omitted    At the end of the period.
     *                    1               At the beginning of the period.
     *
     * @return float|string
     */
    public static function interest($rate, $periods, $presentValue, $start, $end, $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD)
    {
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $periods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($periods);
        $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($presentValue);
        $start = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($start);
        $end = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($end);
        $type = $type === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($type);
        try {
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateRate($rate);
            $periods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($periods);
            $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePresentValue($presentValue);
            $start = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($start);
            $end = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($end);
            $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePeriodType($type);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Validate parameters
        if ($start < 1 || $start > $end) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        // Calculate
        $interest = 0;
        for ($per = $start; $per <= $end; ++$per) {
            $ipmt = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\Interest::payment($rate, $per, $periods, $presentValue, 0, $type);
            if (\is_string($ipmt)) {
                return $ipmt;
            }
            $interest += $ipmt;
        }
        return $interest;
    }
    /**
     * CUMPRINC.
     *
     * Returns the cumulative principal paid on a loan between the start and end periods.
     *
     * Excel Function:
     *        CUMPRINC(rate,nper,pv,start,end[,type])
     *
     * @param mixed $rate The Interest rate
     * @param mixed $periods The total number of payment periods as an integer
     * @param mixed $presentValue Present Value
     * @param mixed $start The first period in the calculation.
     *                       Payment periods are numbered beginning with 1.
     * @param mixed $end the last period in the calculation
     * @param mixed $type A number 0 or 1 and indicates when payments are due:
     *                    0 or omitted    At the end of the period.
     *                    1               At the beginning of the period.
     *
     * @return float|string
     */
    public static function principal($rate, $periods, $presentValue, $start, $end, $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD)
    {
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $periods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($periods);
        $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($presentValue);
        $start = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($start);
        $end = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($end);
        $type = $type === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($type);
        try {
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateRate($rate);
            $periods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($periods);
            $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePresentValue($presentValue);
            $start = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($start);
            $end = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($end);
            $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePeriodType($type);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Validate parameters
        if ($start < 1 || $start > $end) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
        }
        // Calculate
        $principal = 0;
        for ($per = $start; $per <= $end; ++$per) {
            $ppmt = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\Payments::interestPayment($rate, $per, $periods, $presentValue, 0, $type);
            if (\is_string($ppmt)) {
                return $ppmt;
            }
            $principal += $ppmt;
        }
        return $principal;
    }
}
