<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants as FinancialConstants;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Payments
{
    /**
     * PMT.
     *
     * Returns the constant payment (annuity) for a cash flow with a constant interest rate.
     *
     * @param mixed $interestRate Interest rate per period
     * @param mixed $numberOfPeriods Number of periods
     * @param mixed $presentValue Present Value
     * @param mixed $futureValue Future Value
     * @param mixed $type Payment type: 0 = at the end of each period, 1 = at the beginning of each period
     *
     * @return float|string Result, or a string containing an error
     */
    public static function annuity($interestRate, $numberOfPeriods, $presentValue, $futureValue = 0, $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD)
    {
        $interestRate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($interestRate);
        $numberOfPeriods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($numberOfPeriods);
        $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($presentValue);
        $futureValue = $futureValue === null ? 0.0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($futureValue);
        $type = $type === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($type);
        try {
            $interestRate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateRate($interestRate);
            $numberOfPeriods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($numberOfPeriods);
            $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePresentValue($presentValue);
            $futureValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFutureValue($futureValue);
            $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePeriodType($type);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Calculate
        if ($interestRate != 0.0) {
            return (-$futureValue - $presentValue * (1 + $interestRate) ** $numberOfPeriods) / (1 + $interestRate * $type) / (((1 + $interestRate) ** $numberOfPeriods - 1) / $interestRate);
        }
        return (-$presentValue - $futureValue) / $numberOfPeriods;
    }
    /**
     * PPMT.
     *
     * Returns the interest payment for a given period for an investment based on periodic, constant payments
     *         and a constant interest rate.
     *
     * @param mixed $interestRate Interest rate per period
     * @param mixed $period Period for which we want to find the interest
     * @param mixed $numberOfPeriods Number of periods
     * @param mixed $presentValue Present Value
     * @param mixed $futureValue Future Value
     * @param mixed $type Payment type: 0 = at the end of each period, 1 = at the beginning of each period
     *
     * @return float|string Result, or a string containing an error
     */
    public static function interestPayment($interestRate, $period, $numberOfPeriods, $presentValue, $futureValue = 0, $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD)
    {
        $interestRate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($interestRate);
        $period = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($period);
        $numberOfPeriods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($numberOfPeriods);
        $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($presentValue);
        $futureValue = $futureValue === null ? 0.0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($futureValue);
        $type = $type === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::PAYMENT_END_OF_PERIOD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($type);
        try {
            $interestRate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateRate($interestRate);
            $period = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($period);
            $numberOfPeriods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateInt($numberOfPeriods);
            $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePresentValue($presentValue);
            $futureValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFutureValue($futureValue);
            $type = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePeriodType($type);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Validate parameters
        if ($period <= 0 || $period > $numberOfPeriods) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        // Calculate
        $interestAndPrincipal = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\InterestAndPrincipal($interestRate, $period, $numberOfPeriods, $presentValue, $futureValue, $type);
        return $interestAndPrincipal->principal();
    }
}
