<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Single
{
    /**
     * FVSCHEDULE.
     *
     * Returns the future value of an initial principal after applying a series of compound interest rates.
     * Use FVSCHEDULE to calculate the future value of an investment with a variable or adjustable rate.
     *
     * Excel Function:
     *        FVSCHEDULE(principal,schedule)
     *
     * @param mixed $principal the present value
     * @param float[] $schedule an array of interest rates to apply
     *
     * @return float|string
     */
    public static function futureValue($principal, $schedule)
    {
        $principal = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($principal);
        $schedule = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArray($schedule);
        try {
            $principal = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFloat($principal);
            foreach ($schedule as $rate) {
                $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFloat($rate);
                $principal *= 1 + $rate;
            }
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        return $principal;
    }
    /**
     * PDURATION.
     *
     * Calculates the number of periods required for an investment to reach a specified value.
     *
     * @param mixed $rate Interest rate per period
     * @param mixed $presentValue Present Value
     * @param mixed $futureValue Future Value
     *
     * @return float|string Result, or a string containing an error
     */
    public static function periods($rate, $presentValue, $futureValue)
    {
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($presentValue);
        $futureValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($futureValue);
        try {
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateRate($rate);
            $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePresentValue($presentValue);
            $futureValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFutureValue($futureValue);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Validate parameters
        if ($rate <= 0.0 || $presentValue <= 0.0 || $futureValue <= 0.0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return (\log($futureValue) - \log($presentValue)) / \log(1 + $rate);
    }
    /**
     * RRI.
     *
     * Calculates the interest rate required for an investment to grow to a specified future value .
     *
     * @param float $periods The number of periods over which the investment is made
     * @param float $presentValue Present Value
     * @param float $futureValue Future Value
     *
     * @return float|string Result, or a string containing an error
     */
    public static function interestRate($periods = 0.0, $presentValue = 0.0, $futureValue = 0.0)
    {
        $periods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($periods);
        $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($presentValue);
        $futureValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($futureValue);
        try {
            $periods = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFloat($periods);
            $presentValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validatePresentValue($presentValue);
            $futureValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\CashFlowValidations::validateFutureValue($futureValue);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Validate parameters
        if ($periods <= 0.0 || $presentValue <= 0.0 || $futureValue < 0.0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return ($futureValue / $presentValue) ** (1 / $periods) - 1;
    }
}
