<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class InterestRate
{
    /**
     * EFFECT.
     *
     * Returns the effective interest rate given the nominal rate and the number of
     *        compounding payments per year.
     *
     * Excel Function:
     *        EFFECT(nominal_rate,npery)
     *
     * @param mixed $nominalRate Nominal interest rate as a float
     * @param mixed $periodsPerYear Integer number of compounding payments per year
     *
     * @return float|string
     */
    public static function effective($nominalRate = 0, $periodsPerYear = 0)
    {
        $nominalRate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($nominalRate);
        $periodsPerYear = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($periodsPerYear);
        try {
            $nominalRate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateFloat($nominalRate);
            $periodsPerYear = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateInt($periodsPerYear);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($nominalRate <= 0 || $periodsPerYear < 1) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return (1 + $nominalRate / $periodsPerYear) ** $periodsPerYear - 1;
    }
    /**
     * NOMINAL.
     *
     * Returns the nominal interest rate given the effective rate and the number of compounding payments per year.
     *
     * @param mixed $effectiveRate Effective interest rate as a float
     * @param mixed $periodsPerYear Integer number of compounding payments per year
     *
     * @return float|string Result, or a string containing an error
     */
    public static function nominal($effectiveRate = 0, $periodsPerYear = 0)
    {
        $effectiveRate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($effectiveRate);
        $periodsPerYear = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($periodsPerYear);
        try {
            $effectiveRate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateFloat($effectiveRate);
            $periodsPerYear = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateInt($periodsPerYear);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($effectiveRate <= 0 || $periodsPerYear < 1) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        // Calculate
        return $periodsPerYear * (($effectiveRate + 1) ** (1 / $periodsPerYear) - 1);
    }
}
