<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants as FinancialConstants;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class AccruedInterest
{
    public const ACCRINT_CALCMODE_ISSUE_TO_SETTLEMENT = \true;
    public const ACCRINT_CALCMODE_FIRST_INTEREST_TO_SETTLEMENT = \false;
    /**
     * ACCRINT.
     *
     * Returns the accrued interest for a security that pays periodic interest.
     *
     * Excel Function:
     *        ACCRINT(issue,firstinterest,settlement,rate,par,frequency[,basis][,calc_method])
     *
     * @param mixed $issue the security's issue date
     * @param mixed $firstInterest the security's first interest date
     * @param mixed $settlement The security's settlement date.
     *                              The security settlement date is the date after the issue date
     *                                  when the security is traded to the buyer.
     * @param mixed $rate The security's annual coupon rate
     * @param mixed $parValue The security's par value.
     *                            If you omit par, ACCRINT uses $1,000.
     * @param mixed $frequency The number of coupon payments per year.
     *                             Valid frequency values are:
     *                               1    Annual
     *                               2    Semi-Annual
     *                               4    Quarterly
     * @param mixed $basis The type of day count to use.
     *                         0 or omitted    US (NASD) 30/360
     *                         1               Actual/actual
     *                         2               Actual/360
     *                         3               Actual/365
     *                         4               European 30/360
     * @param mixed $calcMethod
     *
     * @return float|string Result, or a string containing an error
     */
    public static function periodic($issue, $firstInterest, $settlement, $rate, $parValue = 1000, $frequency = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::FREQUENCY_ANNUAL, $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD, $calcMethod = self::ACCRINT_CALCMODE_ISSUE_TO_SETTLEMENT)
    {
        $issue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($issue);
        $firstInterest = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($firstInterest);
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $parValue = $parValue === null ? 1000 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($parValue);
        $frequency = $frequency === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::FREQUENCY_ANNUAL : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($frequency);
        $basis = $basis === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($basis);
        try {
            $issue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateIssueDate($issue);
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSettlementDate($settlement);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSecurityPeriod($issue, $settlement);
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateRate($rate);
            $parValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateParValue($parValue);
            $frequency = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateFrequency($frequency);
            $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateBasis($basis);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $daysBetweenIssueAndSettlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($issue, $settlement, $basis);
        if (!\is_numeric($daysBetweenIssueAndSettlement)) {
            //    return date error
            return $daysBetweenIssueAndSettlement;
        }
        $daysBetweenFirstInterestAndSettlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($firstInterest, $settlement, $basis);
        if (!\is_numeric($daysBetweenFirstInterestAndSettlement)) {
            //    return date error
            return $daysBetweenFirstInterestAndSettlement;
        }
        return $parValue * $rate * $daysBetweenIssueAndSettlement;
    }
    /**
     * ACCRINTM.
     *
     * Returns the accrued interest for a security that pays interest at maturity.
     *
     * Excel Function:
     *        ACCRINTM(issue,settlement,rate[,par[,basis]])
     *
     * @param mixed $issue The security's issue date
     * @param mixed $settlement The security's settlement (or maturity) date
     * @param mixed $rate The security's annual coupon rate
     * @param mixed $parValue The security's par value.
     *                            If you omit parValue, ACCRINT uses $1,000.
     * @param mixed $basis The type of day count to use.
     *                         0 or omitted    US (NASD) 30/360
     *                         1               Actual/actual
     *                         2               Actual/360
     *                         3               Actual/365
     *                         4               European 30/360
     *
     * @return float|string Result, or a string containing an error
     */
    public static function atMaturity($issue, $settlement, $rate, $parValue = 1000, $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD)
    {
        $issue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($issue);
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $parValue = $parValue === null ? 1000 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($parValue);
        $basis = $basis === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($basis);
        try {
            $issue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateIssueDate($issue);
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSettlementDate($settlement);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSecurityPeriod($issue, $settlement);
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateRate($rate);
            $parValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateParValue($parValue);
            $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateBasis($basis);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $daysBetweenIssueAndSettlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($issue, $settlement, $basis);
        if (!\is_numeric($daysBetweenIssueAndSettlement)) {
            //    return date error
            return $daysBetweenIssueAndSettlement;
        }
        return $parValue * $rate * $daysBetweenIssueAndSettlement;
    }
}
