<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants as FinancialConstants;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Helpers;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Price
{
    /**
     * PRICE.
     *
     * Returns the price per $100 face value of a security that pays periodic interest.
     *
     * @param mixed $settlement The security's settlement date.
     *                              The security settlement date is the date after the issue date when the security
     *                              is traded to the buyer.
     * @param mixed $maturity The security's maturity date.
     *                                The maturity date is the date when the security expires.
     * @param mixed $rate the security's annual coupon rate
     * @param mixed $yield the security's annual yield
     * @param mixed $redemption The number of coupon payments per year.
     *                              For annual payments, frequency = 1;
     *                              for semiannual, frequency = 2;
     *                              for quarterly, frequency = 4.
     * @param mixed $frequency
     * @param mixed $basis The type of day count to use.
     *                         0 or omitted    US (NASD) 30/360
     *                         1               Actual/actual
     *                         2               Actual/360
     *                         3               Actual/365
     *                         4               European 30/360
     *
     * @return float|string Result, or a string containing an error
     */
    public static function price($settlement, $maturity, $rate, $yield, $redemption, $frequency, $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD)
    {
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($maturity);
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $yield = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($yield);
        $redemption = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($redemption);
        $frequency = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($frequency);
        $basis = $basis === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($basis);
        try {
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSettlementDate($settlement);
            $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateMaturityDate($maturity);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSecurityPeriod($settlement, $maturity);
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateRate($rate);
            $yield = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateYield($yield);
            $redemption = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateRedemption($redemption);
            $frequency = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateFrequency($frequency);
            $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateBasis($basis);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $dsc = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::COUPDAYSNC($settlement, $maturity, $frequency, $basis);
        $e = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::COUPDAYS($settlement, $maturity, $frequency, $basis);
        $n = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::COUPNUM($settlement, $maturity, $frequency, $basis);
        $a = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::COUPDAYBS($settlement, $maturity, $frequency, $basis);
        $baseYF = 1.0 + $yield / $frequency;
        $rfp = 100 * ($rate / $frequency);
        $de = $dsc / $e;
        $result = $redemption / $baseYF ** (--$n + $de);
        for ($k = 0; $k <= $n; ++$k) {
            $result += $rfp / $baseYF ** ($k + $de);
        }
        $result -= $rfp * ($a / $e);
        return $result;
    }
    /**
     * PRICEDISC.
     *
     * Returns the price per $100 face value of a discounted security.
     *
     * @param mixed $settlement The security's settlement date.
     *                              The security settlement date is the date after the issue date when the security
     *                              is traded to the buyer.
     * @param mixed $maturity The security's maturity date.
     *                                The maturity date is the date when the security expires.
     * @param mixed $discount The security's discount rate
     * @param mixed $redemption The security's redemption value per $100 face value
     * @param mixed $basis The type of day count to use.
     *                         0 or omitted    US (NASD) 30/360
     *                         1               Actual/actual
     *                         2               Actual/360
     *                         3               Actual/365
     *                         4               European 30/360
     *
     * @return float|string Result, or a string containing an error
     */
    public static function priceDiscounted($settlement, $maturity, $discount, $redemption, $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD)
    {
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($maturity);
        $discount = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($discount);
        $redemption = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($redemption);
        $basis = $basis === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($basis);
        try {
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSettlementDate($settlement);
            $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateMaturityDate($maturity);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSecurityPeriod($settlement, $maturity);
            $discount = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateDiscount($discount);
            $redemption = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateRedemption($redemption);
            $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateBasis($basis);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $daysBetweenSettlementAndMaturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($settlement, $maturity, $basis);
        if (!\is_numeric($daysBetweenSettlementAndMaturity)) {
            //    return date error
            return $daysBetweenSettlementAndMaturity;
        }
        return $redemption * (1 - $discount * $daysBetweenSettlementAndMaturity);
    }
    /**
     * PRICEMAT.
     *
     * Returns the price per $100 face value of a security that pays interest at maturity.
     *
     * @param mixed $settlement The security's settlement date.
     *                              The security's settlement date is the date after the issue date when the
     *                              security is traded to the buyer.
     * @param mixed $maturity The security's maturity date.
     *                                The maturity date is the date when the security expires.
     * @param mixed $issue The security's issue date
     * @param mixed $rate The security's interest rate at date of issue
     * @param mixed $yield The security's annual yield
     * @param mixed $basis The type of day count to use.
     *                         0 or omitted    US (NASD) 30/360
     *                         1               Actual/actual
     *                         2               Actual/360
     *                         3               Actual/365
     *                         4               European 30/360
     *
     * @return float|string Result, or a string containing an error
     */
    public static function priceAtMaturity($settlement, $maturity, $issue, $rate, $yield, $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD)
    {
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($maturity);
        $issue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($issue);
        $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($rate);
        $yield = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($yield);
        $basis = $basis === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($basis);
        try {
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSettlementDate($settlement);
            $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateMaturityDate($maturity);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSecurityPeriod($settlement, $maturity);
            $issue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateIssueDate($issue);
            $rate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateRate($rate);
            $yield = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateYield($yield);
            $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateBasis($basis);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $daysPerYear = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Helpers::daysPerYear(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateParts::year($settlement), $basis);
        if (!\is_numeric($daysPerYear)) {
            return $daysPerYear;
        }
        $daysBetweenIssueAndSettlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($issue, $settlement, $basis);
        if (!\is_numeric($daysBetweenIssueAndSettlement)) {
            //    return date error
            return $daysBetweenIssueAndSettlement;
        }
        $daysBetweenIssueAndSettlement *= $daysPerYear;
        $daysBetweenIssueAndMaturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($issue, $maturity, $basis);
        if (!\is_numeric($daysBetweenIssueAndMaturity)) {
            //    return date error
            return $daysBetweenIssueAndMaturity;
        }
        $daysBetweenIssueAndMaturity *= $daysPerYear;
        $daysBetweenSettlementAndMaturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($settlement, $maturity, $basis);
        if (!\is_numeric($daysBetweenSettlementAndMaturity)) {
            //    return date error
            return $daysBetweenSettlementAndMaturity;
        }
        $daysBetweenSettlementAndMaturity *= $daysPerYear;
        return (100 + $daysBetweenIssueAndMaturity / $daysPerYear * $rate * 100) / (1 + $daysBetweenSettlementAndMaturity / $daysPerYear * $yield) - $daysBetweenIssueAndSettlement / $daysPerYear * $rate * 100;
    }
    /**
     * RECEIVED.
     *
     * Returns the amount received at maturity for a fully invested Security.
     *
     * @param mixed $settlement The security's settlement date.
     *                              The security settlement date is the date after the issue date when the security
     *                                  is traded to the buyer.
     * @param mixed $maturity The security's maturity date.
     *                            The maturity date is the date when the security expires.
     * @param mixed $investment The amount invested in the security
     * @param mixed $discount The security's discount rate
     * @param mixed $basis The type of day count to use.
     *                         0 or omitted    US (NASD) 30/360
     *                         1               Actual/actual
     *                         2               Actual/360
     *                         3               Actual/365
     *                         4               European 30/360
     *
     * @return float|string Result, or a string containing an error
     */
    public static function received($settlement, $maturity, $investment, $discount, $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD)
    {
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($maturity);
        $investment = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($investment);
        $discount = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($discount);
        $basis = $basis === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($basis);
        try {
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSettlementDate($settlement);
            $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateMaturityDate($maturity);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSecurityPeriod($settlement, $maturity);
            $investment = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateFloat($investment);
            $discount = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateDiscount($discount);
            $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateBasis($basis);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($investment <= 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        $daysBetweenSettlementAndMaturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($settlement, $maturity, $basis);
        if (!\is_numeric($daysBetweenSettlementAndMaturity)) {
            //    return date error
            return $daysBetweenSettlementAndMaturity;
        }
        return $investment / (1 - $discount * $daysBetweenSettlementAndMaturity);
    }
}
