<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants as FinancialConstants;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Rates
{
    /**
     * DISC.
     *
     * Returns the discount rate for a security.
     *
     * Excel Function:
     *        DISC(settlement,maturity,price,redemption[,basis])
     *
     * @param mixed $settlement The security's settlement date.
     *                              The security settlement date is the date after the issue
     *                                  date when the security is traded to the buyer.
     * @param mixed $maturity The security's maturity date.
     *                            The maturity date is the date when the security expires.
     * @param mixed $price The security's price per $100 face value
     * @param mixed $redemption The security's redemption value per $100 face value
     * @param mixed $basis The type of day count to use.
     *                         0 or omitted    US (NASD) 30/360
     *                         1               Actual/actual
     *                         2               Actual/360
     *                         3               Actual/365
     *                         4               European 30/360
     *
     * @return float|string
     */
    public static function discount($settlement, $maturity, $price, $redemption, $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD)
    {
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($maturity);
        $price = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($price);
        $redemption = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($redemption);
        $basis = $basis === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($basis);
        try {
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSettlementDate($settlement);
            $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateMaturityDate($maturity);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSecurityPeriod($settlement, $maturity);
            $price = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validatePrice($price);
            $redemption = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateRedemption($redemption);
            $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateBasis($basis);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($price <= 0.0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        $daysBetweenSettlementAndMaturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($settlement, $maturity, $basis);
        if (!\is_numeric($daysBetweenSettlementAndMaturity)) {
            //    return date error
            return $daysBetweenSettlementAndMaturity;
        }
        return (1 - $price / $redemption) / $daysBetweenSettlementAndMaturity;
    }
    /**
     * INTRATE.
     *
     * Returns the interest rate for a fully invested security.
     *
     * Excel Function:
     *        INTRATE(settlement,maturity,investment,redemption[,basis])
     *
     * @param mixed $settlement The security's settlement date.
     *                              The security settlement date is the date after the issue date when the security
     *                                  is traded to the buyer.
     * @param mixed $maturity The security's maturity date.
     *                            The maturity date is the date when the security expires.
     * @param mixed $investment the amount invested in the security
     * @param mixed $redemption the amount to be received at maturity
     * @param mixed $basis The type of day count to use.
     *                         0 or omitted    US (NASD) 30/360
     *                         1               Actual/actual
     *                         2               Actual/360
     *                         3               Actual/365
     *                         4               European 30/360
     *
     * @return float|string
     */
    public static function interest($settlement, $maturity, $investment, $redemption, $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD)
    {
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($maturity);
        $investment = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($investment);
        $redemption = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($redemption);
        $basis = $basis === null ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_NASD : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($basis);
        try {
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSettlementDate($settlement);
            $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateMaturityDate($maturity);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateSecurityPeriod($settlement, $maturity);
            $investment = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateFloat($investment);
            $redemption = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateRedemption($redemption);
            $basis = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\SecurityValidations::validateBasis($basis);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($investment <= 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        $daysBetweenSettlementAndMaturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::fraction($settlement, $maturity, $basis);
        if (!\is_numeric($daysBetweenSettlementAndMaturity)) {
            //    return date error
            return $daysBetweenSettlementAndMaturity;
        }
        return ($redemption / $investment - 1) / $daysBetweenSettlementAndMaturity;
    }
}
