<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants as FinancialConstants;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class TreasuryBill
{
    /**
     * TBILLEQ.
     *
     * Returns the bond-equivalent yield for a Treasury bill.
     *
     * @param mixed $settlement The Treasury bill's settlement date.
     *                                The Treasury bill's settlement date is the date after the issue date
     *                                    when the Treasury bill is traded to the buyer.
     * @param mixed $maturity The Treasury bill's maturity date.
     *                                The maturity date is the date when the Treasury bill expires.
     * @param mixed $discount The Treasury bill's discount rate
     *
     * @return float|string Result, or a string containing an error
     */
    public static function bondEquivalentYield($settlement, $maturity, $discount)
    {
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($maturity);
        $discount = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($discount);
        try {
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateSettlementDate($settlement);
            $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateMaturityDate($maturity);
            $discount = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateFloat($discount);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($discount <= 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        $daysBetweenSettlementAndMaturity = $maturity - $settlement;
        $daysPerYear = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Helpers::daysPerYear(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateParts::year($maturity), \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_ACTUAL);
        if ($daysBetweenSettlementAndMaturity > $daysPerYear || $daysBetweenSettlementAndMaturity < 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return 365 * $discount / (360 - $discount * $daysBetweenSettlementAndMaturity);
    }
    /**
     * TBILLPRICE.
     *
     * Returns the price per $100 face value for a Treasury bill.
     *
     * @param mixed $settlement The Treasury bill's settlement date.
     *                                The Treasury bill's settlement date is the date after the issue date
     *                                    when the Treasury bill is traded to the buyer.
     * @param mixed $maturity The Treasury bill's maturity date.
     *                                The maturity date is the date when the Treasury bill expires.
     * @param mixed $discount The Treasury bill's discount rate
     *
     * @return float|string Result, or a string containing an error
     */
    public static function price($settlement, $maturity, $discount)
    {
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($maturity);
        $discount = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($discount);
        try {
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateSettlementDate($settlement);
            $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateMaturityDate($maturity);
            $discount = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateFloat($discount);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($discount <= 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        $daysBetweenSettlementAndMaturity = $maturity - $settlement;
        $daysPerYear = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Helpers::daysPerYear(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateParts::year($maturity), \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_ACTUAL);
        if ($daysBetweenSettlementAndMaturity > $daysPerYear || $daysBetweenSettlementAndMaturity < 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        $price = 100 * (1 - $discount * $daysBetweenSettlementAndMaturity / 360);
        if ($price < 0.0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return $price;
    }
    /**
     * TBILLYIELD.
     *
     * Returns the yield for a Treasury bill.
     *
     * @param mixed $settlement The Treasury bill's settlement date.
     *                                The Treasury bill's settlement date is the date after the issue date when
     *                                    the Treasury bill is traded to the buyer.
     * @param mixed $maturity The Treasury bill's maturity date.
     *                                The maturity date is the date when the Treasury bill expires.
     * @param mixed $price The Treasury bill's price per $100 face value
     *
     * @return float|string
     */
    public static function yield($settlement, $maturity, $price)
    {
        $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($settlement);
        $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($maturity);
        $price = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($price);
        try {
            $settlement = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateSettlementDate($settlement);
            $maturity = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validateMaturityDate($maturity);
            $price = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\FinancialValidations::validatePrice($price);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $daysBetweenSettlementAndMaturity = $maturity - $settlement;
        $daysPerYear = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Helpers::daysPerYear(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateParts::year($maturity), \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Constants::BASIS_DAYS_PER_YEAR_ACTUAL);
        if ($daysBetweenSettlementAndMaturity > $daysPerYear || $daysBetweenSettlementAndMaturity < 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return (100 - $price) / $price * (360 / $daysBetweenSettlementAndMaturity);
    }
}
