<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate;
class Address
{
    public const ADDRESS_ABSOLUTE = 1;
    public const ADDRESS_COLUMN_RELATIVE = 2;
    public const ADDRESS_ROW_RELATIVE = 3;
    public const ADDRESS_RELATIVE = 4;
    public const REFERENCE_STYLE_A1 = \true;
    public const REFERENCE_STYLE_R1C1 = \false;
    /**
     * ADDRESS.
     *
     * Creates a cell address as text, given specified row and column numbers.
     *
     * Excel Function:
     *        =ADDRESS(row, column, [relativity], [referenceStyle], [sheetText])
     *
     * @param mixed $row Row number (integer) to use in the cell reference
     * @param mixed $column Column number (integer) to use in the cell reference
     * @param mixed $relativity Integer flag indicating the type of reference to return
     *                             1 or omitted    Absolute
     *                             2               Absolute row; relative column
     *                             3               Relative row; absolute column
     *                             4               Relative
     * @param mixed $referenceStyle A logical (boolean) value that specifies the A1 or R1C1 reference style.
     *                                TRUE or omitted    ADDRESS returns an A1-style reference
     *                                FALSE              ADDRESS returns an R1C1-style reference
     * @param mixed $sheetName Optional Name of worksheet to use
     *
     * @return string
     */
    public static function cell($row, $column, $relativity = 1, $referenceStyle = \true, $sheetName = '')
    {
        $row = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($row);
        $column = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($column);
        $relativity = $relativity === null ? 1 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($relativity);
        $referenceStyle = $referenceStyle === null ? \true : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($referenceStyle);
        $sheetName = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($sheetName);
        if ($row < 1 || $column < 1) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
        }
        $sheetName = self::sheetName($sheetName);
        if (!\is_bool($referenceStyle) || $referenceStyle === self::REFERENCE_STYLE_A1) {
            return self::formatAsA1($row, $column, $relativity, $sheetName);
        }
        return self::formatAsR1C1($row, $column, $relativity, $sheetName);
    }
    private static function sheetName(string $sheetName)
    {
        if ($sheetName > '') {
            if (\strpos($sheetName, ' ') !== \false || \strpos($sheetName, '[') !== \false) {
                $sheetName = "'{$sheetName}'";
            }
            $sheetName .= '!';
        }
        return $sheetName;
    }
    private static function formatAsA1(int $row, int $column, int $relativity, string $sheetName) : string
    {
        $rowRelative = $columnRelative = '$';
        if ($relativity == self::ADDRESS_COLUMN_RELATIVE || $relativity == self::ADDRESS_RELATIVE) {
            $columnRelative = '';
        }
        if ($relativity == self::ADDRESS_ROW_RELATIVE || $relativity == self::ADDRESS_RELATIVE) {
            $rowRelative = '';
        }
        $column = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($column);
        return "{$sheetName}{$columnRelative}{$column}{$rowRelative}{$row}";
    }
    private static function formatAsR1C1(int $row, int $column, int $relativity, string $sheetName) : string
    {
        if ($relativity == self::ADDRESS_COLUMN_RELATIVE || $relativity == self::ADDRESS_RELATIVE) {
            $column = "[{$column}]";
        }
        if ($relativity == self::ADDRESS_ROW_RELATIVE || $relativity == self::ADDRESS_RELATIVE) {
            $row = "[{$row}]";
        }
        return "{$sheetName}R{$row}C{$column}";
    }
}
