<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical;
class Factorial
{
    /**
     * FACT.
     *
     * Returns the factorial of a number.
     * The factorial of a number is equal to 1*2*3*...* number.
     *
     * Excel Function:
     *        FACT(factVal)
     *
     * @param float $factVal Factorial Value
     *
     * @return float|int|string Factorial, or a string containing an error
     */
    public static function fact($factVal)
    {
        try {
            $factVal = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNumericNullBool($factVal);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNotNegative($factVal);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $factLoop = \floor($factVal);
        if ($factVal > $factLoop) {
            if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getCompatibilityMode() == \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::COMPATIBILITY_GNUMERIC) {
                return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Gamma::gammaValue($factVal + 1);
            }
        }
        $factorial = 1;
        while ($factLoop > 1) {
            $factorial *= $factLoop--;
        }
        return $factorial;
    }
    /**
     * FACTDOUBLE.
     *
     * Returns the double factorial of a number.
     *
     * Excel Function:
     *        FACTDOUBLE(factVal)
     *
     * @param float $factVal Factorial Value
     *
     * @return float|int|string Double Factorial, or a string containing an error
     */
    public static function factDouble($factVal)
    {
        try {
            $factVal = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNumericNullSubstitution($factVal, 0);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNotNegative($factVal);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $factLoop = \floor($factVal);
        $factorial = 1;
        while ($factLoop > 1) {
            $factorial *= $factLoop;
            $factLoop -= 2;
        }
        return $factorial;
    }
    /**
     * MULTINOMIAL.
     *
     * Returns the ratio of the factorial of a sum of values to the product of factorials.
     *
     * @param mixed[] $args An array of mixed values for the Data Series
     *
     * @return float|string The result, or a string containing an error
     */
    public static function multinomial(...$args)
    {
        $summer = 0;
        $divisor = 1;
        try {
            // Loop through arguments
            foreach (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArray($args) as $argx) {
                $arg = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNumericNullSubstitution($argx, null);
                \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNotNegative($arg);
                $arg = (int) $arg;
                $summer += $arg;
                $divisor *= self::fact($arg);
            }
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $summer = self::fact($summer);
        return $summer / $divisor;
    }
}
