<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig;

use ScopedGtbabel\Matrix\Builder;
use ScopedGtbabel\Matrix\Div0Exception as MatrixDiv0Exception;
use ScopedGtbabel\Matrix\Exception as MatrixException;
use ScopedGtbabel\Matrix\Matrix;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class MatrixFunctions
{
    /**
     * Convert parameter to matrix.
     *
     * @param mixed $matrixValues A matrix of values
     */
    private static function getMatrix($matrixValues) : \ScopedGtbabel\Matrix\Matrix
    {
        $matrixData = [];
        if (!\is_array($matrixValues)) {
            $matrixValues = [[$matrixValues]];
        }
        $row = 0;
        foreach ($matrixValues as $matrixRow) {
            if (!\is_array($matrixRow)) {
                $matrixRow = [$matrixRow];
            }
            $column = 0;
            foreach ($matrixRow as $matrixCell) {
                if (\is_string($matrixCell) || $matrixCell === null) {
                    throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE());
                }
                $matrixData[$row][$column] = $matrixCell;
                ++$column;
            }
            ++$row;
        }
        return new \ScopedGtbabel\Matrix\Matrix($matrixData);
    }
    /**
     * MDETERM.
     *
     * Returns the matrix determinant of an array.
     *
     * Excel Function:
     *        MDETERM(array)
     *
     * @param mixed $matrixValues A matrix of values
     *
     * @return float|string The result, or a string containing an error
     */
    public static function determinant($matrixValues)
    {
        try {
            $matrix = self::getMatrix($matrixValues);
            return $matrix->determinant();
        } catch (\ScopedGtbabel\Matrix\Exception $ex) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
    }
    /**
     * MINVERSE.
     *
     * Returns the inverse matrix for the matrix stored in an array.
     *
     * Excel Function:
     *        MINVERSE(array)
     *
     * @param mixed $matrixValues A matrix of values
     *
     * @return array|string The result, or a string containing an error
     */
    public static function inverse($matrixValues)
    {
        try {
            $matrix = self::getMatrix($matrixValues);
            return $matrix->inverse()->toArray();
        } catch (\ScopedGtbabel\Matrix\Div0Exception $e) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        } catch (\ScopedGtbabel\Matrix\Exception $e) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
    }
    /**
     * MMULT.
     *
     * @param mixed $matrixData1 A matrix of values
     * @param mixed $matrixData2 A matrix of values
     *
     * @return array|string The result, or a string containing an error
     */
    public static function multiply($matrixData1, $matrixData2)
    {
        try {
            $matrixA = self::getMatrix($matrixData1);
            $matrixB = self::getMatrix($matrixData2);
            return $matrixA->multiply($matrixB)->toArray();
        } catch (\ScopedGtbabel\Matrix\Exception $ex) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
    }
    /**
     * MUnit.
     *
     * @param mixed $dimension Number of rows and columns
     *
     * @return array|string The result, or a string containing an error
     */
    public static function identity($dimension)
    {
        try {
            $dimension = (int) \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNumericNullBool($dimension);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validatePositive($dimension, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE());
            $matrix = \ScopedGtbabel\Matrix\Builder::createIdentityMatrix($dimension, 0)->toArray();
            return $matrix;
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
    }
}
