<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Deviations
{
    /**
     * DEVSQ.
     *
     * Returns the sum of squares of deviations of data points from their sample mean.
     *
     * Excel Function:
     *        DEVSQ(value1[,value2[, ...]])
     *
     * @param mixed ...$args Data values
     *
     * @return float|string
     */
    public static function sumSquares(...$args)
    {
        $aArgs = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArrayIndexed($args);
        $aMean = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages::average($aArgs);
        if (!\is_numeric($aMean)) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        // Return value
        $returnValue = 0.0;
        $aCount = -1;
        foreach ($aArgs as $k => $arg) {
            // Is it a numeric value?
            if (\is_bool($arg) && (!\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::isCellValue($k) || \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getCompatibilityMode() == \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::COMPATIBILITY_OPENOFFICE)) {
                $arg = (int) $arg;
            }
            if (\is_numeric($arg) && !\is_string($arg)) {
                $returnValue += ($arg - $aMean) ** 2;
                ++$aCount;
            }
        }
        return $aCount === 0 ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE() : $returnValue;
    }
    /**
     * KURT.
     *
     * Returns the kurtosis of a data set. Kurtosis characterizes the relative peakedness
     * or flatness of a distribution compared with the normal distribution. Positive
     * kurtosis indicates a relatively peaked distribution. Negative kurtosis indicates a
     * relatively flat distribution.
     *
     * @param array ...$args Data Series
     *
     * @return float|string
     */
    public static function kurtosis(...$args)
    {
        $aArgs = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArrayIndexed($args);
        $mean = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages::average($aArgs);
        if (!\is_numeric($mean)) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::DIV0();
        }
        $stdDev = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StandardDeviations::STDEV($aArgs);
        if ($stdDev > 0) {
            $count = $summer = 0;
            foreach ($aArgs as $k => $arg) {
                if (\is_bool($arg) && !\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::isMatrixValue($k)) {
                } else {
                    // Is it a numeric value?
                    if (\is_numeric($arg) && !\is_string($arg)) {
                        $summer += (($arg - $mean) / $stdDev) ** 4;
                        ++$count;
                    }
                }
            }
            if ($count > 3) {
                return $summer * ($count * ($count + 1) / (($count - 1) * ($count - 2) * ($count - 3))) - 3 * ($count - 1) ** 2 / (($count - 2) * ($count - 3));
            }
        }
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::DIV0();
    }
    /**
     * SKEW.
     *
     * Returns the skewness of a distribution. Skewness characterizes the degree of asymmetry
     * of a distribution around its mean. Positive skewness indicates a distribution with an
     * asymmetric tail extending toward more positive values. Negative skewness indicates a
     * distribution with an asymmetric tail extending toward more negative values.
     *
     * @param array ...$args Data Series
     *
     * @return float|int|string The result, or a string containing an error
     */
    public static function skew(...$args)
    {
        $aArgs = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArrayIndexed($args);
        $mean = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages::average($aArgs);
        if (!\is_numeric($mean)) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::DIV0();
        }
        $stdDev = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StandardDeviations::STDEV($aArgs);
        if ($stdDev === 0.0 || \is_string($stdDev)) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::DIV0();
        }
        $count = $summer = 0;
        // Loop through arguments
        foreach ($aArgs as $k => $arg) {
            if (\is_bool($arg) && !\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::isMatrixValue($k)) {
            } elseif (!\is_numeric($arg)) {
                return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            } else {
                // Is it a numeric value?
                if (\is_numeric($arg) && !\is_string($arg)) {
                    $summer += (($arg - $mean) / $stdDev) ** 3;
                    ++$count;
                }
            }
        }
        if ($count > 2) {
            return $summer * ($count / (($count - 1) * ($count - 2)));
        }
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::DIV0();
    }
}
