<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\IntOrFloat;
class Permutations
{
    /**
     * PERMUT.
     *
     * Returns the number of permutations for a given number of objects that can be
     *        selected from number objects. A permutation is any set or subset of objects or
     *        events where internal order is significant. Permutations are different from
     *        combinations, for which the internal order is not significant. Use this function
     *        for lottery-style probability calculations.
     *
     * @param mixed $numObjs Integer number of different objects
     * @param mixed $numInSet Integer number of objects in each permutation
     *
     * @return float|int|string Number of permutations, or a string containing an error
     */
    public static function PERMUT($numObjs, $numInSet)
    {
        $numObjs = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($numObjs);
        $numInSet = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($numInSet);
        try {
            $numObjs = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StatisticalValidations::validateInt($numObjs);
            $numInSet = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StatisticalValidations::validateInt($numInSet);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($numObjs < $numInSet) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        $result = \round(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($numObjs) / \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($numObjs - $numInSet));
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\IntOrFloat::evaluate($result);
    }
    /**
     * PERMUTATIONA.
     *
     * Returns the number of permutations for a given number of objects (with repetitions)
     *     that can be selected from the total objects.
     *
     * @param mixed $numObjs Integer number of different objects
     * @param mixed $numInSet Integer number of objects in each permutation
     *
     * @return float|int|string Number of permutations, or a string containing an error
     */
    public static function PERMUTATIONA($numObjs, $numInSet)
    {
        $numObjs = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($numObjs);
        $numInSet = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($numInSet);
        try {
            $numObjs = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StatisticalValidations::validateInt($numObjs);
            $numInSet = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StatisticalValidations::validateInt($numInSet);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($numObjs < 0 || $numInSet < 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        $result = $numObjs ** $numInSet;
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\IntOrFloat::evaluate($result);
    }
}
