<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData;

use DateTimeInterface;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception as CalcExp;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\NumberFormat;
class Format
{
    /**
     * DOLLAR.
     *
     * This function converts a number to text using currency format, with the decimals rounded to the specified place.
     * The format used is $#,##0.00_);($#,##0.00)..
     *
     * @param mixed $value The value to format
     * @param mixed $decimals The number of digits to display to the right of the decimal point (as an integer).
     *                            If decimals is negative, number is rounded to the left of the decimal point.
     *                            If you omit decimals, it is assumed to be 2
     */
    public static function DOLLAR($value = 0, $decimals = 2) : string
    {
        try {
            $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractFloat($value);
            $decimals = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractInt($decimals, -100, 0, \true);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $mask = '$#,##0';
        if ($decimals > 0) {
            $mask .= '.' . \str_repeat('0', $decimals);
        } else {
            $round = 10 ** \abs($decimals);
            if ($value < 0) {
                $round = 0 - $round;
            }
            $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Round::multiple($value, $round);
        }
        $mask = "{$mask};-{$mask}";
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\NumberFormat::toFormattedString($value, $mask);
    }
    /**
     * FIXED.
     *
     * @param mixed $value The value to format
     * @param mixed $decimals Integer value for the number of decimal places that should be formatted
     * @param mixed $noCommas Boolean value indicating whether the value should have thousands separators or not
     */
    public static function FIXEDFORMAT($value, $decimals = 2, $noCommas = \false) : string
    {
        try {
            $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractFloat($value);
            $decimals = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractInt($decimals, -100, 0, \true);
            $noCommas = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($noCommas);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        $valueResult = \round($value, $decimals);
        if ($decimals < 0) {
            $decimals = 0;
        }
        if ($noCommas === \false) {
            $valueResult = \number_format($valueResult, $decimals, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::getDecimalSeparator(), \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::getThousandsSeparator());
        }
        return (string) $valueResult;
    }
    /**
     * TEXT.
     *
     * @param mixed $value The value to format
     * @param mixed $format A string with the Format mask that should be used
     */
    public static function TEXTFORMAT($value, $format) : string
    {
        $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractString($value);
        $format = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractString($format);
        if (!\is_numeric($value) && \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::isDateTimeFormatCode($format)) {
            $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateValue::fromString($value);
        }
        return (string) \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\NumberFormat::toFormattedString($value, $format);
    }
    /**
     * @param mixed $value Value to check
     *
     * @return mixed
     */
    private static function convertValue($value)
    {
        $value = $value === null ? 0 : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($value);
        if (\is_bool($value)) {
            if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getCompatibilityMode() === \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::COMPATIBILITY_OPENOFFICE) {
                $value = (int) $value;
            } else {
                throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE());
            }
        }
        return $value;
    }
    /**
     * VALUE.
     *
     * @param mixed $value Value to check
     *
     * @return DateTimeInterface|float|int|string A string if arguments are invalid
     */
    public static function VALUE($value = '')
    {
        try {
            $value = self::convertValue($value);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if (!\is_numeric($value)) {
            $numberValue = \str_replace(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::getThousandsSeparator(), '', \trim($value, " \t\n\r\0\v" . \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::getCurrencyCode()));
            if (\is_numeric($numberValue)) {
                return (float) $numberValue;
            }
            $dateSetting = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getReturnDateType();
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::setReturnDateType(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::RETURNDATE_EXCEL);
            if (\strpos($value, ':') !== \false) {
                $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\TimeValue::fromString($value);
                if ($timeValue !== \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE()) {
                    \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::setReturnDateType($dateSetting);
                    return $timeValue;
                }
            }
            $dateValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateValue::fromString($value);
            if ($dateValue !== \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE()) {
                \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::setReturnDateType($dateSetting);
                return $dateValue;
            }
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::setReturnDateType($dateSetting);
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
        }
        return (float) $value;
    }
    /**
     * @param mixed $decimalSeparator
     */
    private static function getDecimalSeparator($decimalSeparator) : string
    {
        $decimalSeparator = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($decimalSeparator);
        return empty($decimalSeparator) ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::getDecimalSeparator() : (string) $decimalSeparator;
    }
    /**
     * @param mixed $groupSeparator
     */
    private static function getGroupSeparator($groupSeparator) : string
    {
        $groupSeparator = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($groupSeparator);
        return empty($groupSeparator) ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::getThousandsSeparator() : (string) $groupSeparator;
    }
    /**
     * NUMBERVALUE.
     *
     * @param mixed $value The value to format
     * @param mixed $decimalSeparator A string with the decimal separator to use, defaults to locale defined value
     * @param mixed $groupSeparator A string with the group/thousands separator to use, defaults to locale defined value
     *
     * @return float|string
     */
    public static function NUMBERVALUE($value = '', $decimalSeparator = null, $groupSeparator = null)
    {
        try {
            $value = self::convertValue($value);
            $decimalSeparator = self::getDecimalSeparator($decimalSeparator);
            $groupSeparator = self::getGroupSeparator($groupSeparator);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if (!\is_numeric($value)) {
            $decimalPositions = \preg_match_all('/' . \preg_quote($decimalSeparator) . '/', $value, $matches, \PREG_OFFSET_CAPTURE);
            if ($decimalPositions > 1) {
                return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            }
            $decimalOffset = \array_pop($matches[0])[1];
            if (\strpos($value, $groupSeparator, $decimalOffset) !== \false) {
                return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            }
            $value = \str_replace([$groupSeparator, $decimalSeparator], ['', '.'], $value);
            // Handle the special case of trailing % signs
            $percentageString = \rtrim($value, '%');
            if (!\is_numeric($percentageString)) {
                return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            }
            $percentageAdjustment = \strlen($value) - \strlen($percentageString);
            if ($percentageAdjustment) {
                $value = (float) $percentageString;
                $value /= 10 ** ($percentageAdjustment * 2);
            }
        }
        return \is_array($value) ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE() : (float) $value;
    }
}
