<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\IReader;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\File;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\IWriter;
/**
 * Factory to create readers and writers easily.
 *
 * It is not required to use this class, but it should make it easier to read and write files.
 * Especially for reading files with an unknown format.
 */
abstract class IOFactory
{
    private static $readers = ['Xlsx' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx::class, 'Xls' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xls::class, 'Xml' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xml::class, 'Ods' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Ods::class, 'Slk' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Slk::class, 'Gnumeric' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Gnumeric::class, 'Html' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Html::class, 'Csv' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Csv::class];
    private static $writers = ['Xls' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xls::class, 'Xlsx' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx::class, 'Ods' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods::class, 'Csv' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Csv::class, 'Html' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Html::class, 'Tcpdf' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Pdf\Tcpdf::class, 'Dompdf' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Pdf\Dompdf::class, 'Mpdf' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Pdf\Mpdf::class];
    /**
     * Create Writer\IWriter.
     */
    public static function createWriter(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet $spreadsheet, string $writerType) : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\IWriter
    {
        if (!isset(self::$writers[$writerType])) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Exception("No writer found for type {$writerType}");
        }
        // Instantiate writer
        $className = self::$writers[$writerType];
        return new $className($spreadsheet);
    }
    /**
     * Create IReader.
     */
    public static function createReader(string $readerType) : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\IReader
    {
        if (!isset(self::$readers[$readerType])) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Exception("No reader found for type {$readerType}");
        }
        // Instantiate reader
        $className = self::$readers[$readerType];
        return new $className();
    }
    /**
     * Loads Spreadsheet from file using automatic Reader\IReader resolution.
     *
     * @param string $filename The name of the spreadsheet file
     */
    public static function load(string $filename, int $flags = 0) : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet
    {
        $reader = self::createReaderForFile($filename);
        return $reader->load($filename, $flags);
    }
    /**
     * Identify file type using automatic IReader resolution.
     */
    public static function identify(string $filename) : string
    {
        $reader = self::createReaderForFile($filename);
        $className = \get_class($reader);
        $classType = \explode('\\', $className);
        unset($reader);
        return \array_pop($classType);
    }
    /**
     * Create Reader\IReader for file using automatic IReader resolution.
     */
    public static function createReaderForFile(string $filename) : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\IReader
    {
        \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\File::assertFile($filename);
        // First, lucky guess by inspecting file extension
        $guessedReader = self::getReaderTypeFromExtension($filename);
        if ($guessedReader !== null) {
            $reader = self::createReader($guessedReader);
            // Let's see if we are lucky
            if ($reader->canRead($filename)) {
                return $reader;
            }
        }
        // If we reach here then "lucky guess" didn't give any result
        // Try walking through all the options in self::$autoResolveClasses
        foreach (self::$readers as $type => $class) {
            //    Ignore our original guess, we know that won't work
            if ($type !== $guessedReader) {
                $reader = self::createReader($type);
                if ($reader->canRead($filename)) {
                    return $reader;
                }
            }
        }
        throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Exception('Unable to identify a reader for this file');
    }
    /**
     * Guess a reader type from the file extension, if any.
     */
    private static function getReaderTypeFromExtension(string $filename) : ?string
    {
        $pathinfo = \pathinfo($filename);
        if (!isset($pathinfo['extension'])) {
            return null;
        }
        switch (\strtolower($pathinfo['extension'])) {
            case 'xlsx':
            // Excel (OfficeOpenXML) Spreadsheet
            case 'xlsm':
            // Excel (OfficeOpenXML) Macro Spreadsheet (macros will be discarded)
            case 'xltx':
            // Excel (OfficeOpenXML) Template
            case 'xltm':
                // Excel (OfficeOpenXML) Macro Template (macros will be discarded)
                return 'Xlsx';
            case 'xls':
            // Excel (BIFF) Spreadsheet
            case 'xlt':
                // Excel (BIFF) Template
                return 'Xls';
            case 'ods':
            // Open/Libre Offic Calc
            case 'ots':
                // Open/Libre Offic Calc Template
                return 'Ods';
            case 'slk':
                return 'Slk';
            case 'xml':
                // Excel 2003 SpreadSheetML
                return 'Xml';
            case 'gnumeric':
                return 'Gnumeric';
            case 'htm':
            case 'html':
                return 'Html';
            case 'csv':
                // Do nothing
                // We must not try to use CSV reader since it loads
                // all files including Excel files etc.
                return null;
            default:
                return null;
        }
    }
    /**
     * Register a writer with its type and class name.
     */
    public static function registerWriter(string $writerType, string $writerClass) : void
    {
        if (!\is_a($writerClass, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\IWriter::class, \true)) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Exception('Registered writers must implement ' . \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\IWriter::class);
        }
        self::$writers[$writerType] = $writerClass;
    }
    /**
     * Register a reader with its type and class name.
     */
    public static function registerReader(string $readerType, string $readerClass) : void
    {
        if (!\is_a($readerClass, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\IReader::class, \true)) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Exception('Registered readers must implement ' . \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\IReader::class);
        }
        self::$readers[$readerType] = $readerClass;
    }
}
