<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engine\CyclicReferenceStack;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engine\Logger;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Token\Stack;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\DefinedName;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\ReferenceHelper;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use ReflectionClassConstant;
use ReflectionMethod;
use ReflectionParameter;
class Calculation
{
    /** Constants                */
    /** Regular Expressions        */
    //    Numeric operand
    const CALCULATION_REGEXP_NUMBER = '[-+]?\\d*\\.?\\d+(e[-+]?\\d+)?';
    //    String operand
    const CALCULATION_REGEXP_STRING = '"(?:[^"]|"")*"';
    //    Opening bracket
    const CALCULATION_REGEXP_OPENBRACE = '\\(';
    //    Function (allow for the old @ symbol that could be used to prefix a function, but we'll ignore it)
    const CALCULATION_REGEXP_FUNCTION = '@?(?:_xlfn\\.)?([\\p{L}][\\p{L}\\p{N}\\.]*)[\\s]*\\(';
    //    Cell reference (cell or range of cells, with or without a sheet reference)
    const CALCULATION_REGEXP_CELLREF = '((([^\\s,!&%^\\/\\*\\+<>=-]*)|(\'[^\']*\')|(\\"[^\\"]*\\"))!)?\\$?\\b([a-z]{1,3})\\$?(\\d{1,7})(?![\\w.])';
    //    Cell reference (with or without a sheet reference) ensuring absolute/relative
    const CALCULATION_REGEXP_CELLREF_RELATIVE = '((([^\\s\\(,!&%^\\/\\*\\+<>=-]*)|(\'[^\']*\')|(\\"[^\\"]*\\"))!)?(\\$?\\b[a-z]{1,3})(\\$?\\d{1,7})(?![\\w.])';
    const CALCULATION_REGEXP_COLUMN_RANGE = '(((([^\\s\\(,!&%^\\/\\*\\+<>=-]*)|(\'[^\']*\')|(\\"[^\\"]*\\"))!)?(\\$?[a-z]{1,3})):(?![.*])';
    const CALCULATION_REGEXP_ROW_RANGE = '(((([^\\s\\(,!&%^\\/\\*\\+<>=-]*)|(\'[^\']*\')|(\\"[^\\"]*\\"))!)?(\\$?[1-9][0-9]{0,6})):(?![.*])';
    //    Cell reference (with or without a sheet reference) ensuring absolute/relative
    //    Cell ranges ensuring absolute/relative
    const CALCULATION_REGEXP_COLUMNRANGE_RELATIVE = '(\\$?[a-z]{1,3}):(\\$?[a-z]{1,3})';
    const CALCULATION_REGEXP_ROWRANGE_RELATIVE = '(\\$?\\d{1,7}):(\\$?\\d{1,7})';
    //    Defined Names: Named Range of cells, or Named Formulae
    const CALCULATION_REGEXP_DEFINEDNAME = '((([^\\s,!&%^\\/\\*\\+<>=-]*)|(\'[^\']*\')|(\\"[^\\"]*\\"))!)?([_\\p{L}][_\\p{L}\\p{N}\\.]*)';
    //    Error
    const CALCULATION_REGEXP_ERROR = '\\#[A-Z][A-Z0_\\/]*[!\\?]?';
    /** constants */
    const RETURN_ARRAY_AS_ERROR = 'error';
    const RETURN_ARRAY_AS_VALUE = 'value';
    const RETURN_ARRAY_AS_ARRAY = 'array';
    const FORMULA_OPEN_FUNCTION_BRACE = '{';
    const FORMULA_CLOSE_FUNCTION_BRACE = '}';
    const FORMULA_STRING_QUOTE = '"';
    private static $returnArrayAsType = self::RETURN_ARRAY_AS_VALUE;
    /**
     * Instance of this class.
     *
     * @var Calculation
     */
    private static $instance;
    /**
     * Instance of the spreadsheet this Calculation Engine is using.
     *
     * @var Spreadsheet
     */
    private $spreadsheet;
    /**
     * Calculation cache.
     *
     * @var array
     */
    private $calculationCache = [];
    /**
     * Calculation cache enabled.
     *
     * @var bool
     */
    private $calculationCacheEnabled = \true;
    /**
     * Used to generate unique store keys.
     *
     * @var int
     */
    private $branchStoreKeyCounter = 0;
    private $branchPruningEnabled = \true;
    /**
     * List of operators that can be used within formulae
     * The true/false value indicates whether it is a binary operator or a unary operator.
     *
     * @var array
     */
    private static $operators = ['+' => \true, '-' => \true, '*' => \true, '/' => \true, '^' => \true, '&' => \true, '%' => \false, '~' => \false, '>' => \true, '<' => \true, '=' => \true, '>=' => \true, '<=' => \true, '<>' => \true, '|' => \true, ':' => \true];
    /**
     * List of binary operators (those that expect two operands).
     *
     * @var array
     */
    private static $binaryOperators = ['+' => \true, '-' => \true, '*' => \true, '/' => \true, '^' => \true, '&' => \true, '>' => \true, '<' => \true, '=' => \true, '>=' => \true, '<=' => \true, '<>' => \true, '|' => \true, ':' => \true];
    /**
     * The debug log generated by the calculation engine.
     *
     * @var Logger
     */
    private $debugLog;
    /**
     * Flag to determine how formula errors should be handled
     *        If true, then a user error will be triggered
     *        If false, then an exception will be thrown.
     *
     * @var bool
     */
    public $suppressFormulaErrors = \false;
    /**
     * Error message for any error that was raised/thrown by the calculation engine.
     *
     * @var null|string
     */
    public $formulaError;
    /**
     * Reference Helper.
     *
     * @var ReferenceHelper
     */
    private static $referenceHelper;
    /**
     * An array of the nested cell references accessed by the calculation engine, used for the debug log.
     *
     * @var CyclicReferenceStack
     */
    private $cyclicReferenceStack;
    private $cellStack = [];
    /**
     * Current iteration counter for cyclic formulae
     * If the value is 0 (or less) then cyclic formulae will throw an exception,
     * otherwise they will iterate to the limit defined here before returning a result.
     *
     * @var int
     */
    private $cyclicFormulaCounter = 1;
    private $cyclicFormulaCell = '';
    /**
     * Number of iterations for cyclic formulae.
     *
     * @var int
     */
    public $cyclicFormulaCount = 1;
    /**
     * Epsilon Precision used for comparisons in calculations.
     *
     * @var float
     */
    private $delta = 1.0E-13;
    /**
     * The current locale setting.
     *
     * @var string
     */
    private static $localeLanguage = 'en_us';
    //    US English    (default locale)
    /**
     * List of available locale settings
     * Note that this is read for the locale subdirectory only when requested.
     *
     * @var string[]
     */
    private static $validLocaleLanguages = ['en'];
    /**
     * Locale-specific argument separator for function arguments.
     *
     * @var string
     */
    private static $localeArgumentSeparator = ',';
    private static $localeFunctions = [];
    /**
     * Locale-specific translations for Excel constants (True, False and Null).
     *
     * @var array<string, string>
     */
    public static $localeBoolean = ['TRUE' => 'TRUE', 'FALSE' => 'FALSE', 'NULL' => 'NULL'];
    /**
     * Excel constant string translations to their PHP equivalents
     * Constant conversion from text name/value to actual (datatyped) value.
     *
     * @var array<string, mixed>
     */
    private static $excelConstants = ['TRUE' => \true, 'FALSE' => \false, 'NULL' => null];
    // PhpSpreadsheet functions
    private static $phpSpreadsheetFunctions = ['ABS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Absolute::class, 'evaluate'], 'argumentCount' => '1'], 'ACCRINT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\AccruedInterest::class, 'periodic'], 'argumentCount' => '4-8'], 'ACCRINTM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\AccruedInterest::class, 'atMaturity'], 'argumentCount' => '3-5'], 'ACOS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cosine::class, 'acos'], 'argumentCount' => '1'], 'ACOSH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cosine::class, 'acosh'], 'argumentCount' => '1'], 'ACOT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cotangent::class, 'acot'], 'argumentCount' => '1'], 'ACOTH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cotangent::class, 'acoth'], 'argumentCount' => '1'], 'ADDRESS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\Address::class, 'cell'], 'argumentCount' => '2-5'], 'AGGREGATE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3+'], 'AMORDEGRC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Amortization::class, 'AMORDEGRC'], 'argumentCount' => '6,7'], 'AMORLINC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Amortization::class, 'AMORLINC'], 'argumentCount' => '6,7'], 'AND' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Operations::class, 'logicalAnd'], 'argumentCount' => '1+'], 'ARABIC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Arabic::class, 'evaluate'], 'argumentCount' => '1'], 'AREAS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1'], 'ARRAYTOTEXT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'ASC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1'], 'ASIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Sine::class, 'asin'], 'argumentCount' => '1'], 'ASINH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Sine::class, 'asinh'], 'argumentCount' => '1'], 'ATAN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Tangent::class, 'atan'], 'argumentCount' => '1'], 'ATAN2' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Tangent::class, 'atan2'], 'argumentCount' => '2'], 'ATANH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Tangent::class, 'atanh'], 'argumentCount' => '1'], 'AVEDEV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages::class, 'averageDeviations'], 'argumentCount' => '1+'], 'AVERAGE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages::class, 'average'], 'argumentCount' => '1+'], 'AVERAGEA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages::class, 'averageA'], 'argumentCount' => '1+'], 'AVERAGEIF' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Conditional::class, 'AVERAGEIF'], 'argumentCount' => '2,3'], 'AVERAGEIFS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Conditional::class, 'AVERAGEIFS'], 'argumentCount' => '3+'], 'BAHTTEXT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1'], 'BASE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Base::class, 'evaluate'], 'argumentCount' => '2,3'], 'BESSELI' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\BesselI::class, 'BESSELI'], 'argumentCount' => '2'], 'BESSELJ' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\BesselJ::class, 'BESSELJ'], 'argumentCount' => '2'], 'BESSELK' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\BesselK::class, 'BESSELK'], 'argumentCount' => '2'], 'BESSELY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\BesselY::class, 'BESSELY'], 'argumentCount' => '2'], 'BETADIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Beta::class, 'distribution'], 'argumentCount' => '3-5'], 'BETA.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '4-6'], 'BETAINV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Beta::class, 'inverse'], 'argumentCount' => '3-5'], 'BETA.INV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Beta::class, 'inverse'], 'argumentCount' => '3-5'], 'BIN2DEC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertBinary::class, 'toDecimal'], 'argumentCount' => '1'], 'BIN2HEX' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertBinary::class, 'toHex'], 'argumentCount' => '1,2'], 'BIN2OCT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertBinary::class, 'toOctal'], 'argumentCount' => '1,2'], 'BINOMDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Binomial::class, 'distribution'], 'argumentCount' => '4'], 'BINOM.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Binomial::class, 'distribution'], 'argumentCount' => '4'], 'BINOM.DIST.RANGE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Binomial::class, 'range'], 'argumentCount' => '3,4'], 'BINOM.INV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Binomial::class, 'inverse'], 'argumentCount' => '3'], 'BITAND' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\BitWise::class, 'BITAND'], 'argumentCount' => '2'], 'BITOR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\BitWise::class, 'BITOR'], 'argumentCount' => '2'], 'BITXOR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\BitWise::class, 'BITXOR'], 'argumentCount' => '2'], 'BITLSHIFT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\BitWise::class, 'BITLSHIFT'], 'argumentCount' => '2'], 'BITRSHIFT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\BitWise::class, 'BITRSHIFT'], 'argumentCount' => '2'], 'CEILING' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Ceiling::class, 'ceiling'], 'argumentCount' => '1-2'], 'CEILING.MATH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Ceiling::class, 'math'], 'argumentCount' => '1-3'], 'CEILING.PRECISE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Ceiling::class, 'precise'], 'argumentCount' => '1,2'], 'CELL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1,2'], 'CHAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\CharacterConvert::class, 'character'], 'argumentCount' => '1'], 'CHIDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\ChiSquared::class, 'distributionRightTail'], 'argumentCount' => '2'], 'CHISQ.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\ChiSquared::class, 'distributionLeftTail'], 'argumentCount' => '3'], 'CHISQ.DIST.RT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\ChiSquared::class, 'distributionRightTail'], 'argumentCount' => '2'], 'CHIINV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\ChiSquared::class, 'inverseRightTail'], 'argumentCount' => '2'], 'CHISQ.INV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\ChiSquared::class, 'inverseLeftTail'], 'argumentCount' => '2'], 'CHISQ.INV.RT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\ChiSquared::class, 'inverseRightTail'], 'argumentCount' => '2'], 'CHITEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\ChiSquared::class, 'test'], 'argumentCount' => '2'], 'CHISQ.TEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\ChiSquared::class, 'test'], 'argumentCount' => '2'], 'CHOOSE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\Selection::class, 'CHOOSE'], 'argumentCount' => '2+'], 'CLEAN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Trim::class, 'nonPrintable'], 'argumentCount' => '1'], 'CODE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\CharacterConvert::class, 'code'], 'argumentCount' => '1'], 'COLUMN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\RowColumnInformation::class, 'COLUMN'], 'argumentCount' => '-1', 'passCellReference' => \true, 'passByReference' => [\true]], 'COLUMNS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\RowColumnInformation::class, 'COLUMNS'], 'argumentCount' => '1'], 'COMBIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Combinations::class, 'withoutRepetition'], 'argumentCount' => '2'], 'COMBINA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Combinations::class, 'withRepetition'], 'argumentCount' => '2'], 'COMPLEX' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\Complex::class, 'COMPLEX'], 'argumentCount' => '2,3'], 'CONCAT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Concatenate::class, 'CONCATENATE'], 'argumentCount' => '1+'], 'CONCATENATE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Concatenate::class, 'CONCATENATE'], 'argumentCount' => '1+'], 'CONFIDENCE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Confidence::class, 'CONFIDENCE'], 'argumentCount' => '3'], 'CONFIDENCE.NORM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Confidence::class, 'CONFIDENCE'], 'argumentCount' => '3'], 'CONFIDENCE.T' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3'], 'CONVERT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertUOM::class, 'CONVERT'], 'argumentCount' => '3'], 'CORREL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'CORREL'], 'argumentCount' => '2'], 'COS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cosine::class, 'cos'], 'argumentCount' => '1'], 'COSH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cosine::class, 'cosh'], 'argumentCount' => '1'], 'COT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cotangent::class, 'cot'], 'argumentCount' => '1'], 'COTH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cotangent::class, 'coth'], 'argumentCount' => '1'], 'COUNT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Counts::class, 'COUNT'], 'argumentCount' => '1+'], 'COUNTA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Counts::class, 'COUNTA'], 'argumentCount' => '1+'], 'COUNTBLANK' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Counts::class, 'COUNTBLANK'], 'argumentCount' => '1'], 'COUNTIF' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Conditional::class, 'COUNTIF'], 'argumentCount' => '2'], 'COUNTIFS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Conditional::class, 'COUNTIFS'], 'argumentCount' => '2+'], 'COUPDAYBS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::class, 'COUPDAYBS'], 'argumentCount' => '3,4'], 'COUPDAYS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::class, 'COUPDAYS'], 'argumentCount' => '3,4'], 'COUPDAYSNC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::class, 'COUPDAYSNC'], 'argumentCount' => '3,4'], 'COUPNCD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::class, 'COUPNCD'], 'argumentCount' => '3,4'], 'COUPNUM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::class, 'COUPNUM'], 'argumentCount' => '3,4'], 'COUPPCD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Coupons::class, 'COUPPCD'], 'argumentCount' => '3,4'], 'COVAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'COVAR'], 'argumentCount' => '2'], 'COVARIANCE.P' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'COVAR'], 'argumentCount' => '2'], 'COVARIANCE.S' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'CRITBINOM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Binomial::class, 'inverse'], 'argumentCount' => '3'], 'CSC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cosecant::class, 'csc'], 'argumentCount' => '1'], 'CSCH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Cosecant::class, 'csch'], 'argumentCount' => '1'], 'CUBEKPIMEMBER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_CUBE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'CUBEMEMBER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_CUBE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'CUBEMEMBERPROPERTY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_CUBE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'CUBERANKEDMEMBER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_CUBE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'CUBESET' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_CUBE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'CUBESETCOUNT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_CUBE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'CUBEVALUE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_CUBE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'CUMIPMT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\Cumulative::class, 'interest'], 'argumentCount' => '6'], 'CUMPRINC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\Cumulative::class, 'principal'], 'argumentCount' => '6'], 'DATE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Date::class, 'fromYMD'], 'argumentCount' => '3'], 'DATEDIF' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Difference::class, 'interval'], 'argumentCount' => '2,3'], 'DATESTRING' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'DATEVALUE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateValue::class, 'fromString'], 'argumentCount' => '1'], 'DAVERAGE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DAverage::class, 'evaluate'], 'argumentCount' => '3'], 'DAY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateParts::class, 'day'], 'argumentCount' => '1'], 'DAYS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Days::class, 'between'], 'argumentCount' => '2'], 'DAYS360' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Days360::class, 'between'], 'argumentCount' => '2,3'], 'DB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Depreciation::class, 'DB'], 'argumentCount' => '4,5'], 'DBCS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1'], 'DCOUNT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DCount::class, 'evaluate'], 'argumentCount' => '3'], 'DCOUNTA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DCountA::class, 'evaluate'], 'argumentCount' => '3'], 'DDB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Depreciation::class, 'DDB'], 'argumentCount' => '4,5'], 'DEC2BIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertDecimal::class, 'toBinary'], 'argumentCount' => '1,2'], 'DEC2HEX' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertDecimal::class, 'toHex'], 'argumentCount' => '1,2'], 'DEC2OCT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertDecimal::class, 'toOctal'], 'argumentCount' => '1,2'], 'DECIMAL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'DEGREES' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Angle::class, 'toDegrees'], 'argumentCount' => '1'], 'DELTA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\Compare::class, 'DELTA'], 'argumentCount' => '1,2'], 'DEVSQ' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Deviations::class, 'sumSquares'], 'argumentCount' => '1+'], 'DGET' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DGet::class, 'evaluate'], 'argumentCount' => '3'], 'DISC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\Rates::class, 'discount'], 'argumentCount' => '4,5'], 'DMAX' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DMax::class, 'evaluate'], 'argumentCount' => '3'], 'DMIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DMin::class, 'evaluate'], 'argumentCount' => '3'], 'DOLLAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Format::class, 'DOLLAR'], 'argumentCount' => '1,2'], 'DOLLARDE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Dollar::class, 'decimal'], 'argumentCount' => '2'], 'DOLLARFR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Dollar::class, 'fractional'], 'argumentCount' => '2'], 'DPRODUCT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DProduct::class, 'evaluate'], 'argumentCount' => '3'], 'DSTDEV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DStDev::class, 'evaluate'], 'argumentCount' => '3'], 'DSTDEVP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DStDevP::class, 'evaluate'], 'argumentCount' => '3'], 'DSUM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DSum::class, 'evaluate'], 'argumentCount' => '3'], 'DURATION' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '5,6'], 'DVAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DVar::class, 'evaluate'], 'argumentCount' => '3'], 'DVARP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATABASE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Database\DVarP::class, 'evaluate'], 'argumentCount' => '3'], 'ECMA.CEILING' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1,2'], 'EDATE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Month::class, 'adjust'], 'argumentCount' => '2'], 'EFFECT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\InterestRate::class, 'effective'], 'argumentCount' => '2'], 'ENCODEURL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_WEB, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Web\Service::class, 'urlEncode'], 'argumentCount' => '1'], 'EOMONTH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Month::class, 'lastDay'], 'argumentCount' => '2'], 'ERF' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\Erf::class, 'ERF'], 'argumentCount' => '1,2'], 'ERF.PRECISE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\Erf::class, 'ERFPRECISE'], 'argumentCount' => '1'], 'ERFC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ErfC::class, 'ERFC'], 'argumentCount' => '1'], 'ERFC.PRECISE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ErfC::class, 'ERFC'], 'argumentCount' => '1'], 'ERROR.TYPE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'errorType'], 'argumentCount' => '1'], 'EVEN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Round::class, 'even'], 'argumentCount' => '1'], 'EXACT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Text::class, 'exact'], 'argumentCount' => '2'], 'EXP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Exp::class, 'evaluate'], 'argumentCount' => '1'], 'EXPONDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Exponential::class, 'distribution'], 'argumentCount' => '3'], 'EXPON.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Exponential::class, 'distribution'], 'argumentCount' => '3'], 'FACT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::class, 'fact'], 'argumentCount' => '1'], 'FACTDOUBLE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::class, 'factDouble'], 'argumentCount' => '1'], 'FALSE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Boolean::class, 'FALSE'], 'argumentCount' => '0'], 'FDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3'], 'F.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\F::class, 'distribution'], 'argumentCount' => '4'], 'F.DIST.RT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3'], 'FILTER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3+'], 'FILTERXML' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_WEB, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'FIND' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Search::class, 'sensitive'], 'argumentCount' => '2,3'], 'FINDB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Search::class, 'sensitive'], 'argumentCount' => '2,3'], 'FINV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3'], 'F.INV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3'], 'F.INV.RT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3'], 'FISHER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Fisher::class, 'distribution'], 'argumentCount' => '1'], 'FISHERINV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Fisher::class, 'inverse'], 'argumentCount' => '1'], 'FIXED' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Format::class, 'FIXEDFORMAT'], 'argumentCount' => '1-3'], 'FLOOR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Floor::class, 'floor'], 'argumentCount' => '1-2'], 'FLOOR.MATH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Floor::class, 'math'], 'argumentCount' => '1-3'], 'FLOOR.PRECISE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Floor::class, 'precise'], 'argumentCount' => '1-2'], 'FORECAST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'FORECAST'], 'argumentCount' => '3'], 'FORECAST.ETS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3-6'], 'FORECAST.ETS.CONFINT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3-6'], 'FORECAST.ETS.SEASONALITY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2-4'], 'FORECAST.ETS.STAT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3-6'], 'FORECAST.LINEAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'FORECAST'], 'argumentCount' => '3'], 'FORMULATEXT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\Formula::class, 'text'], 'argumentCount' => '1', 'passCellReference' => \true, 'passByReference' => [\true]], 'FREQUENCY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'FTEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'F.TEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'FV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic::class, 'futureValue'], 'argumentCount' => '3-5'], 'FVSCHEDULE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Single::class, 'futureValue'], 'argumentCount' => '2'], 'GAMMA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Gamma::class, 'gamma'], 'argumentCount' => '1'], 'GAMMADIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Gamma::class, 'distribution'], 'argumentCount' => '4'], 'GAMMA.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Gamma::class, 'distribution'], 'argumentCount' => '4'], 'GAMMAINV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Gamma::class, 'inverse'], 'argumentCount' => '3'], 'GAMMA.INV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Gamma::class, 'inverse'], 'argumentCount' => '3'], 'GAMMALN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Gamma::class, 'ln'], 'argumentCount' => '1'], 'GAMMALN.PRECISE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Gamma::class, 'ln'], 'argumentCount' => '1'], 'GAUSS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StandardNormal::class, 'gauss'], 'argumentCount' => '1'], 'GCD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Gcd::class, 'evaluate'], 'argumentCount' => '1+'], 'GEOMEAN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages\Mean::class, 'geometric'], 'argumentCount' => '1+'], 'GESTEP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\Compare::class, 'GESTEP'], 'argumentCount' => '1,2'], 'GETPIVOTDATA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2+'], 'GROWTH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'GROWTH'], 'argumentCount' => '1-4'], 'HARMEAN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages\Mean::class, 'harmonic'], 'argumentCount' => '1+'], 'HEX2BIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertHex::class, 'toBinary'], 'argumentCount' => '1,2'], 'HEX2DEC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertHex::class, 'toDecimal'], 'argumentCount' => '1'], 'HEX2OCT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertHex::class, 'toOctal'], 'argumentCount' => '1,2'], 'HLOOKUP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\HLookup::class, 'lookup'], 'argumentCount' => '3,4'], 'HOUR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\TimeParts::class, 'hour'], 'argumentCount' => '1'], 'HYPERLINK' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\Hyperlink::class, 'set'], 'argumentCount' => '1,2', 'passCellReference' => \true], 'HYPGEOMDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\HyperGeometric::class, 'distribution'], 'argumentCount' => '4'], 'HYPGEOM.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '5'], 'IF' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Conditional::class, 'statementIf'], 'argumentCount' => '1-3'], 'IFERROR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Conditional::class, 'IFERROR'], 'argumentCount' => '2'], 'IFNA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Conditional::class, 'IFNA'], 'argumentCount' => '2'], 'IFS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Conditional::class, 'IFS'], 'argumentCount' => '2+'], 'IMABS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMABS'], 'argumentCount' => '1'], 'IMAGINARY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\Complex::class, 'IMAGINARY'], 'argumentCount' => '1'], 'IMARGUMENT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMARGUMENT'], 'argumentCount' => '1'], 'IMCONJUGATE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMCONJUGATE'], 'argumentCount' => '1'], 'IMCOS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMCOS'], 'argumentCount' => '1'], 'IMCOSH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMCOSH'], 'argumentCount' => '1'], 'IMCOT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMCOT'], 'argumentCount' => '1'], 'IMCSC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMCSC'], 'argumentCount' => '1'], 'IMCSCH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMCSCH'], 'argumentCount' => '1'], 'IMDIV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexOperations::class, 'IMDIV'], 'argumentCount' => '2'], 'IMEXP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMEXP'], 'argumentCount' => '1'], 'IMLN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMLN'], 'argumentCount' => '1'], 'IMLOG10' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMLOG10'], 'argumentCount' => '1'], 'IMLOG2' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMLOG2'], 'argumentCount' => '1'], 'IMPOWER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMPOWER'], 'argumentCount' => '2'], 'IMPRODUCT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexOperations::class, 'IMPRODUCT'], 'argumentCount' => '1+'], 'IMREAL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\Complex::class, 'IMREAL'], 'argumentCount' => '1'], 'IMSEC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMSEC'], 'argumentCount' => '1'], 'IMSECH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMSECH'], 'argumentCount' => '1'], 'IMSIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMSIN'], 'argumentCount' => '1'], 'IMSINH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMSINH'], 'argumentCount' => '1'], 'IMSQRT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMSQRT'], 'argumentCount' => '1'], 'IMSUB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexOperations::class, 'IMSUB'], 'argumentCount' => '2'], 'IMSUM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexOperations::class, 'IMSUM'], 'argumentCount' => '1+'], 'IMTAN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ComplexFunctions::class, 'IMTAN'], 'argumentCount' => '1'], 'INDEX' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\Matrix::class, 'index'], 'argumentCount' => '1-4'], 'INDIRECT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\Indirect::class, 'INDIRECT'], 'argumentCount' => '1,2', 'passCellReference' => \true], 'INFO' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1'], 'INT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\IntClass::class, 'evaluate'], 'argumentCount' => '1'], 'INTERCEPT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'INTERCEPT'], 'argumentCount' => '2'], 'INTRATE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\Rates::class, 'interest'], 'argumentCount' => '4,5'], 'IPMT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\Interest::class, 'payment'], 'argumentCount' => '4-6'], 'IRR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Variable\Periodic::class, 'rate'], 'argumentCount' => '1,2'], 'ISBLANK' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isBlank'], 'argumentCount' => '1'], 'ISERR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isErr'], 'argumentCount' => '1'], 'ISERROR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isError'], 'argumentCount' => '1'], 'ISEVEN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isEven'], 'argumentCount' => '1'], 'ISFORMULA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isFormula'], 'argumentCount' => '1', 'passCellReference' => \true, 'passByReference' => [\true]], 'ISLOGICAL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isLogical'], 'argumentCount' => '1'], 'ISNA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isNa'], 'argumentCount' => '1'], 'ISNONTEXT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isNonText'], 'argumentCount' => '1'], 'ISNUMBER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isNumber'], 'argumentCount' => '1'], 'ISO.CEILING' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1,2'], 'ISODD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isOdd'], 'argumentCount' => '1'], 'ISOWEEKNUM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Week::class, 'isoWeekNumber'], 'argumentCount' => '1'], 'ISPMT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\Interest::class, 'schedulePayment'], 'argumentCount' => '4'], 'ISREF' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1'], 'ISTEXT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'isText'], 'argumentCount' => '1'], 'ISTHAIDIGIT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'JIS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1'], 'KURT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Deviations::class, 'kurtosis'], 'argumentCount' => '1+'], 'LARGE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Size::class, 'large'], 'argumentCount' => '2'], 'LCM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Lcm::class, 'evaluate'], 'argumentCount' => '1+'], 'LEFT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Extract::class, 'left'], 'argumentCount' => '1,2'], 'LEFTB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Extract::class, 'left'], 'argumentCount' => '1,2'], 'LEN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Text::class, 'length'], 'argumentCount' => '1'], 'LENB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Text::class, 'length'], 'argumentCount' => '1'], 'LINEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'LINEST'], 'argumentCount' => '1-4'], 'LN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Logarithms::class, 'natural'], 'argumentCount' => '1'], 'LOG' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Logarithms::class, 'withBase'], 'argumentCount' => '1,2'], 'LOG10' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Logarithms::class, 'base10'], 'argumentCount' => '1'], 'LOGEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'LOGEST'], 'argumentCount' => '1-4'], 'LOGINV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\LogNormal::class, 'inverse'], 'argumentCount' => '3'], 'LOGNORMDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\LogNormal::class, 'cumulative'], 'argumentCount' => '3'], 'LOGNORM.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\LogNormal::class, 'distribution'], 'argumentCount' => '4'], 'LOGNORM.INV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\LogNormal::class, 'inverse'], 'argumentCount' => '3'], 'LOOKUP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\Lookup::class, 'lookup'], 'argumentCount' => '2,3'], 'LOWER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\CaseConvert::class, 'lower'], 'argumentCount' => '1'], 'MATCH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\ExcelMatch::class, 'MATCH'], 'argumentCount' => '2,3'], 'MAX' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Maximum::class, 'max'], 'argumentCount' => '1+'], 'MAXA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Maximum::class, 'maxA'], 'argumentCount' => '1+'], 'MAXIFS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Conditional::class, 'MAXIFS'], 'argumentCount' => '3+'], 'MDETERM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\MatrixFunctions::class, 'determinant'], 'argumentCount' => '1'], 'MDURATION' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '5,6'], 'MEDIAN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages::class, 'median'], 'argumentCount' => '1+'], 'MEDIANIF' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2+'], 'MID' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Extract::class, 'mid'], 'argumentCount' => '3'], 'MIDB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Extract::class, 'mid'], 'argumentCount' => '3'], 'MIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Minimum::class, 'min'], 'argumentCount' => '1+'], 'MINA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Minimum::class, 'minA'], 'argumentCount' => '1+'], 'MINIFS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Conditional::class, 'MINIFS'], 'argumentCount' => '3+'], 'MINUTE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\TimeParts::class, 'minute'], 'argumentCount' => '1'], 'MINVERSE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\MatrixFunctions::class, 'inverse'], 'argumentCount' => '1'], 'MIRR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Variable\Periodic::class, 'modifiedRate'], 'argumentCount' => '3'], 'MMULT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\MatrixFunctions::class, 'multiply'], 'argumentCount' => '2'], 'MOD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Operations::class, 'mod'], 'argumentCount' => '2'], 'MODE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages::class, 'mode'], 'argumentCount' => '1+'], 'MODE.MULT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1+'], 'MODE.SNGL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages::class, 'mode'], 'argumentCount' => '1+'], 'MONTH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateParts::class, 'month'], 'argumentCount' => '1'], 'MROUND' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Round::class, 'multiple'], 'argumentCount' => '2'], 'MULTINOMIAL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::class, 'multinomial'], 'argumentCount' => '1+'], 'MUNIT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\MatrixFunctions::class, 'identity'], 'argumentCount' => '1'], 'N' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'n'], 'argumentCount' => '1'], 'NA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'NA'], 'argumentCount' => '0'], 'NEGBINOMDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Binomial::class, 'negative'], 'argumentCount' => '3'], 'NEGBINOM.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '4'], 'NETWORKDAYS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\NetworkDays::class, 'count'], 'argumentCount' => '2-3'], 'NETWORKDAYS.INTL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2-4'], 'NOMINAL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\InterestRate::class, 'nominal'], 'argumentCount' => '2'], 'NORMDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Normal::class, 'distribution'], 'argumentCount' => '4'], 'NORM.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Normal::class, 'distribution'], 'argumentCount' => '4'], 'NORMINV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Normal::class, 'inverse'], 'argumentCount' => '3'], 'NORM.INV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Normal::class, 'inverse'], 'argumentCount' => '3'], 'NORMSDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StandardNormal::class, 'cumulative'], 'argumentCount' => '1'], 'NORM.S.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StandardNormal::class, 'distribution'], 'argumentCount' => '1,2'], 'NORMSINV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StandardNormal::class, 'inverse'], 'argumentCount' => '1'], 'NORM.S.INV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StandardNormal::class, 'inverse'], 'argumentCount' => '1'], 'NOT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Operations::class, 'NOT'], 'argumentCount' => '1'], 'NOW' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Current::class, 'now'], 'argumentCount' => '0'], 'NPER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic::class, 'periods'], 'argumentCount' => '3-5'], 'NPV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Variable\Periodic::class, 'presentValue'], 'argumentCount' => '2+'], 'NUMBERSTRING' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'NUMBERVALUE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Format::class, 'NUMBERVALUE'], 'argumentCount' => '1+'], 'OCT2BIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertOctal::class, 'toBinary'], 'argumentCount' => '1,2'], 'OCT2DEC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertOctal::class, 'toDecimal'], 'argumentCount' => '1'], 'OCT2HEX' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_ENGINEERING, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engineering\ConvertOctal::class, 'toHex'], 'argumentCount' => '1,2'], 'ODD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Round::class, 'odd'], 'argumentCount' => '1'], 'ODDFPRICE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '8,9'], 'ODDFYIELD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '8,9'], 'ODDLPRICE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '7,8'], 'ODDLYIELD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '7,8'], 'OFFSET' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\Offset::class, 'OFFSET'], 'argumentCount' => '3-5', 'passCellReference' => \true, 'passByReference' => [\true]], 'OR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Operations::class, 'logicalOr'], 'argumentCount' => '1+'], 'PDURATION' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Single::class, 'periods'], 'argumentCount' => '3'], 'PEARSON' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'CORREL'], 'argumentCount' => '2'], 'PERCENTILE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Percentiles::class, 'PERCENTILE'], 'argumentCount' => '2'], 'PERCENTILE.EXC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'PERCENTILE.INC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Percentiles::class, 'PERCENTILE'], 'argumentCount' => '2'], 'PERCENTRANK' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Percentiles::class, 'PERCENTRANK'], 'argumentCount' => '2,3'], 'PERCENTRANK.EXC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2,3'], 'PERCENTRANK.INC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Percentiles::class, 'PERCENTRANK'], 'argumentCount' => '2,3'], 'PERMUT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Permutations::class, 'PERMUT'], 'argumentCount' => '2'], 'PERMUTATIONA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Permutations::class, 'PERMUTATIONA'], 'argumentCount' => '2'], 'PHONETIC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1'], 'PHI' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1'], 'PI' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => 'pi', 'argumentCount' => '0'], 'PMT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\Payments::class, 'annuity'], 'argumentCount' => '3-5'], 'POISSON' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Poisson::class, 'distribution'], 'argumentCount' => '3'], 'POISSON.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Poisson::class, 'distribution'], 'argumentCount' => '3'], 'POWER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Operations::class, 'power'], 'argumentCount' => '2'], 'PPMT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\Payments::class, 'interestPayment'], 'argumentCount' => '4-6'], 'PRICE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\Price::class, 'price'], 'argumentCount' => '6,7'], 'PRICEDISC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\Price::class, 'priceDiscounted'], 'argumentCount' => '4,5'], 'PRICEMAT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\Price::class, 'priceAtMaturity'], 'argumentCount' => '5,6'], 'PROB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3,4'], 'PRODUCT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Operations::class, 'product'], 'argumentCount' => '1+'], 'PROPER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\CaseConvert::class, 'proper'], 'argumentCount' => '1'], 'PV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic::class, 'presentValue'], 'argumentCount' => '3-5'], 'QUARTILE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Percentiles::class, 'QUARTILE'], 'argumentCount' => '2'], 'QUARTILE.EXC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'QUARTILE.INC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Percentiles::class, 'QUARTILE'], 'argumentCount' => '2'], 'QUOTIENT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Operations::class, 'quotient'], 'argumentCount' => '2'], 'RADIANS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Angle::class, 'toRadians'], 'argumentCount' => '1'], 'RAND' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Random::class, 'rand'], 'argumentCount' => '0'], 'RANDARRAY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '0-5'], 'RANDBETWEEN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Random::class, 'randBetween'], 'argumentCount' => '2'], 'RANK' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Percentiles::class, 'RANK'], 'argumentCount' => '2,3'], 'RANK.AVG' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2,3'], 'RANK.EQ' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Percentiles::class, 'RANK'], 'argumentCount' => '2,3'], 'RATE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Constant\Periodic\Interest::class, 'rate'], 'argumentCount' => '3-6'], 'RECEIVED' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\Price::class, 'received'], 'argumentCount' => '4-5'], 'REPLACE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Replace::class, 'replace'], 'argumentCount' => '4'], 'REPLACEB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Replace::class, 'replace'], 'argumentCount' => '4'], 'REPT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Concatenate::class, 'builtinREPT'], 'argumentCount' => '2'], 'RIGHT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Extract::class, 'right'], 'argumentCount' => '1,2'], 'RIGHTB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Extract::class, 'right'], 'argumentCount' => '1,2'], 'ROMAN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Roman::class, 'evaluate'], 'argumentCount' => '1,2'], 'ROUND' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Round::class, 'round'], 'argumentCount' => '2'], 'ROUNDBAHTDOWN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'ROUNDBAHTUP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'ROUNDDOWN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Round::class, 'down'], 'argumentCount' => '2'], 'ROUNDUP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Round::class, 'up'], 'argumentCount' => '2'], 'ROW' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\RowColumnInformation::class, 'ROW'], 'argumentCount' => '-1', 'passCellReference' => \true, 'passByReference' => [\true]], 'ROWS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\RowColumnInformation::class, 'ROWS'], 'argumentCount' => '1'], 'RRI' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Single::class, 'interestRate'], 'argumentCount' => '3'], 'RSQ' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'RSQ'], 'argumentCount' => '2'], 'RTD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1+'], 'SEARCH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Search::class, 'insensitive'], 'argumentCount' => '2,3'], 'SEARCHB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Search::class, 'insensitive'], 'argumentCount' => '2,3'], 'SEC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Secant::class, 'sec'], 'argumentCount' => '1'], 'SECH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Secant::class, 'sech'], 'argumentCount' => '1'], 'SECOND' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\TimeParts::class, 'second'], 'argumentCount' => '1'], 'SEQUENCE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'SERIESSUM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\SeriesSum::class, 'evaluate'], 'argumentCount' => '4'], 'SHEET' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '0,1'], 'SHEETS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '0,1'], 'SIGN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Sign::class, 'evaluate'], 'argumentCount' => '1'], 'SIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Sine::class, 'sin'], 'argumentCount' => '1'], 'SINH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Sine::class, 'sinh'], 'argumentCount' => '1'], 'SKEW' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Deviations::class, 'skew'], 'argumentCount' => '1+'], 'SKEW.P' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1+'], 'SLN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Depreciation::class, 'SLN'], 'argumentCount' => '3'], 'SLOPE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'SLOPE'], 'argumentCount' => '2'], 'SMALL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Size::class, 'small'], 'argumentCount' => '2'], 'SORT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1+'], 'SORTBY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2+'], 'SQRT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Sqrt::class, 'sqrt'], 'argumentCount' => '1'], 'SQRTPI' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Sqrt::class, 'pi'], 'argumentCount' => '1'], 'STANDARDIZE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Standardize::class, 'execute'], 'argumentCount' => '3'], 'STDEV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StandardDeviations::class, 'STDEV'], 'argumentCount' => '1+'], 'STDEV.S' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StandardDeviations::class, 'STDEV'], 'argumentCount' => '1+'], 'STDEV.P' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StandardDeviations::class, 'STDEVP'], 'argumentCount' => '1+'], 'STDEVA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StandardDeviations::class, 'STDEVA'], 'argumentCount' => '1+'], 'STDEVP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StandardDeviations::class, 'STDEVP'], 'argumentCount' => '1+'], 'STDEVPA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StandardDeviations::class, 'STDEVPA'], 'argumentCount' => '1+'], 'STEYX' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'STEYX'], 'argumentCount' => '2'], 'SUBSTITUTE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Replace::class, 'substitute'], 'argumentCount' => '3,4'], 'SUBTOTAL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Subtotal::class, 'evaluate'], 'argumentCount' => '2+', 'passCellReference' => \true], 'SUM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Sum::class, 'sumErroringStrings'], 'argumentCount' => '1+'], 'SUMIF' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Conditional::class, 'SUMIF'], 'argumentCount' => '2,3'], 'SUMIFS' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Conditional::class, 'SUMIFS'], 'argumentCount' => '3+'], 'SUMPRODUCT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Sum::class, 'product'], 'argumentCount' => '1+'], 'SUMSQ' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\SumSquares::class, 'sumSquare'], 'argumentCount' => '1+'], 'SUMX2MY2' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\SumSquares::class, 'sumXSquaredMinusYSquared'], 'argumentCount' => '2'], 'SUMX2PY2' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\SumSquares::class, 'sumXSquaredPlusYSquared'], 'argumentCount' => '2'], 'SUMXMY2' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\SumSquares::class, 'sumXMinusYSquared'], 'argumentCount' => '2'], 'SWITCH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Conditional::class, 'statementSwitch'], 'argumentCount' => '3+'], 'SYD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Depreciation::class, 'SYD'], 'argumentCount' => '4'], 'T' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Text::class, 'test'], 'argumentCount' => '1'], 'TAN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Tangent::class, 'tan'], 'argumentCount' => '1'], 'TANH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trig\Tangent::class, 'tanh'], 'argumentCount' => '1'], 'TBILLEQ' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\TreasuryBill::class, 'bondEquivalentYield'], 'argumentCount' => '3'], 'TBILLPRICE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\TreasuryBill::class, 'price'], 'argumentCount' => '3'], 'TBILLYIELD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\TreasuryBill::class, 'yield'], 'argumentCount' => '3'], 'TDIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StudentT::class, 'distribution'], 'argumentCount' => '3'], 'T.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3'], 'T.DIST.2T' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'T.DIST.RT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'TEXT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Format::class, 'TEXTFORMAT'], 'argumentCount' => '2'], 'TEXTJOIN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Concatenate::class, 'TEXTJOIN'], 'argumentCount' => '3+'], 'THAIDAYOFWEEK' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'THAIDIGIT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'THAIMONTHOFYEAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'THAINUMSOUND' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'THAINUMSTRING' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'THAISTRINGLENGTH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'THAIYEAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'TIME' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Time::class, 'fromHMS'], 'argumentCount' => '3'], 'TIMEVALUE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\TimeValue::class, 'fromString'], 'argumentCount' => '1'], 'TINV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StudentT::class, 'inverse'], 'argumentCount' => '2'], 'T.INV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StudentT::class, 'inverse'], 'argumentCount' => '2'], 'T.INV.2T' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2'], 'TODAY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Current::class, 'today'], 'argumentCount' => '0'], 'TRANSPOSE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\Matrix::class, 'transpose'], 'argumentCount' => '1'], 'TREND' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Trends::class, 'TREND'], 'argumentCount' => '1-4'], 'TRIM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Trim::class, 'spaces'], 'argumentCount' => '1'], 'TRIMMEAN' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Averages\Mean::class, 'trim'], 'argumentCount' => '2'], 'TRUE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Boolean::class, 'TRUE'], 'argumentCount' => '0'], 'TRUNC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_MATH_AND_TRIG, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Trunc::class, 'evaluate'], 'argumentCount' => '1,2'], 'TTEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '4'], 'T.TEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '4'], 'TYPE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_INFORMATION, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'TYPE'], 'argumentCount' => '1'], 'UNICHAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\CharacterConvert::class, 'character'], 'argumentCount' => '1'], 'UNICODE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\CharacterConvert::class, 'code'], 'argumentCount' => '1'], 'UNIQUE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '1+'], 'UPPER' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\CaseConvert::class, 'upper'], 'argumentCount' => '1'], 'USDOLLAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Dollar::class, 'format'], 'argumentCount' => '2'], 'VALUE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Format::class, 'VALUE'], 'argumentCount' => '1'], 'VALUETOTEXT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_TEXT_AND_DATA, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '?'], 'VAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Variances::class, 'VAR'], 'argumentCount' => '1+'], 'VAR.P' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Variances::class, 'VARP'], 'argumentCount' => '1+'], 'VAR.S' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Variances::class, 'VAR'], 'argumentCount' => '1+'], 'VARA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Variances::class, 'VARA'], 'argumentCount' => '1+'], 'VARP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Variances::class, 'VARP'], 'argumentCount' => '1+'], 'VARPA' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Variances::class, 'VARPA'], 'argumentCount' => '1+'], 'VDB' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '5-7'], 'VLOOKUP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef\VLookup::class, 'lookup'], 'argumentCount' => '3,4'], 'WEBSERVICE' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_WEB, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Web\Service::class, 'webService'], 'argumentCount' => '1'], 'WEEKDAY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Week::class, 'day'], 'argumentCount' => '1,2'], 'WEEKNUM' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Week::class, 'number'], 'argumentCount' => '1,2'], 'WEIBULL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Weibull::class, 'distribution'], 'argumentCount' => '4'], 'WEIBULL.DIST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\Weibull::class, 'distribution'], 'argumentCount' => '4'], 'WORKDAY' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\WorkDay::class, 'date'], 'argumentCount' => '2-3'], 'WORKDAY.INTL' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2-4'], 'XIRR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Variable\NonPeriodic::class, 'rate'], 'argumentCount' => '2,3'], 'XLOOKUP' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '3-6'], 'XNPV' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\CashFlow\Variable\NonPeriodic::class, 'presentValue'], 'argumentCount' => '3'], 'XMATCH' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOOKUP_AND_REFERENCE, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '2,3'], 'XOR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_LOGICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Logical\Operations::class, 'logicalXor'], 'argumentCount' => '1+'], 'YEAR' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\DateParts::class, 'year'], 'argumentCount' => '1'], 'YEARFRAC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_DATE_AND_TIME, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\YearFrac::class, 'fraction'], 'argumentCount' => '2,3'], 'YIELD' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'DUMMY'], 'argumentCount' => '6,7'], 'YIELDDISC' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\Yields::class, 'yieldDiscounted'], 'argumentCount' => '4,5'], 'YIELDMAT' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_FINANCIAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Financial\Securities\Yields::class, 'yieldAtMaturity'], 'argumentCount' => '5,6'], 'ZTEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StandardNormal::class, 'zTest'], 'argumentCount' => '2-3'], 'Z.TEST' => ['category' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Category::CATEGORY_STATISTICAL, 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StandardNormal::class, 'zTest'], 'argumentCount' => '2-3']];
    //    Internal functions used for special control purposes
    private static $controlFunctions = ['MKMATRIX' => ['argumentCount' => '*', 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Internal\MakeMatrix::class, 'make']], 'NAME.ERROR' => ['argumentCount' => '*', 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::class, 'NAME']], 'WILDCARDMATCH' => ['argumentCount' => '2', 'functionCall' => [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Internal\WildcardMatch::class, 'compare']]];
    public function __construct(?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet $spreadsheet = null)
    {
        $this->delta = 1 * 10 ** (0 - \ini_get('precision'));
        $this->spreadsheet = $spreadsheet;
        $this->cyclicReferenceStack = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engine\CyclicReferenceStack();
        $this->debugLog = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Engine\Logger($this->cyclicReferenceStack);
        self::$referenceHelper = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\ReferenceHelper::getInstance();
    }
    private static function loadLocales() : void
    {
        $localeFileDirectory = __DIR__ . '/locale/';
        foreach (\glob($localeFileDirectory . '*', \GLOB_ONLYDIR) as $filename) {
            $filename = \substr($filename, \strlen($localeFileDirectory));
            if ($filename != 'en') {
                self::$validLocaleLanguages[] = $filename;
            }
        }
    }
    /**
     * Get an instance of this class.
     *
     * @param ?Spreadsheet $spreadsheet Injected spreadsheet for working with a PhpSpreadsheet Spreadsheet object,
     *                                    or NULL to create a standalone calculation engine
     */
    public static function getInstance(?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet $spreadsheet = null) : self
    {
        if ($spreadsheet !== null) {
            $instance = $spreadsheet->getCalculationEngine();
            if (isset($instance)) {
                return $instance;
            }
        }
        if (!isset(self::$instance) || self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    /**
     * Flush the calculation cache for any existing instance of this class
     *        but only if a Calculation instance exists.
     */
    public function flushInstance() : void
    {
        $this->clearCalculationCache();
        $this->clearBranchStore();
    }
    /**
     * Get the Logger for this calculation engine instance.
     *
     * @return Logger
     */
    public function getDebugLog()
    {
        return $this->debugLog;
    }
    /**
     * __clone implementation. Cloning should not be allowed in a Singleton!
     */
    public final function __clone()
    {
        throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception('Cloning the calculation engine is not allowed!');
    }
    /**
     * Return the locale-specific translation of TRUE.
     *
     * @return string locale-specific translation of TRUE
     */
    public static function getTRUE() : string
    {
        return self::$localeBoolean['TRUE'];
    }
    /**
     * Return the locale-specific translation of FALSE.
     *
     * @return string locale-specific translation of FALSE
     */
    public static function getFALSE() : string
    {
        return self::$localeBoolean['FALSE'];
    }
    /**
     * Set the Array Return Type (Array or Value of first element in the array).
     *
     * @param string $returnType Array return type
     *
     * @return bool Success or failure
     */
    public static function setArrayReturnType($returnType)
    {
        if ($returnType == self::RETURN_ARRAY_AS_VALUE || $returnType == self::RETURN_ARRAY_AS_ERROR || $returnType == self::RETURN_ARRAY_AS_ARRAY) {
            self::$returnArrayAsType = $returnType;
            return \true;
        }
        return \false;
    }
    /**
     * Return the Array Return Type (Array or Value of first element in the array).
     *
     * @return string $returnType Array return type
     */
    public static function getArrayReturnType()
    {
        return self::$returnArrayAsType;
    }
    /**
     * Is calculation caching enabled?
     *
     * @return bool
     */
    public function getCalculationCacheEnabled()
    {
        return $this->calculationCacheEnabled;
    }
    /**
     * Enable/disable calculation cache.
     *
     * @param bool $pValue
     */
    public function setCalculationCacheEnabled($pValue) : void
    {
        $this->calculationCacheEnabled = $pValue;
        $this->clearCalculationCache();
    }
    /**
     * Enable calculation cache.
     */
    public function enableCalculationCache() : void
    {
        $this->setCalculationCacheEnabled(\true);
    }
    /**
     * Disable calculation cache.
     */
    public function disableCalculationCache() : void
    {
        $this->setCalculationCacheEnabled(\false);
    }
    /**
     * Clear calculation cache.
     */
    public function clearCalculationCache() : void
    {
        $this->calculationCache = [];
    }
    /**
     * Clear calculation cache for a specified worksheet.
     *
     * @param string $worksheetName
     */
    public function clearCalculationCacheForWorksheet($worksheetName) : void
    {
        if (isset($this->calculationCache[$worksheetName])) {
            unset($this->calculationCache[$worksheetName]);
        }
    }
    /**
     * Rename calculation cache for a specified worksheet.
     *
     * @param string $fromWorksheetName
     * @param string $toWorksheetName
     */
    public function renameCalculationCacheForWorksheet($fromWorksheetName, $toWorksheetName) : void
    {
        if (isset($this->calculationCache[$fromWorksheetName])) {
            $this->calculationCache[$toWorksheetName] =& $this->calculationCache[$fromWorksheetName];
            unset($this->calculationCache[$fromWorksheetName]);
        }
    }
    /**
     * Enable/disable calculation cache.
     *
     * @param mixed $enabled
     */
    public function setBranchPruningEnabled($enabled) : void
    {
        $this->branchPruningEnabled = $enabled;
    }
    public function enableBranchPruning() : void
    {
        $this->setBranchPruningEnabled(\true);
    }
    public function disableBranchPruning() : void
    {
        $this->setBranchPruningEnabled(\false);
    }
    public function clearBranchStore() : void
    {
        $this->branchStoreKeyCounter = 0;
    }
    /**
     * Get the currently defined locale code.
     *
     * @return string
     */
    public function getLocale()
    {
        return self::$localeLanguage;
    }
    private function getLocaleFile(string $localeDir, string $locale, string $language, string $file) : string
    {
        $localeFileName = $localeDir . \str_replace('_', \DIRECTORY_SEPARATOR, $locale) . \DIRECTORY_SEPARATOR . $file;
        if (!\file_exists($localeFileName)) {
            //    If there isn't a locale specific file, look for a language specific file
            $localeFileName = $localeDir . $language . \DIRECTORY_SEPARATOR . $file;
            if (!\file_exists($localeFileName)) {
                throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception('Locale file not found');
            }
        }
        return $localeFileName;
    }
    /**
     * Set the locale code.
     *
     * @param string $locale The locale to use for formula translation, eg: 'en_us'
     *
     * @return bool
     */
    public function setLocale(string $locale)
    {
        //    Identify our locale and language
        $language = $locale = \strtolower($locale);
        if (\strpos($locale, '_') !== \false) {
            [$language] = \explode('_', $locale);
        }
        if (\count(self::$validLocaleLanguages) == 1) {
            self::loadLocales();
        }
        //    Test whether we have any language data for this language (any locale)
        if (\in_array($language, self::$validLocaleLanguages)) {
            //    initialise language/locale settings
            self::$localeFunctions = [];
            self::$localeArgumentSeparator = ',';
            self::$localeBoolean = ['TRUE' => 'TRUE', 'FALSE' => 'FALSE', 'NULL' => 'NULL'];
            //    Default is US English, if user isn't requesting US english, then read the necessary data from the locale files
            if ($locale !== 'en_us') {
                $localeDir = \implode(\DIRECTORY_SEPARATOR, [__DIR__, 'locale', null]);
                //    Search for a file with a list of function names for locale
                try {
                    $functionNamesFile = $this->getLocaleFile($localeDir, $locale, $language, 'functions');
                } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
                    return \false;
                }
                //    Retrieve the list of locale or language specific function names
                $localeFunctions = \file($functionNamesFile, \FILE_IGNORE_NEW_LINES | \FILE_SKIP_EMPTY_LINES);
                foreach ($localeFunctions as $localeFunction) {
                    [$localeFunction] = \explode('##', $localeFunction);
                    //    Strip out comments
                    if (\strpos($localeFunction, '=') !== \false) {
                        [$fName, $lfName] = \array_map('trim', \explode('=', $localeFunction));
                        if (isset(self::$phpSpreadsheetFunctions[$fName]) && $lfName != '' && $fName != $lfName) {
                            self::$localeFunctions[$fName] = $lfName;
                        }
                    }
                }
                //    Default the TRUE and FALSE constants to the locale names of the TRUE() and FALSE() functions
                if (isset(self::$localeFunctions['TRUE'])) {
                    self::$localeBoolean['TRUE'] = self::$localeFunctions['TRUE'];
                }
                if (isset(self::$localeFunctions['FALSE'])) {
                    self::$localeBoolean['FALSE'] = self::$localeFunctions['FALSE'];
                }
                try {
                    $configFile = $this->getLocaleFile($localeDir, $locale, $language, 'config');
                } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
                    return \false;
                }
                $localeSettings = \file($configFile, \FILE_IGNORE_NEW_LINES | \FILE_SKIP_EMPTY_LINES);
                foreach ($localeSettings as $localeSetting) {
                    [$localeSetting] = \explode('##', $localeSetting);
                    //    Strip out comments
                    if (\strpos($localeSetting, '=') !== \false) {
                        [$settingName, $settingValue] = \array_map('trim', \explode('=', $localeSetting));
                        $settingName = \strtoupper($settingName);
                        if ($settingValue !== '') {
                            switch ($settingName) {
                                case 'ARGUMENTSEPARATOR':
                                    self::$localeArgumentSeparator = $settingValue;
                                    break;
                            }
                        }
                    }
                }
            }
            self::$functionReplaceFromExcel = self::$functionReplaceToExcel = self::$functionReplaceFromLocale = self::$functionReplaceToLocale = null;
            self::$localeLanguage = $locale;
            return \true;
        }
        return \false;
    }
    /**
     * @param string $fromSeparator
     * @param string $toSeparator
     * @param string $formula
     * @param bool $inBraces
     *
     * @return string
     */
    public static function translateSeparator($fromSeparator, $toSeparator, $formula, &$inBraces)
    {
        $strlen = \mb_strlen($formula);
        for ($i = 0; $i < $strlen; ++$i) {
            $chr = \mb_substr($formula, $i, 1);
            switch ($chr) {
                case self::FORMULA_OPEN_FUNCTION_BRACE:
                    $inBraces = \true;
                    break;
                case self::FORMULA_CLOSE_FUNCTION_BRACE:
                    $inBraces = \false;
                    break;
                case $fromSeparator:
                    if (!$inBraces) {
                        $formula = \mb_substr($formula, 0, $i) . $toSeparator . \mb_substr($formula, $i + 1);
                    }
            }
        }
        return $formula;
    }
    /**
     * @param string[] $from
     * @param string[] $to
     * @param string $formula
     * @param string $fromSeparator
     * @param string $toSeparator
     *
     * @return string
     */
    private static function translateFormula(array $from, array $to, $formula, $fromSeparator, $toSeparator)
    {
        //    Convert any Excel function names to the required language
        if (self::$localeLanguage !== 'en_us') {
            $inBraces = \false;
            //    If there is the possibility of braces within a quoted string, then we don't treat those as matrix indicators
            if (\strpos($formula, self::FORMULA_STRING_QUOTE) !== \false) {
                //    So instead we skip replacing in any quoted strings by only replacing in every other array element after we've exploded
                //        the formula
                $temp = \explode(self::FORMULA_STRING_QUOTE, $formula);
                $i = \false;
                foreach ($temp as &$value) {
                    //    Only count/replace in alternating array entries
                    if ($i = !$i) {
                        $value = \preg_replace($from, $to, $value);
                        $value = self::translateSeparator($fromSeparator, $toSeparator, $value, $inBraces);
                    }
                }
                unset($value);
                //    Then rebuild the formula string
                $formula = \implode(self::FORMULA_STRING_QUOTE, $temp);
            } else {
                //    If there's no quoted strings, then we do a simple count/replace
                $formula = \preg_replace($from, $to, $formula);
                $formula = self::translateSeparator($fromSeparator, $toSeparator, $formula, $inBraces);
            }
        }
        return $formula;
    }
    private static $functionReplaceFromExcel;
    private static $functionReplaceToLocale;
    public function _translateFormulaToLocale($formula)
    {
        if (self::$functionReplaceFromExcel === null) {
            self::$functionReplaceFromExcel = [];
            foreach (\array_keys(self::$localeFunctions) as $excelFunctionName) {
                self::$functionReplaceFromExcel[] = '/(@?[^\\w\\.])' . \preg_quote($excelFunctionName, '/') . '([\\s]*\\()/Ui';
            }
            foreach (\array_keys(self::$localeBoolean) as $excelBoolean) {
                self::$functionReplaceFromExcel[] = '/(@?[^\\w\\.])' . \preg_quote($excelBoolean, '/') . '([^\\w\\.])/Ui';
            }
        }
        if (self::$functionReplaceToLocale === null) {
            self::$functionReplaceToLocale = [];
            foreach (self::$localeFunctions as $localeFunctionName) {
                self::$functionReplaceToLocale[] = '$1' . \trim($localeFunctionName) . '$2';
            }
            foreach (self::$localeBoolean as $localeBoolean) {
                self::$functionReplaceToLocale[] = '$1' . \trim($localeBoolean) . '$2';
            }
        }
        return self::translateFormula(self::$functionReplaceFromExcel, self::$functionReplaceToLocale, $formula, ',', self::$localeArgumentSeparator);
    }
    private static $functionReplaceFromLocale;
    private static $functionReplaceToExcel;
    public function _translateFormulaToEnglish($formula)
    {
        if (self::$functionReplaceFromLocale === null) {
            self::$functionReplaceFromLocale = [];
            foreach (self::$localeFunctions as $localeFunctionName) {
                self::$functionReplaceFromLocale[] = '/(@?[^\\w\\.])' . \preg_quote($localeFunctionName, '/') . '([\\s]*\\()/Ui';
            }
            foreach (self::$localeBoolean as $excelBoolean) {
                self::$functionReplaceFromLocale[] = '/(@?[^\\w\\.])' . \preg_quote($excelBoolean, '/') . '([^\\w\\.])/Ui';
            }
        }
        if (self::$functionReplaceToExcel === null) {
            self::$functionReplaceToExcel = [];
            foreach (\array_keys(self::$localeFunctions) as $excelFunctionName) {
                self::$functionReplaceToExcel[] = '$1' . \trim($excelFunctionName) . '$2';
            }
            foreach (\array_keys(self::$localeBoolean) as $excelBoolean) {
                self::$functionReplaceToExcel[] = '$1' . \trim($excelBoolean) . '$2';
            }
        }
        return self::translateFormula(self::$functionReplaceFromLocale, self::$functionReplaceToExcel, $formula, self::$localeArgumentSeparator, ',');
    }
    public static function localeFunc($function)
    {
        if (self::$localeLanguage !== 'en_us') {
            $functionName = \trim($function, '(');
            if (isset(self::$localeFunctions[$functionName])) {
                $brace = $functionName != $function;
                $function = self::$localeFunctions[$functionName];
                if ($brace) {
                    $function .= '(';
                }
            }
        }
        return $function;
    }
    /**
     * Wrap string values in quotes.
     *
     * @param mixed $value
     *
     * @return mixed
     */
    public static function wrapResult($value)
    {
        if (\is_string($value)) {
            //    Error values cannot be "wrapped"
            if (\preg_match('/^' . self::CALCULATION_REGEXP_ERROR . '$/i', $value, $match)) {
                //    Return Excel errors "as is"
                return $value;
            }
            //    Return strings wrapped in quotes
            return self::FORMULA_STRING_QUOTE . $value . self::FORMULA_STRING_QUOTE;
        } elseif (\is_float($value) && (\is_nan($value) || \is_infinite($value))) {
            //    Convert numeric errors to NaN error
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return $value;
    }
    /**
     * Remove quotes used as a wrapper to identify string values.
     *
     * @param mixed $value
     *
     * @return mixed
     */
    public static function unwrapResult($value)
    {
        if (\is_string($value)) {
            if (isset($value[0]) && $value[0] == self::FORMULA_STRING_QUOTE && \substr($value, -1) == self::FORMULA_STRING_QUOTE) {
                return \substr($value, 1, -1);
            }
            //    Convert numeric errors to NAN error
        } elseif (\is_float($value) && (\is_nan($value) || \is_infinite($value))) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return $value;
    }
    /**
     * Calculate cell value (using formula from a cell ID)
     * Retained for backward compatibility.
     *
     * @param Cell $pCell Cell to calculate
     *
     * @return mixed
     */
    public function calculate(?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell $pCell = null)
    {
        try {
            return $this->calculateCellValue($pCell);
        } catch (\Exception $e) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception($e->getMessage());
        }
    }
    /**
     * Calculate the value of a cell formula.
     *
     * @param Cell $pCell Cell to calculate
     * @param bool $resetLog Flag indicating whether the debug log should be reset or not
     *
     * @return mixed
     */
    public function calculateCellValue(?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell $pCell = null, $resetLog = \true)
    {
        if ($pCell === null) {
            return null;
        }
        $returnArrayAsType = self::$returnArrayAsType;
        if ($resetLog) {
            //    Initialise the logging settings if requested
            $this->formulaError = null;
            $this->debugLog->clearLog();
            $this->cyclicReferenceStack->clear();
            $this->cyclicFormulaCounter = 1;
            self::$returnArrayAsType = self::RETURN_ARRAY_AS_ARRAY;
        }
        //    Execute the calculation for the cell formula
        $this->cellStack[] = ['sheet' => $pCell->getWorksheet()->getTitle(), 'cell' => $pCell->getCoordinate()];
        try {
            $result = self::unwrapResult($this->_calculateFormulaValue($pCell->getValue(), $pCell->getCoordinate(), $pCell));
            $cellAddress = \array_pop($this->cellStack);
            $this->spreadsheet->getSheetByName($cellAddress['sheet'])->getCell($cellAddress['cell']);
        } catch (\Exception $e) {
            $cellAddress = \array_pop($this->cellStack);
            $this->spreadsheet->getSheetByName($cellAddress['sheet'])->getCell($cellAddress['cell']);
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception($e->getMessage());
        }
        if (\is_array($result) && self::$returnArrayAsType != self::RETURN_ARRAY_AS_ARRAY) {
            self::$returnArrayAsType = $returnArrayAsType;
            $testResult = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArray($result);
            if (self::$returnArrayAsType == self::RETURN_ARRAY_AS_ERROR) {
                return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            }
            //    If there's only a single cell in the array, then we allow it
            if (\count($testResult) != 1) {
                //    If keys are numeric, then it's a matrix result rather than a cell range result, so we permit it
                $r = \array_keys($result);
                $r = \array_shift($r);
                if (!\is_numeric($r)) {
                    return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
                }
                if (\is_array($result[$r])) {
                    $c = \array_keys($result[$r]);
                    $c = \array_shift($c);
                    if (!\is_numeric($c)) {
                        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
                    }
                }
            }
            $result = \array_shift($testResult);
        }
        self::$returnArrayAsType = $returnArrayAsType;
        if ($result === null && $pCell->getWorksheet()->getSheetView()->getShowZeros()) {
            return 0;
        } elseif (\is_float($result) && (\is_nan($result) || \is_infinite($result))) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return $result;
    }
    /**
     * Validate and parse a formula string.
     *
     * @param string $formula Formula to parse
     *
     * @return array|bool
     */
    public function parseFormula($formula)
    {
        //    Basic validation that this is indeed a formula
        //    We return an empty array if not
        $formula = \trim($formula);
        if (!isset($formula[0]) || $formula[0] != '=') {
            return [];
        }
        $formula = \ltrim(\substr($formula, 1));
        if (!isset($formula[0])) {
            return [];
        }
        //    Parse the formula and return the token stack
        return $this->internalParseFormula($formula);
    }
    /**
     * Calculate the value of a formula.
     *
     * @param string $formula Formula to parse
     * @param string $cellID Address of the cell to calculate
     * @param Cell $pCell Cell to calculate
     *
     * @return mixed
     */
    public function calculateFormula($formula, $cellID = null, ?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell $pCell = null)
    {
        //    Initialise the logging settings
        $this->formulaError = null;
        $this->debugLog->clearLog();
        $this->cyclicReferenceStack->clear();
        $resetCache = $this->getCalculationCacheEnabled();
        if ($this->spreadsheet !== null && $cellID === null && $pCell === null) {
            $cellID = 'A1';
            $pCell = $this->spreadsheet->getActiveSheet()->getCell($cellID);
        } else {
            //    Disable calculation cacheing because it only applies to cell calculations, not straight formulae
            //    But don't actually flush any cache
            $this->calculationCacheEnabled = \false;
        }
        //    Execute the calculation
        try {
            $result = self::unwrapResult($this->_calculateFormulaValue($formula, $cellID, $pCell));
        } catch (\Exception $e) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception($e->getMessage());
        }
        if ($this->spreadsheet === null) {
            //    Reset calculation cacheing to its previous state
            $this->calculationCacheEnabled = $resetCache;
        }
        return $result;
    }
    /**
     * @param mixed $cellValue
     */
    public function getValueFromCache(string $cellReference, &$cellValue) : bool
    {
        $this->debugLog->writeDebugLog("Testing cache value for cell {$cellReference}");
        // Is calculation cacheing enabled?
        // If so, is the required value present in calculation cache?
        if ($this->calculationCacheEnabled && isset($this->calculationCache[$cellReference])) {
            $this->debugLog->writeDebugLog("Retrieving value for cell {$cellReference} from cache");
            // Return the cached result
            $cellValue = $this->calculationCache[$cellReference];
            return \true;
        }
        return \false;
    }
    /**
     * @param string $cellReference
     * @param mixed $cellValue
     */
    public function saveValueToCache($cellReference, $cellValue) : void
    {
        if ($this->calculationCacheEnabled) {
            $this->calculationCache[$cellReference] = $cellValue;
        }
    }
    /**
     * Parse a cell formula and calculate its value.
     *
     * @param string $formula The formula to parse and calculate
     * @param string $cellID The ID (e.g. A3) of the cell that we are calculating
     * @param Cell $pCell Cell to calculate
     *
     * @return mixed
     */
    public function _calculateFormulaValue($formula, $cellID = null, ?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell $pCell = null)
    {
        $cellValue = null;
        //  Quote-Prefixed cell values cannot be formulae, but are treated as strings
        if ($pCell !== null && $pCell->getStyle()->getQuotePrefix() === \true) {
            return self::wrapResult((string) $formula);
        }
        if (\preg_match('/^=\\s*cmd\\s*\\|/miu', $formula) !== 0) {
            return self::wrapResult($formula);
        }
        //    Basic validation that this is indeed a formula
        //    We simply return the cell value if not
        $formula = \trim($formula);
        if ($formula[0] != '=') {
            return self::wrapResult($formula);
        }
        $formula = \ltrim(\substr($formula, 1));
        if (!isset($formula[0])) {
            return self::wrapResult($formula);
        }
        $pCellParent = $pCell !== null ? $pCell->getWorksheet() : null;
        $wsTitle = $pCellParent !== null ? $pCellParent->getTitle() : "\0Wrk";
        $wsCellReference = $wsTitle . '!' . $cellID;
        if ($cellID !== null && $this->getValueFromCache($wsCellReference, $cellValue)) {
            return $cellValue;
        }
        $this->debugLog->writeDebugLog("Evaluating formula for cell {$wsCellReference}");
        if ($wsTitle[0] !== "\0" && $this->cyclicReferenceStack->onStack($wsCellReference)) {
            if ($this->cyclicFormulaCount <= 0) {
                $this->cyclicFormulaCell = '';
                return $this->raiseFormulaError('Cyclic Reference in Formula');
            } elseif ($this->cyclicFormulaCell === $wsCellReference) {
                ++$this->cyclicFormulaCounter;
                if ($this->cyclicFormulaCounter >= $this->cyclicFormulaCount) {
                    $this->cyclicFormulaCell = '';
                    return $cellValue;
                }
            } elseif ($this->cyclicFormulaCell == '') {
                if ($this->cyclicFormulaCounter >= $this->cyclicFormulaCount) {
                    return $cellValue;
                }
                $this->cyclicFormulaCell = $wsCellReference;
            }
        }
        $this->debugLog->writeDebugLog("Formula for cell {$wsCellReference} is {$formula}");
        //    Parse the formula onto the token stack and calculate the value
        $this->cyclicReferenceStack->push($wsCellReference);
        $cellValue = $this->processTokenStack($this->internalParseFormula($formula, $pCell), $cellID, $pCell);
        $this->cyclicReferenceStack->pop();
        // Save to calculation cache
        if ($cellID !== null) {
            $this->saveValueToCache($wsCellReference, $cellValue);
        }
        //    Return the calculated value
        return $cellValue;
    }
    /**
     * Ensure that paired matrix operands are both matrices and of the same size.
     *
     * @param mixed $operand1 First matrix operand
     * @param mixed $operand2 Second matrix operand
     * @param int $resize Flag indicating whether the matrices should be resized to match
     *                                        and (if so), whether the smaller dimension should grow or the
     *                                        larger should shrink.
     *                                            0 = no resize
     *                                            1 = shrink to fit
     *                                            2 = extend to fit
     *
     * @return array
     */
    private static function checkMatrixOperands(&$operand1, &$operand2, $resize = 1)
    {
        //    Examine each of the two operands, and turn them into an array if they aren't one already
        //    Note that this function should only be called if one or both of the operand is already an array
        if (!\is_array($operand1)) {
            [$matrixRows, $matrixColumns] = self::getMatrixDimensions($operand2);
            $operand1 = \array_fill(0, $matrixRows, \array_fill(0, $matrixColumns, $operand1));
            $resize = 0;
        } elseif (!\is_array($operand2)) {
            [$matrixRows, $matrixColumns] = self::getMatrixDimensions($operand1);
            $operand2 = \array_fill(0, $matrixRows, \array_fill(0, $matrixColumns, $operand2));
            $resize = 0;
        }
        [$matrix1Rows, $matrix1Columns] = self::getMatrixDimensions($operand1);
        [$matrix2Rows, $matrix2Columns] = self::getMatrixDimensions($operand2);
        if ($matrix1Rows == $matrix2Columns && $matrix2Rows == $matrix1Columns) {
            $resize = 1;
        }
        if ($resize == 2) {
            //    Given two matrices of (potentially) unequal size, convert the smaller in each dimension to match the larger
            self::resizeMatricesExtend($operand1, $operand2, $matrix1Rows, $matrix1Columns, $matrix2Rows, $matrix2Columns);
        } elseif ($resize == 1) {
            //    Given two matrices of (potentially) unequal size, convert the larger in each dimension to match the smaller
            self::resizeMatricesShrink($operand1, $operand2, $matrix1Rows, $matrix1Columns, $matrix2Rows, $matrix2Columns);
        }
        return [$matrix1Rows, $matrix1Columns, $matrix2Rows, $matrix2Columns];
    }
    /**
     * Read the dimensions of a matrix, and re-index it with straight numeric keys starting from row 0, column 0.
     *
     * @param array $matrix matrix operand
     *
     * @return int[] An array comprising the number of rows, and number of columns
     */
    public static function getMatrixDimensions(array &$matrix)
    {
        $matrixRows = \count($matrix);
        $matrixColumns = 0;
        foreach ($matrix as $rowKey => $rowValue) {
            if (!\is_array($rowValue)) {
                $matrix[$rowKey] = [$rowValue];
                $matrixColumns = \max(1, $matrixColumns);
            } else {
                $matrix[$rowKey] = \array_values($rowValue);
                $matrixColumns = \max(\count($rowValue), $matrixColumns);
            }
        }
        $matrix = \array_values($matrix);
        return [$matrixRows, $matrixColumns];
    }
    /**
     * Ensure that paired matrix operands are both matrices of the same size.
     *
     * @param mixed $matrix1 First matrix operand
     * @param mixed $matrix2 Second matrix operand
     * @param int $matrix1Rows Row size of first matrix operand
     * @param int $matrix1Columns Column size of first matrix operand
     * @param int $matrix2Rows Row size of second matrix operand
     * @param int $matrix2Columns Column size of second matrix operand
     */
    private static function resizeMatricesShrink(&$matrix1, &$matrix2, $matrix1Rows, $matrix1Columns, $matrix2Rows, $matrix2Columns) : void
    {
        if ($matrix2Columns < $matrix1Columns || $matrix2Rows < $matrix1Rows) {
            if ($matrix2Rows < $matrix1Rows) {
                for ($i = $matrix2Rows; $i < $matrix1Rows; ++$i) {
                    unset($matrix1[$i]);
                }
            }
            if ($matrix2Columns < $matrix1Columns) {
                for ($i = 0; $i < $matrix1Rows; ++$i) {
                    for ($j = $matrix2Columns; $j < $matrix1Columns; ++$j) {
                        unset($matrix1[$i][$j]);
                    }
                }
            }
        }
        if ($matrix1Columns < $matrix2Columns || $matrix1Rows < $matrix2Rows) {
            if ($matrix1Rows < $matrix2Rows) {
                for ($i = $matrix1Rows; $i < $matrix2Rows; ++$i) {
                    unset($matrix2[$i]);
                }
            }
            if ($matrix1Columns < $matrix2Columns) {
                for ($i = 0; $i < $matrix2Rows; ++$i) {
                    for ($j = $matrix1Columns; $j < $matrix2Columns; ++$j) {
                        unset($matrix2[$i][$j]);
                    }
                }
            }
        }
    }
    /**
     * Ensure that paired matrix operands are both matrices of the same size.
     *
     * @param mixed $matrix1 First matrix operand
     * @param mixed $matrix2 Second matrix operand
     * @param int $matrix1Rows Row size of first matrix operand
     * @param int $matrix1Columns Column size of first matrix operand
     * @param int $matrix2Rows Row size of second matrix operand
     * @param int $matrix2Columns Column size of second matrix operand
     */
    private static function resizeMatricesExtend(&$matrix1, &$matrix2, $matrix1Rows, $matrix1Columns, $matrix2Rows, $matrix2Columns) : void
    {
        if ($matrix2Columns < $matrix1Columns || $matrix2Rows < $matrix1Rows) {
            if ($matrix2Columns < $matrix1Columns) {
                for ($i = 0; $i < $matrix2Rows; ++$i) {
                    $x = $matrix2[$i][$matrix2Columns - 1];
                    for ($j = $matrix2Columns; $j < $matrix1Columns; ++$j) {
                        $matrix2[$i][$j] = $x;
                    }
                }
            }
            if ($matrix2Rows < $matrix1Rows) {
                $x = $matrix2[$matrix2Rows - 1];
                for ($i = 0; $i < $matrix1Rows; ++$i) {
                    $matrix2[$i] = $x;
                }
            }
        }
        if ($matrix1Columns < $matrix2Columns || $matrix1Rows < $matrix2Rows) {
            if ($matrix1Columns < $matrix2Columns) {
                for ($i = 0; $i < $matrix1Rows; ++$i) {
                    $x = $matrix1[$i][$matrix1Columns - 1];
                    for ($j = $matrix1Columns; $j < $matrix2Columns; ++$j) {
                        $matrix1[$i][$j] = $x;
                    }
                }
            }
            if ($matrix1Rows < $matrix2Rows) {
                $x = $matrix1[$matrix1Rows - 1];
                for ($i = 0; $i < $matrix2Rows; ++$i) {
                    $matrix1[$i] = $x;
                }
            }
        }
    }
    /**
     * Format details of an operand for display in the log (based on operand type).
     *
     * @param mixed $value First matrix operand
     *
     * @return mixed
     */
    private function showValue($value)
    {
        if ($this->debugLog->getWriteDebugLog()) {
            $testArray = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArray($value);
            if (\count($testArray) == 1) {
                $value = \array_pop($testArray);
            }
            if (\is_array($value)) {
                $returnMatrix = [];
                $pad = $rpad = ', ';
                foreach ($value as $row) {
                    if (\is_array($row)) {
                        $returnMatrix[] = \implode($pad, \array_map([$this, 'showValue'], $row));
                        $rpad = '; ';
                    } else {
                        $returnMatrix[] = $this->showValue($row);
                    }
                }
                return '{ ' . \implode($rpad, $returnMatrix) . ' }';
            } elseif (\is_string($value) && \trim($value, self::FORMULA_STRING_QUOTE) == $value) {
                return self::FORMULA_STRING_QUOTE . $value . self::FORMULA_STRING_QUOTE;
            } elseif (\is_bool($value)) {
                return $value ? self::$localeBoolean['TRUE'] : self::$localeBoolean['FALSE'];
            }
        }
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($value);
    }
    /**
     * Format type and details of an operand for display in the log (based on operand type).
     *
     * @param mixed $value First matrix operand
     *
     * @return null|string
     */
    private function showTypeDetails($value)
    {
        if ($this->debugLog->getWriteDebugLog()) {
            $testArray = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArray($value);
            if (\count($testArray) == 1) {
                $value = \array_pop($testArray);
            }
            if ($value === null) {
                return 'a NULL value';
            } elseif (\is_float($value)) {
                $typeString = 'a floating point number';
            } elseif (\is_int($value)) {
                $typeString = 'an integer number';
            } elseif (\is_bool($value)) {
                $typeString = 'a boolean';
            } elseif (\is_array($value)) {
                $typeString = 'a matrix';
            } else {
                if ($value == '') {
                    return 'an empty string';
                } elseif ($value[0] == '#') {
                    return 'a ' . $value . ' error';
                }
                $typeString = 'a string';
            }
            return $typeString . ' with a value of ' . $this->showValue($value);
        }
        return null;
    }
    /**
     * @param string $formula
     *
     * @return false|string False indicates an error
     */
    private function convertMatrixReferences($formula)
    {
        static $matrixReplaceFrom = [self::FORMULA_OPEN_FUNCTION_BRACE, ';', self::FORMULA_CLOSE_FUNCTION_BRACE];
        static $matrixReplaceTo = ['MKMATRIX(MKMATRIX(', '),MKMATRIX(', '))'];
        //    Convert any Excel matrix references to the MKMATRIX() function
        if (\strpos($formula, self::FORMULA_OPEN_FUNCTION_BRACE) !== \false) {
            //    If there is the possibility of braces within a quoted string, then we don't treat those as matrix indicators
            if (\strpos($formula, self::FORMULA_STRING_QUOTE) !== \false) {
                //    So instead we skip replacing in any quoted strings by only replacing in every other array element after we've exploded
                //        the formula
                $temp = \explode(self::FORMULA_STRING_QUOTE, $formula);
                //    Open and Closed counts used for trapping mismatched braces in the formula
                $openCount = $closeCount = 0;
                $i = \false;
                foreach ($temp as &$value) {
                    //    Only count/replace in alternating array entries
                    if ($i = !$i) {
                        $openCount += \substr_count($value, self::FORMULA_OPEN_FUNCTION_BRACE);
                        $closeCount += \substr_count($value, self::FORMULA_CLOSE_FUNCTION_BRACE);
                        $value = \str_replace($matrixReplaceFrom, $matrixReplaceTo, $value);
                    }
                }
                unset($value);
                //    Then rebuild the formula string
                $formula = \implode(self::FORMULA_STRING_QUOTE, $temp);
            } else {
                //    If there's no quoted strings, then we do a simple count/replace
                $openCount = \substr_count($formula, self::FORMULA_OPEN_FUNCTION_BRACE);
                $closeCount = \substr_count($formula, self::FORMULA_CLOSE_FUNCTION_BRACE);
                $formula = \str_replace($matrixReplaceFrom, $matrixReplaceTo, $formula);
            }
            //    Trap for mismatched braces and trigger an appropriate error
            if ($openCount < $closeCount) {
                if ($openCount > 0) {
                    return $this->raiseFormulaError("Formula Error: Mismatched matrix braces '}'");
                }
                return $this->raiseFormulaError("Formula Error: Unexpected '}' encountered");
            } elseif ($openCount > $closeCount) {
                if ($closeCount > 0) {
                    return $this->raiseFormulaError("Formula Error: Mismatched matrix braces '{'");
                }
                return $this->raiseFormulaError("Formula Error: Unexpected '{' encountered");
            }
        }
        return $formula;
    }
    //    Binary Operators
    //    These operators always work on two values
    //    Array key is the operator, the value indicates whether this is a left or right associative operator
    private static $operatorAssociativity = [
        '^' => 0,
        //    Exponentiation
        '*' => 0,
        '/' => 0,
        //    Multiplication and Division
        '+' => 0,
        '-' => 0,
        //    Addition and Subtraction
        '&' => 0,
        //    Concatenation
        '|' => 0,
        ':' => 0,
        //    Intersect and Range
        '>' => 0,
        '<' => 0,
        '=' => 0,
        '>=' => 0,
        '<=' => 0,
        '<>' => 0,
    ];
    //    Comparison (Boolean) Operators
    //    These operators work on two values, but always return a boolean result
    private static $comparisonOperators = ['>' => \true, '<' => \true, '=' => \true, '>=' => \true, '<=' => \true, '<>' => \true];
    //    Operator Precedence
    //    This list includes all valid operators, whether binary (including boolean) or unary (such as %)
    //    Array key is the operator, the value is its precedence
    private static $operatorPrecedence = [
        ':' => 8,
        //    Range
        '|' => 7,
        //    Intersect
        '~' => 6,
        //    Negation
        '%' => 5,
        //    Percentage
        '^' => 4,
        //    Exponentiation
        '*' => 3,
        '/' => 3,
        //    Multiplication and Division
        '+' => 2,
        '-' => 2,
        //    Addition and Subtraction
        '&' => 1,
        //    Concatenation
        '>' => 0,
        '<' => 0,
        '=' => 0,
        '>=' => 0,
        '<=' => 0,
        '<>' => 0,
    ];
    // Convert infix to postfix notation
    /**
     * @param string $formula
     *
     * @return array<int, mixed>|false
     */
    private function internalParseFormula($formula, ?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell $pCell = null)
    {
        if (($formula = $this->convertMatrixReferences(\trim($formula))) === \false) {
            return \false;
        }
        //    If we're using cell caching, then $pCell may well be flushed back to the cache (which detaches the parent worksheet),
        //        so we store the parent worksheet so that we can re-attach it when necessary
        $pCellParent = $pCell !== null ? $pCell->getWorksheet() : null;
        $regexpMatchString = '/^(' . self::CALCULATION_REGEXP_FUNCTION . '|' . self::CALCULATION_REGEXP_CELLREF . '|' . self::CALCULATION_REGEXP_COLUMN_RANGE . '|' . self::CALCULATION_REGEXP_ROW_RANGE . '|' . self::CALCULATION_REGEXP_NUMBER . '|' . self::CALCULATION_REGEXP_STRING . '|' . self::CALCULATION_REGEXP_OPENBRACE . '|' . self::CALCULATION_REGEXP_DEFINEDNAME . '|' . self::CALCULATION_REGEXP_ERROR . ')/sui';
        //    Start with initialisation
        $index = 0;
        $stack = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Token\Stack();
        $output = [];
        $expectingOperator = \false;
        //    We use this test in syntax-checking the expression to determine when a
        //        - is a negation or + is a positive operator rather than an operation
        $expectingOperand = \false;
        //    We use this test in syntax-checking the expression to determine whether an operand
        //        should be null in a function call
        // IF branch pruning
        // currently pending storeKey (last item of the storeKeysStack
        $pendingStoreKey = null;
        // stores a list of storeKeys (string[])
        $pendingStoreKeysStack = [];
        $expectingConditionMap = [];
        // ['storeKey' => true, ...]
        $expectingThenMap = [];
        // ['storeKey' => true, ...]
        $expectingElseMap = [];
        // ['storeKey' => true, ...]
        $parenthesisDepthMap = [];
        // ['storeKey' => 4, ...]
        //    The guts of the lexical parser
        //    Loop through the formula extracting each operator and operand in turn
        while (\true) {
            // Branch pruning: we adapt the output item to the context (it will
            // be used to limit its computation)
            $currentCondition = null;
            $currentOnlyIf = null;
            $currentOnlyIfNot = null;
            $previousStoreKey = null;
            $pendingStoreKey = \end($pendingStoreKeysStack);
            if ($this->branchPruningEnabled) {
                // this is a condition ?
                if (isset($expectingConditionMap[$pendingStoreKey]) && $expectingConditionMap[$pendingStoreKey]) {
                    $currentCondition = $pendingStoreKey;
                    $stackDepth = \count($pendingStoreKeysStack);
                    if ($stackDepth > 1) {
                        // nested if
                        $previousStoreKey = $pendingStoreKeysStack[$stackDepth - 2];
                    }
                }
                if (isset($expectingThenMap[$pendingStoreKey]) && $expectingThenMap[$pendingStoreKey]) {
                    $currentOnlyIf = $pendingStoreKey;
                } elseif (isset($previousStoreKey)) {
                    if (isset($expectingThenMap[$previousStoreKey]) && $expectingThenMap[$previousStoreKey]) {
                        $currentOnlyIf = $previousStoreKey;
                    }
                }
                if (isset($expectingElseMap[$pendingStoreKey]) && $expectingElseMap[$pendingStoreKey]) {
                    $currentOnlyIfNot = $pendingStoreKey;
                } elseif (isset($previousStoreKey)) {
                    if (isset($expectingElseMap[$previousStoreKey]) && $expectingElseMap[$previousStoreKey]) {
                        $currentOnlyIfNot = $previousStoreKey;
                    }
                }
            }
            $opCharacter = $formula[$index];
            //    Get the first character of the value at the current index position
            if (isset(self::$comparisonOperators[$opCharacter]) && \strlen($formula) > $index && isset(self::$comparisonOperators[$formula[$index + 1]])) {
                $opCharacter .= $formula[++$index];
            }
            //    Find out if we're currently at the beginning of a number, variable, cell reference, function, parenthesis or operand
            $isOperandOrFunction = (bool) \preg_match($regexpMatchString, \substr($formula, $index), $match);
            if ($opCharacter == '-' && !$expectingOperator) {
                //    Is it a negation instead of a minus?
                //    Put a negation on the stack
                $stack->push('Unary Operator', '~', null, $currentCondition, $currentOnlyIf, $currentOnlyIfNot);
                ++$index;
                //        and drop the negation symbol
            } elseif ($opCharacter == '%' && $expectingOperator) {
                //    Put a percentage on the stack
                $stack->push('Unary Operator', '%', null, $currentCondition, $currentOnlyIf, $currentOnlyIfNot);
                ++$index;
            } elseif ($opCharacter == '+' && !$expectingOperator) {
                //    Positive (unary plus rather than binary operator plus) can be discarded?
                ++$index;
                //    Drop the redundant plus symbol
            } elseif (($opCharacter == '~' || $opCharacter == '|') && !$isOperandOrFunction) {
                //    We have to explicitly deny a tilde or pipe, because they are legal
                return $this->raiseFormulaError("Formula Error: Illegal character '~'");
                //        on the stack but not in the input expression
            } elseif ((isset(self::$operators[$opCharacter]) || $isOperandOrFunction) && $expectingOperator) {
                //    Are we putting an operator on the stack?
                while ($stack->count() > 0 && ($o2 = $stack->last()) && isset(self::$operators[$o2['value']]) && @(self::$operatorAssociativity[$opCharacter] ? self::$operatorPrecedence[$opCharacter] < self::$operatorPrecedence[$o2['value']] : self::$operatorPrecedence[$opCharacter] <= self::$operatorPrecedence[$o2['value']])) {
                    $output[] = $stack->pop();
                    //    Swap operands and higher precedence operators from the stack to the output
                }
                //    Finally put our current operator onto the stack
                $stack->push('Binary Operator', $opCharacter, null, $currentCondition, $currentOnlyIf, $currentOnlyIfNot);
                ++$index;
                $expectingOperator = \false;
            } elseif ($opCharacter == ')' && $expectingOperator) {
                //    Are we expecting to close a parenthesis?
                $expectingOperand = \false;
                while (($o2 = $stack->pop()) && $o2['value'] != '(') {
                    //    Pop off the stack back to the last (
                    if ($o2 === null) {
                        return $this->raiseFormulaError('Formula Error: Unexpected closing brace ")"');
                    }
                    $output[] = $o2;
                }
                $d = $stack->last(2);
                // Branch pruning we decrease the depth whether is it a function
                // call or a parenthesis
                if (!empty($pendingStoreKey)) {
                    --$parenthesisDepthMap[$pendingStoreKey];
                }
                if (\is_array($d) && \preg_match('/^' . self::CALCULATION_REGEXP_FUNCTION . '$/miu', $d['value'], $matches)) {
                    //    Did this parenthesis just close a function?
                    if (!empty($pendingStoreKey) && $parenthesisDepthMap[$pendingStoreKey] == -1) {
                        // we are closing an IF(
                        if ($d['value'] != 'IF(') {
                            return $this->raiseFormulaError('Parser bug we should be in an "IF("');
                        }
                        if ($expectingConditionMap[$pendingStoreKey]) {
                            return $this->raiseFormulaError('We should not be expecting a condition');
                        }
                        $expectingThenMap[$pendingStoreKey] = \false;
                        $expectingElseMap[$pendingStoreKey] = \false;
                        --$parenthesisDepthMap[$pendingStoreKey];
                        \array_pop($pendingStoreKeysStack);
                        unset($pendingStoreKey);
                    }
                    $functionName = $matches[1];
                    //    Get the function name
                    $d = $stack->pop();
                    $argumentCount = $d['value'];
                    //    See how many arguments there were (argument count is the next value stored on the stack)
                    $output[] = $d;
                    //    Dump the argument count on the output
                    $output[] = $stack->pop();
                    //    Pop the function and push onto the output
                    if (isset(self::$controlFunctions[$functionName])) {
                        $expectedArgumentCount = self::$controlFunctions[$functionName]['argumentCount'];
                        $functionCall = self::$controlFunctions[$functionName]['functionCall'];
                    } elseif (isset(self::$phpSpreadsheetFunctions[$functionName])) {
                        $expectedArgumentCount = self::$phpSpreadsheetFunctions[$functionName]['argumentCount'];
                        $functionCall = self::$phpSpreadsheetFunctions[$functionName]['functionCall'];
                    } else {
                        // did we somehow push a non-function on the stack? this should never happen
                        return $this->raiseFormulaError('Formula Error: Internal error, non-function on stack');
                    }
                    //    Check the argument count
                    $argumentCountError = \false;
                    $expectedArgumentCountString = null;
                    if (\is_numeric($expectedArgumentCount)) {
                        if ($expectedArgumentCount < 0) {
                            if ($argumentCount > \abs($expectedArgumentCount)) {
                                $argumentCountError = \true;
                                $expectedArgumentCountString = 'no more than ' . \abs($expectedArgumentCount);
                            }
                        } else {
                            if ($argumentCount != $expectedArgumentCount) {
                                $argumentCountError = \true;
                                $expectedArgumentCountString = $expectedArgumentCount;
                            }
                        }
                    } elseif ($expectedArgumentCount != '*') {
                        $isOperandOrFunction = \preg_match('/(\\d*)([-+,])(\\d*)/', $expectedArgumentCount, $argMatch);
                        switch ($argMatch[2]) {
                            case '+':
                                if ($argumentCount < $argMatch[1]) {
                                    $argumentCountError = \true;
                                    $expectedArgumentCountString = $argMatch[1] . ' or more ';
                                }
                                break;
                            case '-':
                                if ($argumentCount < $argMatch[1] || $argumentCount > $argMatch[3]) {
                                    $argumentCountError = \true;
                                    $expectedArgumentCountString = 'between ' . $argMatch[1] . ' and ' . $argMatch[3];
                                }
                                break;
                            case ',':
                                if ($argumentCount != $argMatch[1] && $argumentCount != $argMatch[3]) {
                                    $argumentCountError = \true;
                                    $expectedArgumentCountString = 'either ' . $argMatch[1] . ' or ' . $argMatch[3];
                                }
                                break;
                        }
                    }
                    if ($argumentCountError) {
                        return $this->raiseFormulaError("Formula Error: Wrong number of arguments for {$functionName}() function: {$argumentCount} given, " . $expectedArgumentCountString . ' expected');
                    }
                }
                ++$index;
            } elseif ($opCharacter == ',') {
                //    Is this the separator for function arguments?
                if (!empty($pendingStoreKey) && $parenthesisDepthMap[$pendingStoreKey] == 0) {
                    // We must go to the IF next argument
                    if ($expectingConditionMap[$pendingStoreKey]) {
                        $expectingConditionMap[$pendingStoreKey] = \false;
                        $expectingThenMap[$pendingStoreKey] = \true;
                    } elseif ($expectingThenMap[$pendingStoreKey]) {
                        $expectingThenMap[$pendingStoreKey] = \false;
                        $expectingElseMap[$pendingStoreKey] = \true;
                    } elseif ($expectingElseMap[$pendingStoreKey]) {
                        return $this->raiseFormulaError('Reaching fourth argument of an IF');
                    }
                }
                while (($o2 = $stack->pop()) && $o2['value'] != '(') {
                    //    Pop off the stack back to the last (
                    if ($o2 === null) {
                        return $this->raiseFormulaError('Formula Error: Unexpected ,');
                    }
                    $output[] = $o2;
                    // pop the argument expression stuff and push onto the output
                }
                //    If we've a comma when we're expecting an operand, then what we actually have is a null operand;
                //        so push a null onto the stack
                if ($expectingOperand || !$expectingOperator) {
                    $output[] = ['type' => 'Empty Argument', 'value' => self::$excelConstants['NULL'], 'reference' => null];
                }
                // make sure there was a function
                $d = $stack->last(2);
                if (!\preg_match('/^' . self::CALCULATION_REGEXP_FUNCTION . '$/miu', $d['value'], $matches)) {
                    return $this->raiseFormulaError('Formula Error: Unexpected ,');
                }
                $d = $stack->pop();
                $itemStoreKey = $d['storeKey'] ?? null;
                $itemOnlyIf = $d['onlyIf'] ?? null;
                $itemOnlyIfNot = $d['onlyIfNot'] ?? null;
                $stack->push($d['type'], ++$d['value'], $d['reference'], $itemStoreKey, $itemOnlyIf, $itemOnlyIfNot);
                // increment the argument count
                $stack->push('Brace', '(', null, $itemStoreKey, $itemOnlyIf, $itemOnlyIfNot);
                // put the ( back on, we'll need to pop back to it again
                $expectingOperator = \false;
                $expectingOperand = \true;
                ++$index;
            } elseif ($opCharacter == '(' && !$expectingOperator) {
                if (!empty($pendingStoreKey)) {
                    // Branch pruning: we go deeper
                    ++$parenthesisDepthMap[$pendingStoreKey];
                }
                $stack->push('Brace', '(', null, $currentCondition, $currentOnlyIf, $currentOnlyIf);
                ++$index;
            } elseif ($isOperandOrFunction && !$expectingOperator) {
                // do we now have a function/variable/number?
                $expectingOperator = \true;
                $expectingOperand = \false;
                $val = $match[1];
                $length = \strlen($val);
                if (\preg_match('/^' . self::CALCULATION_REGEXP_FUNCTION . '$/miu', $val, $matches)) {
                    $val = \preg_replace('/\\s/u', '', $val);
                    if (isset(self::$phpSpreadsheetFunctions[\strtoupper($matches[1])]) || isset(self::$controlFunctions[\strtoupper($matches[1])])) {
                        // it's a function
                        $valToUpper = \strtoupper($val);
                    } else {
                        $valToUpper = 'NAME.ERROR(';
                    }
                    // here $matches[1] will contain values like "IF"
                    // and $val "IF("
                    if ($this->branchPruningEnabled && $valToUpper == 'IF(') {
                        // we handle a new if
                        $pendingStoreKey = $this->getUnusedBranchStoreKey();
                        $pendingStoreKeysStack[] = $pendingStoreKey;
                        $expectingConditionMap[$pendingStoreKey] = \true;
                        $parenthesisDepthMap[$pendingStoreKey] = 0;
                    } else {
                        // this is not an if but we go deeper
                        if (!empty($pendingStoreKey) && \array_key_exists($pendingStoreKey, $parenthesisDepthMap)) {
                            ++$parenthesisDepthMap[$pendingStoreKey];
                        }
                    }
                    $stack->push('Function', $valToUpper, null, $currentCondition, $currentOnlyIf, $currentOnlyIfNot);
                    // tests if the function is closed right after opening
                    $ax = \preg_match('/^\\s*\\)/u', \substr($formula, $index + $length));
                    if ($ax) {
                        $stack->push('Operand Count for Function ' . $valToUpper . ')', 0, null, $currentCondition, $currentOnlyIf, $currentOnlyIfNot);
                        $expectingOperator = \true;
                    } else {
                        $stack->push('Operand Count for Function ' . $valToUpper . ')', 1, null, $currentCondition, $currentOnlyIf, $currentOnlyIfNot);
                        $expectingOperator = \false;
                    }
                    $stack->push('Brace', '(');
                } elseif (\preg_match('/^' . self::CALCULATION_REGEXP_CELLREF . '$/i', $val, $matches)) {
                    //    Watch for this case-change when modifying to allow cell references in different worksheets...
                    //    Should only be applied to the actual cell column, not the worksheet name
                    //    If the last entry on the stack was a : operator, then we have a cell range reference
                    $testPrevOp = $stack->last(1);
                    if ($testPrevOp !== null && $testPrevOp['value'] === ':') {
                        //    If we have a worksheet reference, then we're playing with a 3D reference
                        if ($matches[2] == '') {
                            //    Otherwise, we 'inherit' the worksheet reference from the start cell reference
                            //    The start of the cell range reference should be the last entry in $output
                            $rangeStartCellRef = $output[\count($output) - 1]['value'];
                            \preg_match('/^' . self::CALCULATION_REGEXP_CELLREF . '$/i', $rangeStartCellRef, $rangeStartMatches);
                            if ($rangeStartMatches[2] > '') {
                                $val = $rangeStartMatches[2] . '!' . $val;
                            }
                        } else {
                            $rangeStartCellRef = $output[\count($output) - 1]['value'];
                            \preg_match('/^' . self::CALCULATION_REGEXP_CELLREF . '$/i', $rangeStartCellRef, $rangeStartMatches);
                            if ($rangeStartMatches[2] !== $matches[2]) {
                                return $this->raiseFormulaError('3D Range references are not yet supported');
                            }
                        }
                    } elseif (\strpos($val, '!') === \false && $pCellParent !== null) {
                        $worksheet = $pCellParent->getTitle();
                        $val = "'{$worksheet}'!{$val}";
                    }
                    $outputItem = $stack->getStackItem('Cell Reference', $val, $val, $currentCondition, $currentOnlyIf, $currentOnlyIfNot);
                    $output[] = $outputItem;
                } else {
                    // it's a variable, constant, string, number or boolean
                    $localeConstant = \false;
                    $stackItemType = 'Value';
                    $stackItemReference = null;
                    //    If the last entry on the stack was a : operator, then we may have a row or column range reference
                    $testPrevOp = $stack->last(1);
                    if ($testPrevOp !== null && $testPrevOp['value'] === ':') {
                        $stackItemType = 'Cell Reference';
                        $startRowColRef = $output[\count($output) - 1]['value'];
                        [$rangeWS1, $startRowColRef] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet::extractSheetTitle($startRowColRef, \true);
                        $rangeSheetRef = $rangeWS1;
                        if ($rangeWS1 !== '') {
                            $rangeWS1 .= '!';
                        }
                        $rangeSheetRef = \trim($rangeSheetRef, "'");
                        [$rangeWS2, $val] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet::extractSheetTitle($val, \true);
                        if ($rangeWS2 !== '') {
                            $rangeWS2 .= '!';
                        } else {
                            $rangeWS2 = $rangeWS1;
                        }
                        $refSheet = $pCellParent;
                        if ($pCellParent !== null && $rangeSheetRef !== '' && $rangeSheetRef !== $pCellParent->getTitle()) {
                            $refSheet = $pCellParent->getParent()->getSheetByName($rangeSheetRef);
                        }
                        if (\ctype_digit($val) && $val <= 1048576) {
                            //    Row range
                            $stackItemType = 'Row Reference';
                            $endRowColRef = $refSheet !== null ? $refSheet->getHighestDataColumn($val) : 'XFD';
                            //    Max 16,384 columns for Excel2007
                            $val = "{$rangeWS2}{$endRowColRef}{$val}";
                        } elseif (\ctype_alpha($val) && \strlen($val) <= 3) {
                            //    Column range
                            $stackItemType = 'Column Reference';
                            $endRowColRef = $refSheet !== null ? $refSheet->getHighestDataRow($val) : 1048576;
                            //    Max 1,048,576 rows for Excel2007
                            $val = "{$rangeWS2}{$val}{$endRowColRef}";
                        }
                        $stackItemReference = $val;
                    } elseif ($opCharacter == self::FORMULA_STRING_QUOTE) {
                        //    UnEscape any quotes within the string
                        $val = self::wrapResult(\str_replace('""', self::FORMULA_STRING_QUOTE, self::unwrapResult($val)));
                    } elseif (isset(self::$excelConstants[\trim(\strtoupper($val))])) {
                        $stackItemType = 'Constant';
                        $excelConstant = \trim(\strtoupper($val));
                        $val = self::$excelConstants[$excelConstant];
                    } elseif (($localeConstant = \array_search(\trim(\strtoupper($val)), self::$localeBoolean)) !== \false) {
                        $stackItemType = 'Constant';
                        $val = self::$excelConstants[$localeConstant];
                    } elseif (\preg_match('/^' . self::CALCULATION_REGEXP_ROW_RANGE . '/miu', \substr($formula, $index), $rowRangeReference)) {
                        $val = $rowRangeReference[1];
                        $length = \strlen($rowRangeReference[1]);
                        $stackItemType = 'Row Reference';
                        $column = 'A';
                        if ($testPrevOp !== null && $testPrevOp['value'] === ':' && $pCellParent !== null) {
                            $column = $pCellParent->getHighestDataColumn($val);
                        }
                        $val = "{$rowRangeReference[2]}{$column}{$rowRangeReference[7]}";
                        $stackItemReference = $val;
                    } elseif (\preg_match('/^' . self::CALCULATION_REGEXP_COLUMN_RANGE . '/miu', \substr($formula, $index), $columnRangeReference)) {
                        $val = $columnRangeReference[1];
                        $length = \strlen($val);
                        $stackItemType = 'Column Reference';
                        $row = '1';
                        if ($testPrevOp !== null && $testPrevOp['value'] === ':' && $pCellParent !== null) {
                            $row = $pCellParent->getHighestDataRow($val);
                        }
                        $val = "{$val}{$row}";
                        $stackItemReference = $val;
                    } elseif (\preg_match('/^' . self::CALCULATION_REGEXP_DEFINEDNAME . '.*/miu', $val, $match)) {
                        $stackItemType = 'Defined Name';
                        $stackItemReference = $val;
                    } elseif (\is_numeric($val)) {
                        if (\strpos($val, '.') !== \false || \stripos($val, 'e') !== \false || $val > \PHP_INT_MAX || $val < -\PHP_INT_MAX) {
                            $val = (float) $val;
                        } else {
                            $val = (int) $val;
                        }
                    }
                    $details = $stack->getStackItem($stackItemType, $val, $stackItemReference, $currentCondition, $currentOnlyIf, $currentOnlyIfNot);
                    if ($localeConstant) {
                        $details['localeValue'] = $localeConstant;
                    }
                    $output[] = $details;
                }
                $index += $length;
            } elseif ($opCharacter == '$') {
                // absolute row or column range
                ++$index;
            } elseif ($opCharacter == ')') {
                // miscellaneous error checking
                if ($expectingOperand) {
                    $output[] = ['type' => 'Empty Argument', 'value' => self::$excelConstants['NULL'], 'reference' => null];
                    $expectingOperand = \false;
                    $expectingOperator = \true;
                } else {
                    return $this->raiseFormulaError("Formula Error: Unexpected ')'");
                }
            } elseif (isset(self::$operators[$opCharacter]) && !$expectingOperator) {
                return $this->raiseFormulaError("Formula Error: Unexpected operator '{$opCharacter}'");
            } else {
                // I don't even want to know what you did to get here
                return $this->raiseFormulaError('Formula Error: An unexpected error occurred');
            }
            //    Test for end of formula string
            if ($index == \strlen($formula)) {
                //    Did we end with an operator?.
                //    Only valid for the % unary operator
                if (isset(self::$operators[$opCharacter]) && $opCharacter != '%') {
                    return $this->raiseFormulaError("Formula Error: Operator '{$opCharacter}' has no operands");
                }
                break;
            }
            //    Ignore white space
            while ($formula[$index] == "\n" || $formula[$index] == "\r") {
                ++$index;
            }
            if ($formula[$index] == ' ') {
                while ($formula[$index] == ' ') {
                    ++$index;
                }
                //    If we're expecting an operator, but only have a space between the previous and next operands (and both are
                //        Cell References) then we have an INTERSECTION operator
                if ($expectingOperator && (\preg_match('/^' . self::CALCULATION_REGEXP_CELLREF . '.*/Ui', \substr($formula, $index), $match) && $output[\count($output) - 1]['type'] == 'Cell Reference' || \preg_match('/^' . self::CALCULATION_REGEXP_DEFINEDNAME . '.*/miu', \substr($formula, $index), $match) && ($output[\count($output) - 1]['type'] == 'Defined Name' || $output[\count($output) - 1]['type'] == 'Value'))) {
                    while ($stack->count() > 0 && ($o2 = $stack->last()) && isset(self::$operators[$o2['value']]) && @(self::$operatorAssociativity[$opCharacter] ? self::$operatorPrecedence[$opCharacter] < self::$operatorPrecedence[$o2['value']] : self::$operatorPrecedence[$opCharacter] <= self::$operatorPrecedence[$o2['value']])) {
                        $output[] = $stack->pop();
                        //    Swap operands and higher precedence operators from the stack to the output
                    }
                    $stack->push('Binary Operator', '|');
                    //    Put an Intersect Operator on the stack
                    $expectingOperator = \false;
                }
            }
        }
        while (($op = $stack->pop()) !== null) {
            // pop everything off the stack and push onto output
            if (\is_array($op) && $op['value'] == '(' || $op === '(') {
                return $this->raiseFormulaError("Formula Error: Expecting ')'");
                // if there are any opening braces on the stack, then braces were unbalanced
            }
            $output[] = $op;
        }
        return $output;
    }
    private static function dataTestReference(&$operandData)
    {
        $operand = $operandData['value'];
        if ($operandData['reference'] === null && \is_array($operand)) {
            $rKeys = \array_keys($operand);
            $rowKey = \array_shift($rKeys);
            $cKeys = \array_keys(\array_keys($operand[$rowKey]));
            $colKey = \array_shift($cKeys);
            if (\ctype_upper("{$colKey}")) {
                $operandData['reference'] = $colKey . $rowKey;
            }
        }
        return $operand;
    }
    // evaluate postfix notation
    /**
     * @param mixed $tokens
     * @param null|string $cellID
     *
     * @return array<int, mixed>|false
     */
    private function processTokenStack($tokens, $cellID = null, ?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell $pCell = null)
    {
        if ($tokens == \false) {
            return \false;
        }
        //    If we're using cell caching, then $pCell may well be flushed back to the cache (which detaches the parent cell collection),
        //        so we store the parent cell collection so that we can re-attach it when necessary
        $pCellWorksheet = $pCell !== null ? $pCell->getWorksheet() : null;
        $pCellParent = $pCell !== null ? $pCell->getParent() : null;
        $stack = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Token\Stack();
        // Stores branches that have been pruned
        $fakedForBranchPruning = [];
        // help us to know when pruning ['branchTestId' => true/false]
        $branchStore = [];
        //    Loop through each token in turn
        foreach ($tokens as $tokenData) {
            $token = $tokenData['value'];
            // Branch pruning: skip useless resolutions
            $storeKey = $tokenData['storeKey'] ?? null;
            if ($this->branchPruningEnabled && isset($tokenData['onlyIf'])) {
                $onlyIfStoreKey = $tokenData['onlyIf'];
                $storeValue = $branchStore[$onlyIfStoreKey] ?? null;
                $storeValueAsBool = $storeValue === null ? \true : (bool) \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($storeValue);
                if (\is_array($storeValue)) {
                    $wrappedItem = \end($storeValue);
                    $storeValue = \end($wrappedItem);
                }
                if (isset($storeValue) && (!$storeValueAsBool || \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::isError($storeValue) || $storeValue === 'Pruned branch')) {
                    // If branching value is not true, we don't need to compute
                    if (!isset($fakedForBranchPruning['onlyIf-' . $onlyIfStoreKey])) {
                        $stack->push('Value', 'Pruned branch (only if ' . $onlyIfStoreKey . ') ' . $token);
                        $fakedForBranchPruning['onlyIf-' . $onlyIfStoreKey] = \true;
                    }
                    if (isset($storeKey)) {
                        // We are processing an if condition
                        // We cascade the pruning to the depending branches
                        $branchStore[$storeKey] = 'Pruned branch';
                        $fakedForBranchPruning['onlyIfNot-' . $storeKey] = \true;
                        $fakedForBranchPruning['onlyIf-' . $storeKey] = \true;
                    }
                    continue;
                }
            }
            if ($this->branchPruningEnabled && isset($tokenData['onlyIfNot'])) {
                $onlyIfNotStoreKey = $tokenData['onlyIfNot'];
                $storeValue = $branchStore[$onlyIfNotStoreKey] ?? null;
                $storeValueAsBool = $storeValue === null ? \true : (bool) \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($storeValue);
                if (\is_array($storeValue)) {
                    $wrappedItem = \end($storeValue);
                    $storeValue = \end($wrappedItem);
                }
                if (isset($storeValue) && ($storeValueAsBool || \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::isError($storeValue) || $storeValue === 'Pruned branch')) {
                    // If branching value is true, we don't need to compute
                    if (!isset($fakedForBranchPruning['onlyIfNot-' . $onlyIfNotStoreKey])) {
                        $stack->push('Value', 'Pruned branch (only if not ' . $onlyIfNotStoreKey . ') ' . $token);
                        $fakedForBranchPruning['onlyIfNot-' . $onlyIfNotStoreKey] = \true;
                    }
                    if (isset($storeKey)) {
                        // We are processing an if condition
                        // We cascade the pruning to the depending branches
                        $branchStore[$storeKey] = 'Pruned branch';
                        $fakedForBranchPruning['onlyIfNot-' . $storeKey] = \true;
                        $fakedForBranchPruning['onlyIf-' . $storeKey] = \true;
                    }
                    continue;
                }
            }
            // if the token is a binary operator, pop the top two values off the stack, do the operation, and push the result back on the stack
            if (!\is_numeric($token) && isset(self::$binaryOperators[$token])) {
                //    We must have two operands, error if we don't
                if (($operand2Data = $stack->pop()) === null) {
                    return $this->raiseFormulaError('Internal error - Operand value missing from stack');
                }
                if (($operand1Data = $stack->pop()) === null) {
                    return $this->raiseFormulaError('Internal error - Operand value missing from stack');
                }
                $operand1 = self::dataTestReference($operand1Data);
                $operand2 = self::dataTestReference($operand2Data);
                //    Log what we're doing
                if ($token == ':') {
                    $this->debugLog->writeDebugLog('Evaluating Range ', $this->showValue($operand1Data['reference']), ' ', $token, ' ', $this->showValue($operand2Data['reference']));
                } else {
                    $this->debugLog->writeDebugLog('Evaluating ', $this->showValue($operand1), ' ', $token, ' ', $this->showValue($operand2));
                }
                //    Process the operation in the appropriate manner
                switch ($token) {
                    //    Comparison (Boolean) Operators
                    case '>':
                    //    Greater than
                    case '<':
                    //    Less than
                    case '>=':
                    //    Greater than or Equal to
                    case '<=':
                    //    Less than or Equal to
                    case '=':
                    //    Equality
                    case '<>':
                        //    Inequality
                        $result = $this->executeBinaryComparisonOperation($cellID, $operand1, $operand2, $token, $stack);
                        if (isset($storeKey)) {
                            $branchStore[$storeKey] = $result;
                        }
                        break;
                    //    Binary Operators
                    case ':':
                        //    Range
                        if (\strpos($operand1Data['reference'], '!') !== \false) {
                            [$sheet1, $operand1Data['reference']] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet::extractSheetTitle($operand1Data['reference'], \true);
                        } else {
                            $sheet1 = $pCellParent !== null ? $pCellWorksheet->getTitle() : '';
                        }
                        [$sheet2, $operand2Data['reference']] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet::extractSheetTitle($operand2Data['reference'], \true);
                        if (empty($sheet2)) {
                            $sheet2 = $sheet1;
                        }
                        if (\trim($sheet1, "'") === \trim($sheet2, "'")) {
                            if ($operand1Data['reference'] === null) {
                                if (\trim($operand1Data['value']) != '' && \is_numeric($operand1Data['value'])) {
                                    $operand1Data['reference'] = $pCell->getColumn() . $operand1Data['value'];
                                } elseif (\trim($operand1Data['reference']) == '') {
                                    $operand1Data['reference'] = $pCell->getCoordinate();
                                } else {
                                    $operand1Data['reference'] = $operand1Data['value'] . $pCell->getRow();
                                }
                            }
                            if ($operand2Data['reference'] === null) {
                                if (\trim($operand2Data['value']) != '' && \is_numeric($operand2Data['value'])) {
                                    $operand2Data['reference'] = $pCell->getColumn() . $operand2Data['value'];
                                } elseif (\trim($operand2Data['reference']) == '') {
                                    $operand2Data['reference'] = $pCell->getCoordinate();
                                } else {
                                    $operand2Data['reference'] = $operand2Data['value'] . $pCell->getRow();
                                }
                            }
                            $oData = \array_merge(\explode(':', $operand1Data['reference']), \explode(':', $operand2Data['reference']));
                            $oCol = $oRow = [];
                            foreach ($oData as $oDatum) {
                                $oCR = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::coordinateFromString($oDatum);
                                $oCol[] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::columnIndexFromString($oCR[0]) - 1;
                                $oRow[] = $oCR[1];
                            }
                            $cellRef = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex(\min($oCol) + 1) . \min($oRow) . ':' . \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex(\max($oCol) + 1) . \max($oRow);
                            if ($pCellParent !== null) {
                                $cellValue = $this->extractCellRange($cellRef, $this->spreadsheet->getSheetByName($sheet1), \false);
                            } else {
                                return $this->raiseFormulaError('Unable to access Cell Reference');
                            }
                            $stack->push('Cell Reference', $cellValue, $cellRef);
                        } else {
                            $stack->push('Error', \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::REF(), null);
                        }
                        break;
                    case '+':
                        //    Addition
                        $result = $this->executeNumericBinaryOperation($operand1, $operand2, $token, 'plusEquals', $stack);
                        if (isset($storeKey)) {
                            $branchStore[$storeKey] = $result;
                        }
                        break;
                    case '-':
                        //    Subtraction
                        $result = $this->executeNumericBinaryOperation($operand1, $operand2, $token, 'minusEquals', $stack);
                        if (isset($storeKey)) {
                            $branchStore[$storeKey] = $result;
                        }
                        break;
                    case '*':
                        //    Multiplication
                        $result = $this->executeNumericBinaryOperation($operand1, $operand2, $token, 'arrayTimesEquals', $stack);
                        if (isset($storeKey)) {
                            $branchStore[$storeKey] = $result;
                        }
                        break;
                    case '/':
                        //    Division
                        $result = $this->executeNumericBinaryOperation($operand1, $operand2, $token, 'arrayRightDivide', $stack);
                        if (isset($storeKey)) {
                            $branchStore[$storeKey] = $result;
                        }
                        break;
                    case '^':
                        //    Exponential
                        $result = $this->executeNumericBinaryOperation($operand1, $operand2, $token, 'power', $stack);
                        if (isset($storeKey)) {
                            $branchStore[$storeKey] = $result;
                        }
                        break;
                    case '&':
                        //    Concatenation
                        //    If either of the operands is a matrix, we need to treat them both as matrices
                        //        (converting the other operand to a matrix if need be); then perform the required
                        //        matrix operation
                        if (\is_bool($operand1)) {
                            $operand1 = $operand1 ? self::$localeBoolean['TRUE'] : self::$localeBoolean['FALSE'];
                        }
                        if (\is_bool($operand2)) {
                            $operand2 = $operand2 ? self::$localeBoolean['TRUE'] : self::$localeBoolean['FALSE'];
                        }
                        if (\is_array($operand1) || \is_array($operand2)) {
                            //    Ensure that both operands are arrays/matrices
                            self::checkMatrixOperands($operand1, $operand2, 2);
                            try {
                                //    Convert operand 1 from a PHP array to a matrix
                                $matrix = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\JAMA\Matrix($operand1);
                                //    Perform the required operation against the operand 1 matrix, passing in operand 2
                                $matrixResult = $matrix->concat($operand2);
                                $result = $matrixResult->getArray();
                            } catch (\Exception $ex) {
                                $this->debugLog->writeDebugLog('JAMA Matrix Exception: ', $ex->getMessage());
                                $result = '#VALUE!';
                            }
                        } else {
                            $result = self::FORMULA_STRING_QUOTE . \str_replace('""', self::FORMULA_STRING_QUOTE, self::unwrapResult($operand1) . self::unwrapResult($operand2)) . self::FORMULA_STRING_QUOTE;
                        }
                        $this->debugLog->writeDebugLog('Evaluation Result is ', $this->showTypeDetails($result));
                        $stack->push('Value', $result);
                        if (isset($storeKey)) {
                            $branchStore[$storeKey] = $result;
                        }
                        break;
                    case '|':
                        //    Intersect
                        $rowIntersect = \array_intersect_key($operand1, $operand2);
                        $cellIntersect = $oCol = $oRow = [];
                        foreach (\array_keys($rowIntersect) as $row) {
                            $oRow[] = $row;
                            foreach ($rowIntersect[$row] as $col => $data) {
                                $oCol[] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::columnIndexFromString($col) - 1;
                                $cellIntersect[$row] = \array_intersect_key($operand1[$row], $operand2[$row]);
                            }
                        }
                        if (\count(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArray($cellIntersect)) === 0) {
                            $this->debugLog->writeDebugLog('Evaluation Result is ', $this->showTypeDetails($cellIntersect));
                            $stack->push('Error', \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::null(), null);
                        } else {
                            $cellRef = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex(\min($oCol) + 1) . \min($oRow) . ':' . \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex(\max($oCol) + 1) . \max($oRow);
                            $this->debugLog->writeDebugLog('Evaluation Result is ', $this->showTypeDetails($cellIntersect));
                            $stack->push('Value', $cellIntersect, $cellRef);
                        }
                        break;
                }
                // if the token is a unary operator, pop one value off the stack, do the operation, and push it back on
            } elseif ($token === '~' || $token === '%') {
                if (($arg = $stack->pop()) === null) {
                    return $this->raiseFormulaError('Internal error - Operand value missing from stack');
                }
                $arg = $arg['value'];
                if ($token === '~') {
                    $this->debugLog->writeDebugLog('Evaluating Negation of ', $this->showValue($arg));
                    $multiplier = -1;
                } else {
                    $this->debugLog->writeDebugLog('Evaluating Percentile of ', $this->showValue($arg));
                    $multiplier = 0.01;
                }
                if (\is_array($arg)) {
                    self::checkMatrixOperands($arg, $multiplier, 2);
                    try {
                        $matrix1 = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\JAMA\Matrix($arg);
                        $matrixResult = $matrix1->arrayTimesEquals($multiplier);
                        $result = $matrixResult->getArray();
                    } catch (\Exception $ex) {
                        $this->debugLog->writeDebugLog('JAMA Matrix Exception: ', $ex->getMessage());
                        $result = '#VALUE!';
                    }
                    $this->debugLog->writeDebugLog('Evaluation Result is ', $this->showTypeDetails($result));
                    $stack->push('Value', $result);
                    if (isset($storeKey)) {
                        $branchStore[$storeKey] = $result;
                    }
                } else {
                    $this->executeNumericBinaryOperation($multiplier, $arg, '*', 'arrayTimesEquals', $stack);
                }
            } elseif (\preg_match('/^' . self::CALCULATION_REGEXP_CELLREF . '$/i', $token ?? '', $matches)) {
                $cellRef = null;
                if (isset($matches[8])) {
                    if ($pCell === null) {
                        //                        We can't access the range, so return a REF error
                        $cellValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::REF();
                    } else {
                        $cellRef = $matches[6] . $matches[7] . ':' . $matches[9] . $matches[10];
                        if ($matches[2] > '') {
                            $matches[2] = \trim($matches[2], "\"'");
                            if (\strpos($matches[2], '[') !== \false || \strpos($matches[2], ']') !== \false) {
                                //    It's a Reference to an external spreadsheet (not currently supported)
                                return $this->raiseFormulaError('Unable to access External Workbook');
                            }
                            $matches[2] = \trim($matches[2], "\"'");
                            $this->debugLog->writeDebugLog('Evaluating Cell Range ', $cellRef, ' in worksheet ', $matches[2]);
                            if ($pCellParent !== null) {
                                $cellValue = $this->extractCellRange($cellRef, $this->spreadsheet->getSheetByName($matches[2]), \false);
                            } else {
                                return $this->raiseFormulaError('Unable to access Cell Reference');
                            }
                            $this->debugLog->writeDebugLog('Evaluation Result for cells ', $cellRef, ' in worksheet ', $matches[2], ' is ', $this->showTypeDetails($cellValue));
                        } else {
                            $this->debugLog->writeDebugLog('Evaluating Cell Range ', $cellRef, ' in current worksheet');
                            if ($pCellParent !== null) {
                                $cellValue = $this->extractCellRange($cellRef, $pCellWorksheet, \false);
                            } else {
                                return $this->raiseFormulaError('Unable to access Cell Reference');
                            }
                            $this->debugLog->writeDebugLog('Evaluation Result for cells ', $cellRef, ' is ', $this->showTypeDetails($cellValue));
                        }
                    }
                } else {
                    if ($pCell === null) {
                        // We can't access the cell, so return a REF error
                        $cellValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::REF();
                    } else {
                        $cellRef = $matches[6] . $matches[7];
                        if ($matches[2] > '') {
                            $matches[2] = \trim($matches[2], "\"'");
                            if (\strpos($matches[2], '[') !== \false || \strpos($matches[2], ']') !== \false) {
                                //    It's a Reference to an external spreadsheet (not currently supported)
                                return $this->raiseFormulaError('Unable to access External Workbook');
                            }
                            $this->debugLog->writeDebugLog('Evaluating Cell ', $cellRef, ' in worksheet ', $matches[2]);
                            if ($pCellParent !== null) {
                                $cellSheet = $this->spreadsheet->getSheetByName($matches[2]);
                                if ($cellSheet && $cellSheet->cellExists($cellRef)) {
                                    $cellValue = $this->extractCellRange($cellRef, $this->spreadsheet->getSheetByName($matches[2]), \false);
                                    $pCell->attach($pCellParent);
                                } else {
                                    $cellRef = $cellSheet !== null ? "'{$matches[2]}'!{$cellRef}" : $cellRef;
                                    $cellValue = null;
                                }
                            } else {
                                return $this->raiseFormulaError('Unable to access Cell Reference');
                            }
                            $this->debugLog->writeDebugLog('Evaluation Result for cell ', $cellRef, ' in worksheet ', $matches[2], ' is ', $this->showTypeDetails($cellValue));
                        } else {
                            $this->debugLog->writeDebugLog('Evaluating Cell ', $cellRef, ' in current worksheet');
                            if ($pCellParent->has($cellRef)) {
                                $cellValue = $this->extractCellRange($cellRef, $pCellWorksheet, \false);
                                $pCell->attach($pCellParent);
                            } else {
                                $cellValue = null;
                            }
                            $this->debugLog->writeDebugLog('Evaluation Result for cell ', $cellRef, ' is ', $this->showTypeDetails($cellValue));
                        }
                    }
                }
                $stack->push('Cell Value', $cellValue, $cellRef);
                if (isset($storeKey)) {
                    $branchStore[$storeKey] = $cellValue;
                }
                // if the token is a function, pop arguments off the stack, hand them to the function, and push the result back on
            } elseif (\preg_match('/^' . self::CALCULATION_REGEXP_FUNCTION . '$/miu', $token ?? '', $matches)) {
                if ($pCellParent) {
                    $pCell->attach($pCellParent);
                }
                $functionName = $matches[1];
                $argCount = $stack->pop();
                $argCount = $argCount['value'];
                if ($functionName !== 'MKMATRIX') {
                    $this->debugLog->writeDebugLog('Evaluating Function ', self::localeFunc($functionName), '() with ', $argCount == 0 ? 'no' : $argCount, ' argument', $argCount == 1 ? '' : 's');
                }
                if (isset(self::$phpSpreadsheetFunctions[$functionName]) || isset(self::$controlFunctions[$functionName])) {
                    // function
                    $passByReference = \false;
                    $passCellReference = \false;
                    $functionCall = null;
                    if (isset(self::$phpSpreadsheetFunctions[$functionName])) {
                        $functionCall = self::$phpSpreadsheetFunctions[$functionName]['functionCall'];
                        $passByReference = isset(self::$phpSpreadsheetFunctions[$functionName]['passByReference']);
                        $passCellReference = isset(self::$phpSpreadsheetFunctions[$functionName]['passCellReference']);
                    } elseif (isset(self::$controlFunctions[$functionName])) {
                        $functionCall = self::$controlFunctions[$functionName]['functionCall'];
                        $passByReference = isset(self::$controlFunctions[$functionName]['passByReference']);
                        $passCellReference = isset(self::$controlFunctions[$functionName]['passCellReference']);
                    }
                    // get the arguments for this function
                    $args = $argArrayVals = [];
                    $emptyArguments = [];
                    for ($i = 0; $i < $argCount; ++$i) {
                        $arg = $stack->pop();
                        $a = $argCount - $i - 1;
                        if ($passByReference && isset(self::$phpSpreadsheetFunctions[$functionName]['passByReference'][$a]) && self::$phpSpreadsheetFunctions[$functionName]['passByReference'][$a]) {
                            if ($arg['reference'] === null) {
                                $args[] = $cellID;
                                if ($functionName !== 'MKMATRIX') {
                                    $argArrayVals[] = $this->showValue($cellID);
                                }
                            } else {
                                $args[] = $arg['reference'];
                                if ($functionName !== 'MKMATRIX') {
                                    $argArrayVals[] = $this->showValue($arg['reference']);
                                }
                            }
                        } else {
                            $emptyArguments[] = $arg['type'] === 'Empty Argument';
                            $args[] = self::unwrapResult($arg['value']);
                            if ($functionName !== 'MKMATRIX') {
                                $argArrayVals[] = $this->showValue($arg['value']);
                            }
                        }
                    }
                    //    Reverse the order of the arguments
                    \krsort($args);
                    \krsort($emptyArguments);
                    if ($argCount > 0) {
                        $args = $this->addDefaultArgumentValues($functionCall, $args, $emptyArguments);
                    }
                    if ($passByReference && $argCount == 0) {
                        $args[] = $cellID;
                        $argArrayVals[] = $this->showValue($cellID);
                    }
                    if ($functionName !== 'MKMATRIX') {
                        if ($this->debugLog->getWriteDebugLog()) {
                            \krsort($argArrayVals);
                            $this->debugLog->writeDebugLog('Evaluating ', self::localeFunc($functionName), '( ', \implode(self::$localeArgumentSeparator . ' ', \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenArray($argArrayVals)), ' )');
                        }
                    }
                    //    Process the argument with the appropriate function call
                    $args = $this->addCellReference($args, $passCellReference, $functionCall, $pCell);
                    if (!\is_array($functionCall)) {
                        foreach ($args as &$arg) {
                            $arg = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($arg);
                        }
                        unset($arg);
                    }
                    $result = \call_user_func_array($functionCall, $args);
                    if ($functionName !== 'MKMATRIX') {
                        $this->debugLog->writeDebugLog('Evaluation Result for ', self::localeFunc($functionName), '() function call is ', $this->showTypeDetails($result));
                    }
                    $stack->push('Value', self::wrapResult($result));
                    if (isset($storeKey)) {
                        $branchStore[$storeKey] = $result;
                    }
                }
            } else {
                // if the token is a number, boolean, string or an Excel error, push it onto the stack
                if (isset(self::$excelConstants[\strtoupper($token ?? '')])) {
                    $excelConstant = \strtoupper($token);
                    $stack->push('Constant Value', self::$excelConstants[$excelConstant]);
                    if (isset($storeKey)) {
                        $branchStore[$storeKey] = self::$excelConstants[$excelConstant];
                    }
                    $this->debugLog->writeDebugLog('Evaluating Constant ', $excelConstant, ' as ', $this->showTypeDetails(self::$excelConstants[$excelConstant]));
                } elseif (\is_numeric($token) || $token === null || \is_bool($token) || $token == '' || $token[0] == self::FORMULA_STRING_QUOTE || $token[0] == '#') {
                    $stack->push($tokenData['type'], $token, $tokenData['reference']);
                    if (isset($storeKey)) {
                        $branchStore[$storeKey] = $token;
                    }
                    // if the token is a named range or formula, evaluate it and push the result onto the stack
                } elseif (\preg_match('/^' . self::CALCULATION_REGEXP_DEFINEDNAME . '$/miu', $token, $matches)) {
                    $definedName = $matches[6];
                    if ($pCell === null || $pCellWorksheet === null) {
                        return $this->raiseFormulaError("undefined name '{$token}'");
                    }
                    $this->debugLog->writeDebugLog('Evaluating Defined Name ', $definedName);
                    $namedRange = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\DefinedName::resolveName($definedName, $pCellWorksheet);
                    if ($namedRange === null) {
                        return $this->raiseFormulaError("undefined name '{$definedName}'");
                    }
                    $result = $this->evaluateDefinedName($pCell, $namedRange, $pCellWorksheet, $stack);
                    if (isset($storeKey)) {
                        $branchStore[$storeKey] = $result;
                    }
                } else {
                    return $this->raiseFormulaError("undefined name '{$token}'");
                }
            }
        }
        // when we're out of tokens, the stack should have a single element, the final result
        if ($stack->count() != 1) {
            return $this->raiseFormulaError('internal error');
        }
        $output = $stack->pop();
        $output = $output['value'];
        return $output;
    }
    private function validateBinaryOperand(&$operand, &$stack)
    {
        if (\is_array($operand)) {
            if (\count($operand, \COUNT_RECURSIVE) - \count($operand) == 1) {
                do {
                    $operand = \array_pop($operand);
                } while (\is_array($operand));
            }
        }
        //    Numbers, matrices and booleans can pass straight through, as they're already valid
        if (\is_string($operand)) {
            //    We only need special validations for the operand if it is a string
            //    Start by stripping off the quotation marks we use to identify true excel string values internally
            if ($operand > '' && $operand[0] == self::FORMULA_STRING_QUOTE) {
                $operand = self::unwrapResult($operand);
            }
            //    If the string is a numeric value, we treat it as a numeric, so no further testing
            if (!\is_numeric($operand)) {
                //    If not a numeric, test to see if the value is an Excel error, and so can't be used in normal binary operations
                if ($operand > '' && $operand[0] == '#') {
                    $stack->push('Value', $operand);
                    $this->debugLog->writeDebugLog('Evaluation Result is ', $this->showTypeDetails($operand));
                    return \false;
                } elseif (!\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::convertToNumberIfFraction($operand)) {
                    //    If not a numeric or a fraction, then it's a text string, and so can't be used in mathematical binary operations
                    $stack->push('Error', '#VALUE!');
                    $this->debugLog->writeDebugLog('Evaluation Result is a ', $this->showTypeDetails('#VALUE!'));
                    return \false;
                }
            }
        }
        //    return a true if the value of the operand is one that we can use in normal binary operations
        return \true;
    }
    /**
     * @param null|string $cellID
     * @param mixed $operand1
     * @param mixed $operand2
     * @param string $operation
     *
     * @return array
     */
    private function executeArrayComparison($cellID, $operand1, $operand2, $operation, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Token\Stack &$stack, bool $recursingArrays)
    {
        $result = [];
        if (!\is_array($operand2)) {
            // Operand 1 is an array, Operand 2 is a scalar
            foreach ($operand1 as $x => $operandData) {
                $this->debugLog->writeDebugLog('Evaluating Comparison ', $this->showValue($operandData), ' ', $operation, ' ', $this->showValue($operand2));
                $this->executeBinaryComparisonOperation($cellID, $operandData, $operand2, $operation, $stack);
                $r = $stack->pop();
                $result[$x] = $r['value'];
            }
        } elseif (!\is_array($operand1)) {
            // Operand 1 is a scalar, Operand 2 is an array
            foreach ($operand2 as $x => $operandData) {
                $this->debugLog->writeDebugLog('Evaluating Comparison ', $this->showValue($operand1), ' ', $operation, ' ', $this->showValue($operandData));
                $this->executeBinaryComparisonOperation($cellID, $operand1, $operandData, $operation, $stack);
                $r = $stack->pop();
                $result[$x] = $r['value'];
            }
        } else {
            // Operand 1 and Operand 2 are both arrays
            if (!$recursingArrays) {
                self::checkMatrixOperands($operand1, $operand2, 2);
            }
            foreach ($operand1 as $x => $operandData) {
                $this->debugLog->writeDebugLog('Evaluating Comparison ', $this->showValue($operandData), ' ', $operation, ' ', $this->showValue($operand2[$x]));
                $this->executeBinaryComparisonOperation($cellID, $operandData, $operand2[$x], $operation, $stack, \true);
                $r = $stack->pop();
                $result[$x] = $r['value'];
            }
        }
        //    Log the result details
        $this->debugLog->writeDebugLog('Comparison Evaluation Result is ', $this->showTypeDetails($result));
        //    And push the result onto the stack
        $stack->push('Array', $result);
        return $result;
    }
    /**
     * @param null|string $cellID
     * @param mixed $operand1
     * @param mixed $operand2
     * @param string $operation
     * @param bool $recursingArrays
     *
     * @return mixed
     */
    private function executeBinaryComparisonOperation($cellID, $operand1, $operand2, $operation, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Token\Stack &$stack, $recursingArrays = \false)
    {
        //    If we're dealing with matrix operations, we want a matrix result
        if (\is_array($operand1) || \is_array($operand2)) {
            return $this->executeArrayComparison($cellID, $operand1, $operand2, $operation, $stack, $recursingArrays);
        }
        //    Simple validate the two operands if they are string values
        if (\is_string($operand1) && $operand1 > '' && $operand1[0] == self::FORMULA_STRING_QUOTE) {
            $operand1 = self::unwrapResult($operand1);
        }
        if (\is_string($operand2) && $operand2 > '' && $operand2[0] == self::FORMULA_STRING_QUOTE) {
            $operand2 = self::unwrapResult($operand2);
        }
        // Use case insensitive comparaison if not OpenOffice mode
        if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getCompatibilityMode() != \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::COMPATIBILITY_OPENOFFICE) {
            if (\is_string($operand1)) {
                $operand1 = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::strToUpper($operand1);
            }
            if (\is_string($operand2)) {
                $operand2 = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::strToUpper($operand2);
            }
        }
        $useLowercaseFirstComparison = \is_string($operand1) && \is_string($operand2) && \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getCompatibilityMode() == \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::COMPATIBILITY_OPENOFFICE;
        //    execute the necessary operation
        switch ($operation) {
            //    Greater than
            case '>':
                if ($useLowercaseFirstComparison) {
                    $result = $this->strcmpLowercaseFirst($operand1, $operand2) > 0;
                } else {
                    $result = $operand1 > $operand2;
                }
                break;
            //    Less than
            case '<':
                if ($useLowercaseFirstComparison) {
                    $result = $this->strcmpLowercaseFirst($operand1, $operand2) < 0;
                } else {
                    $result = $operand1 < $operand2;
                }
                break;
            //    Equality
            case '=':
                if (\is_numeric($operand1) && \is_numeric($operand2)) {
                    $result = \abs($operand1 - $operand2) < $this->delta;
                } else {
                    $result = $this->strcmpAllowNull($operand1, $operand2) == 0;
                }
                break;
            //    Greater than or equal
            case '>=':
                if (\is_numeric($operand1) && \is_numeric($operand2)) {
                    $result = \abs($operand1 - $operand2) < $this->delta || $operand1 > $operand2;
                } elseif ($useLowercaseFirstComparison) {
                    $result = $this->strcmpLowercaseFirst($operand1, $operand2) >= 0;
                } else {
                    $result = $this->strcmpAllowNull($operand1, $operand2) >= 0;
                }
                break;
            //    Less than or equal
            case '<=':
                if (\is_numeric($operand1) && \is_numeric($operand2)) {
                    $result = \abs($operand1 - $operand2) < $this->delta || $operand1 < $operand2;
                } elseif ($useLowercaseFirstComparison) {
                    $result = $this->strcmpLowercaseFirst($operand1, $operand2) <= 0;
                } else {
                    $result = $this->strcmpAllowNull($operand1, $operand2) <= 0;
                }
                break;
            //    Inequality
            case '<>':
                if (\is_numeric($operand1) && \is_numeric($operand2)) {
                    $result = \abs($operand1 - $operand2) > 1.0E-14;
                } else {
                    $result = $this->strcmpAllowNull($operand1, $operand2) != 0;
                }
                break;
            default:
                throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception('Unsupported binary comparison operation');
        }
        //    Log the result details
        $this->debugLog->writeDebugLog('Evaluation Result is ', $this->showTypeDetails($result));
        //    And push the result onto the stack
        $stack->push('Value', $result);
        return $result;
    }
    /**
     * Compare two strings in the same way as strcmp() except that lowercase come before uppercase letters.
     *
     * @param null|string $str1 First string value for the comparison
     * @param null|string $str2 Second string value for the comparison
     *
     * @return int
     */
    private function strcmpLowercaseFirst($str1, $str2)
    {
        $inversedStr1 = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::strCaseReverse($str1);
        $inversedStr2 = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::strCaseReverse($str2);
        return \strcmp($inversedStr1 ?? '', $inversedStr2 ?? '');
    }
    /**
     * PHP8.1 deprecates passing null to strcmp.
     *
     * @param null|string $str1 First string value for the comparison
     * @param null|string $str2 Second string value for the comparison
     *
     * @return int
     */
    private function strcmpAllowNull($str1, $str2)
    {
        return \strcmp($str1 ?? '', $str2 ?? '');
    }
    /**
     * @param mixed $operand1
     * @param mixed $operand2
     * @param mixed $operation
     * @param string $matrixFunction
     * @param mixed $stack
     *
     * @return bool|mixed
     */
    private function executeNumericBinaryOperation($operand1, $operand2, $operation, $matrixFunction, &$stack)
    {
        //    Validate the two operands
        if (!$this->validateBinaryOperand($operand1, $stack)) {
            return \false;
        }
        if (!$this->validateBinaryOperand($operand2, $stack)) {
            return \false;
        }
        //    If either of the operands is a matrix, we need to treat them both as matrices
        //        (converting the other operand to a matrix if need be); then perform the required
        //        matrix operation
        if (\is_array($operand1) || \is_array($operand2)) {
            //    Ensure that both operands are arrays/matrices of the same size
            self::checkMatrixOperands($operand1, $operand2, 2);
            try {
                //    Convert operand 1 from a PHP array to a matrix
                $matrix = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\JAMA\Matrix($operand1);
                //    Perform the required operation against the operand 1 matrix, passing in operand 2
                $matrixResult = $matrix->{$matrixFunction}($operand2);
                $result = $matrixResult->getArray();
            } catch (\Exception $ex) {
                $this->debugLog->writeDebugLog('JAMA Matrix Exception: ', $ex->getMessage());
                $result = '#VALUE!';
            }
        } else {
            if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getCompatibilityMode() != \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::COMPATIBILITY_OPENOFFICE && (\is_string($operand1) && !\is_numeric($operand1) && \strlen($operand1) > 0 || \is_string($operand2) && !\is_numeric($operand2) && \strlen($operand2) > 0)) {
                $result = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            } else {
                //    If we're dealing with non-matrix operations, execute the necessary operation
                switch ($operation) {
                    //    Addition
                    case '+':
                        $result = $operand1 + $operand2;
                        break;
                    //    Subtraction
                    case '-':
                        $result = $operand1 - $operand2;
                        break;
                    //    Multiplication
                    case '*':
                        $result = $operand1 * $operand2;
                        break;
                    //    Division
                    case '/':
                        if ($operand2 == 0) {
                            //    Trap for Divide by Zero error
                            $stack->push('Error', '#DIV/0!');
                            $this->debugLog->writeDebugLog('Evaluation Result is ', $this->showTypeDetails('#DIV/0!'));
                            return \false;
                        }
                        $result = $operand1 / $operand2;
                        break;
                    //    Power
                    case '^':
                        $result = $operand1 ** $operand2;
                        break;
                    default:
                        throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception('Unsupported numeric binary operation');
                }
            }
        }
        //    Log the result details
        $this->debugLog->writeDebugLog('Evaluation Result is ', $this->showTypeDetails($result));
        //    And push the result onto the stack
        $stack->push('Value', $result);
        return $result;
    }
    // trigger an error, but nicely, if need be
    protected function raiseFormulaError($errorMessage)
    {
        $this->formulaError = $errorMessage;
        $this->cyclicReferenceStack->clear();
        if (!$this->suppressFormulaErrors) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception($errorMessage);
        }
        \trigger_error($errorMessage, \E_USER_ERROR);
        return \false;
    }
    /**
     * Extract range values.
     *
     * @param string $pRange String based range representation
     * @param Worksheet $worksheet Worksheet
     * @param bool $resetLog Flag indicating whether calculation log should be reset or not
     *
     * @return mixed Array of values in range if range contains more than one element. Otherwise, a single value is returned.
     */
    public function extractCellRange(&$pRange = 'A1', ?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet $worksheet = null, $resetLog = \true)
    {
        // Return value
        $returnValue = [];
        if ($worksheet !== null) {
            $worksheetName = $worksheet->getTitle();
            if (\strpos($pRange, '!') !== \false) {
                [$worksheetName, $pRange] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet::extractSheetTitle($pRange, \true);
                $worksheet = $this->spreadsheet->getSheetByName($worksheetName);
            }
            // Extract range
            $aReferences = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::extractAllCellReferencesInRange($pRange);
            $pRange = "'" . $worksheetName . "'" . '!' . $pRange;
            if (!isset($aReferences[1])) {
                $currentCol = '';
                $currentRow = 0;
                //    Single cell in range
                \sscanf($aReferences[0], '%[A-Z]%d', $currentCol, $currentRow);
                if ($worksheet->cellExists($aReferences[0])) {
                    $returnValue[$currentRow][$currentCol] = $worksheet->getCell($aReferences[0])->getCalculatedValue($resetLog);
                } else {
                    $returnValue[$currentRow][$currentCol] = null;
                }
            } else {
                // Extract cell data for all cells in the range
                foreach ($aReferences as $reference) {
                    $currentCol = '';
                    $currentRow = 0;
                    // Extract range
                    \sscanf($reference, '%[A-Z]%d', $currentCol, $currentRow);
                    if ($worksheet->cellExists($reference)) {
                        $returnValue[$currentRow][$currentCol] = $worksheet->getCell($reference)->getCalculatedValue($resetLog);
                    } else {
                        $returnValue[$currentRow][$currentCol] = null;
                    }
                }
            }
        }
        return $returnValue;
    }
    /**
     * Extract range values.
     *
     * @param string $range String based range representation
     * @param null|Worksheet $worksheet Worksheet
     * @param bool $resetLog Flag indicating whether calculation log should be reset or not
     *
     * @return mixed Array of values in range if range contains more than one element. Otherwise, a single value is returned.
     */
    public function extractNamedRange(string &$range = 'A1', ?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet $worksheet = null, $resetLog = \true)
    {
        // Return value
        $returnValue = [];
        if ($worksheet !== null) {
            if (\strpos($range, '!') !== \false) {
                [$worksheetName, $range] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet::extractSheetTitle($range, \true);
                $worksheet = $this->spreadsheet->getSheetByName($worksheetName);
            }
            // Named range?
            $namedRange = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\DefinedName::resolveName($range, $worksheet);
            if ($namedRange === null) {
                return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::REF();
            }
            $worksheet = $namedRange->getWorksheet();
            $range = $namedRange->getValue();
            $splitRange = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::splitRange($range);
            //    Convert row and column references
            if (\ctype_alpha($splitRange[0][0])) {
                $range = $splitRange[0][0] . '1:' . $splitRange[0][1] . $namedRange->getWorksheet()->getHighestRow();
            } elseif (\ctype_digit($splitRange[0][0])) {
                $range = 'A' . $splitRange[0][0] . ':' . $namedRange->getWorksheet()->getHighestColumn() . $splitRange[0][1];
            }
            // Extract range
            $aReferences = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::extractAllCellReferencesInRange($range);
            if (!isset($aReferences[1])) {
                //    Single cell (or single column or row) in range
                [$currentCol, $currentRow] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::coordinateFromString($aReferences[0]);
                if ($worksheet->cellExists($aReferences[0])) {
                    $returnValue[$currentRow][$currentCol] = $worksheet->getCell($aReferences[0])->getCalculatedValue($resetLog);
                } else {
                    $returnValue[$currentRow][$currentCol] = null;
                }
            } else {
                // Extract cell data for all cells in the range
                foreach ($aReferences as $reference) {
                    // Extract range
                    [$currentCol, $currentRow] = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::coordinateFromString($reference);
                    if ($worksheet->cellExists($reference)) {
                        $returnValue[$currentRow][$currentCol] = $worksheet->getCell($reference)->getCalculatedValue($resetLog);
                    } else {
                        $returnValue[$currentRow][$currentCol] = null;
                    }
                }
            }
        }
        return $returnValue;
    }
    /**
     * Is a specific function implemented?
     *
     * @param string $pFunction Function Name
     *
     * @return bool
     */
    public function isImplemented($pFunction)
    {
        $pFunction = \strtoupper($pFunction);
        $notImplemented = !isset(self::$phpSpreadsheetFunctions[$pFunction]) || \is_array(self::$phpSpreadsheetFunctions[$pFunction]['functionCall']) && self::$phpSpreadsheetFunctions[$pFunction]['functionCall'][1] === 'DUMMY';
        return !$notImplemented;
    }
    /**
     * Get a list of all implemented functions as an array of function objects.
     */
    public function getFunctions() : array
    {
        return self::$phpSpreadsheetFunctions;
    }
    /**
     * Get a list of implemented Excel function names.
     *
     * @return array
     */
    public function getImplementedFunctionNames()
    {
        $returnValue = [];
        foreach (self::$phpSpreadsheetFunctions as $functionName => $function) {
            if ($this->isImplemented($functionName)) {
                $returnValue[] = $functionName;
            }
        }
        return $returnValue;
    }
    private function addDefaultArgumentValues(array $functionCall, array $args, array $emptyArguments) : array
    {
        $reflector = new \ReflectionMethod(\implode('::', $functionCall));
        $methodArguments = $reflector->getParameters();
        if (\count($methodArguments) > 0) {
            // Apply any defaults for empty argument values
            foreach ($emptyArguments as $argumentId => $isArgumentEmpty) {
                if ($isArgumentEmpty === \true) {
                    $reflectedArgumentId = \count($args) - (int) $argumentId - 1;
                    if (!\array_key_exists($reflectedArgumentId, $methodArguments) || $methodArguments[$reflectedArgumentId]->isVariadic()) {
                        break;
                    }
                    $args[$argumentId] = $this->getArgumentDefaultValue($methodArguments[$reflectedArgumentId]);
                }
            }
        }
        return $args;
    }
    /**
     * @return null|mixed
     */
    private function getArgumentDefaultValue(\ReflectionParameter $methodArgument)
    {
        $defaultValue = null;
        if ($methodArgument->isDefaultValueAvailable()) {
            $defaultValue = $methodArgument->getDefaultValue();
            if ($methodArgument->isDefaultValueConstant()) {
                $constantName = $methodArgument->getDefaultValueConstantName() ?? '';
                // read constant value
                if (\strpos($constantName, '::') !== \false) {
                    [$className, $constantName] = \explode('::', $constantName);
                    $constantReflector = new \ReflectionClassConstant($className, $constantName);
                    return $constantReflector->getValue();
                }
                return \constant($constantName);
            }
        }
        return $defaultValue;
    }
    /**
     * Add cell reference if needed while making sure that it is the last argument.
     *
     * @param bool $passCellReference
     * @param array|string $functionCall
     *
     * @return array
     */
    private function addCellReference(array $args, $passCellReference, $functionCall, ?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell $pCell = null)
    {
        if ($passCellReference) {
            if (\is_array($functionCall)) {
                $className = $functionCall[0];
                $methodName = $functionCall[1];
                $reflectionMethod = new \ReflectionMethod($className, $methodName);
                $argumentCount = \count($reflectionMethod->getParameters());
                while (\count($args) < $argumentCount - 1) {
                    $args[] = null;
                }
            }
            $args[] = $pCell;
        }
        return $args;
    }
    private function getUnusedBranchStoreKey()
    {
        $storeKeyValue = 'storeKey-' . $this->branchStoreKeyCounter;
        ++$this->branchStoreKeyCounter;
        return $storeKeyValue;
    }
    private function getTokensAsString($tokens)
    {
        $tokensStr = \array_map(function ($token) {
            $value = $token['value'] ?? 'no value';
            while (\is_array($value)) {
                $value = \array_pop($value);
            }
            return $value;
        }, $tokens);
        return '[ ' . \implode(' | ', $tokensStr) . ' ]';
    }
    /**
     * @return mixed|string
     */
    private function evaluateDefinedName(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell $pCell, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\DefinedName $namedRange, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet $pCellWorksheet, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Token\Stack $stack)
    {
        $definedNameScope = $namedRange->getScope();
        if ($definedNameScope !== null && $definedNameScope !== $pCellWorksheet) {
            // The defined name isn't in our current scope, so #REF
            $result = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::REF();
            $stack->push('Error', $result, $namedRange->getName());
            return $result;
        }
        $definedNameValue = $namedRange->getValue();
        $definedNameType = $namedRange->isFormula() ? 'Formula' : 'Range';
        $definedNameWorksheet = $namedRange->getWorksheet();
        if ($definedNameValue[0] !== '=') {
            $definedNameValue = '=' . $definedNameValue;
        }
        $this->debugLog->writeDebugLog("Defined Name is a {$definedNameType} with a value of {$definedNameValue}");
        $recursiveCalculationCell = $definedNameWorksheet !== null && $definedNameWorksheet !== $pCellWorksheet ? $definedNameWorksheet->getCell('A1') : $pCell;
        $recursiveCalculationCellAddress = $recursiveCalculationCell->getCoordinate();
        // Adjust relative references in ranges and formulae so that we execute the calculation for the correct rows and columns
        $definedNameValue = self::$referenceHelper->updateFormulaReferencesAnyWorksheet($definedNameValue, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::columnIndexFromString($pCell->getColumn()) - 1, $pCell->getRow() - 1);
        $this->debugLog->writeDebugLog("Value adjusted for relative references is {$definedNameValue}");
        $recursiveCalculator = new self($this->spreadsheet);
        $recursiveCalculator->getDebugLog()->setWriteDebugLog($this->getDebugLog()->getWriteDebugLog());
        $recursiveCalculator->getDebugLog()->setEchoDebugLog($this->getDebugLog()->getEchoDebugLog());
        $result = $recursiveCalculator->_calculateFormulaValue($definedNameValue, $recursiveCalculationCellAddress, $recursiveCalculationCell);
        if ($this->getDebugLog()->getWriteDebugLog()) {
            $this->debugLog->mergeDebugLog(\array_slice($recursiveCalculator->getDebugLog()->getLog(), 3));
            $this->debugLog->writeDebugLog("Evaluation Result for Named {$definedNameType} {$namedRange->getName()} is {$this->showTypeDetails($result)}");
        }
        $stack->push('Defined Name', $result, $namedRange->getName());
        return $result;
    }
}
